/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2007 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Viktor Pracht <viktor.pracht@open-xchange.com>
 * 
 */

  //////////////////
 //   LiveGrid   //
//////////////////

/**
 * The LiveGrid constructor.
 * @constructor
 * @param {Array} columns An array with column definitions.
 * Each column definition is an object with the following fields:
 * <ul><li>text is used as the column header,</li>
 * <li>i18n specifies whether the text is translated or used directly as
 * innerHTML,</li>
 * <li>index is used to extract the value of this column from an item object.
 * If not specified, the entire object is passed to {@link #set}.</li>
 * <li>width specifies the width of the column as a CSS length value,</li>
 * <li>style specifies an object with style attributes as elements,</li>
 * <li>sortable specifies whether sorting based on this column is supported,</li>
 * <li>id specifies the column ID,</li>
 * <lI>clear specifies a function which is called with an HTML DIV element as
 * parameter whenever the content of that DIV must be fetched from
 * the server. In the case of a static text in every row, {@link #makeClear}
 * can be used to create a function for this field.</li>
 * <li>set specifies a function which is called with an HTML DIV element and
 * the contents of that DIV whenever new contents must be displayed in the DIV.
 * In the case of pure text content, {@link #defaultSet} can be used
 * directly.</li></ul>
 * @param {Selection} selection The selection to use.
 * @param {String} module The name of a module for which to register Drag'n'Drop
 * functions.
 */
function LiveGrid(columns, selection, module) {
	this.disablehover=false;
	this.emptylivegridtext="This folder is empty";/*i18n*/

	/**
	 * @private
	 */
	this.columns = columns;
	
	/**
	 * @private
	 */
	this.clickprocessed=false;

	/**
	 * The selection displayed in this LiveGrid.
	 */
	this.selection = selection || console.error("LiveGrid without Selection");
	
	/**
	 * @private
	 */
	this.module = module;

	/**
	 * Index of the anchor row.
	 * The anchor is used for multiple-selects with the Shift key.
	 * 0&nbsp;&lt;=&nbsp;anchor&nbsp;&lt;&nbsp;rowCount.
	 * @private
	 */
	this.anchor = 0;
	
	/**
	 * Index of the focus row.
	 * The focus is used for all selects.
	 * 0&nbsp;&lt;=&nbsp;focus&nbsp;&lt;&nbsp;rowCount.
	 * @private
	 */
	this.focus = 0;
	
	/**
	 * @private
	 * Sort order. Either "asc" or "desc".
	 */
	this.sort_order = "asc";
	
	/**
	 * @private
	 * ID of the column used for sorting. null if sorting is disabled.
	 */
	this.sort_id = null;
	
	for (var i = 0; i < columns.length; i++)
		if (!this.sort_id && columns[i].sortable) this.sort_id = columns[i].id;

	this.lineHeight = 1.6;
	
	/**
	 * @private
	 */
	this.from = 0;

	/**
	 * @private
	 */
	this.to = 0;
	
	/**
	 * @private
	 */
	this.rowcache = [];
	
	/**
	 * @private
	 * {Int: true}
	 */
	this.unfetched = {};

	var Self = this;

	/**
	 * @private
	 */
	this.fullRedraw = true;
	
	/**
	 * @private
	 */
	this.scroll_cb = function() { Self.scroll(); };
	
	/**
	 * @private
	 */
	this.key_cb = function(e) { return Self.key(e); };

	/**
	 * @private
	 */
	this.keydown_cb = function(e) {
		this.keydown = true;
		return Self.key(e);
	};
	
	/**
	 * @private
	 */
	this.keypress_cb = function(e) {
		if (this.keydown)
			this.keydown = false;
		else
			return Self.key(e);
	};
	
	/**
	 * @private
	 */
	this.mousedown_cb = function(e) { Self.mousedown(e); };
	
	/**
	 * @private
	 */
	this.click_cb = function(e) { Self.click(e); };
	
	this.language_cb = function(e) { Self.scroll(true); }
	/**
	 * @private
	 */
	this.dblclick_cb = function(e) { Self.dblclick(e); };
	
	/**
	 * @private
	 */
	this.focus_cb = function() {
		focusedElement = Self.parent;
		Self.updateStyles();
	};
	
	/**
	 * @private
	 */
	this.blur_cb = function() {
		if (focusedElement == Self.parent)
			focusedElement = null;
		Self.updateStyles();
	};
	
	/**
	 * @private
	 */
	this.change_cb = function(from, to) {
		Self.rowCount = Self.storage.ids.length;
		from = Math.max(from, Self.from);
		to = Math.min(to, Self.to);
		for (var i = from; i < to; i++) Self.unfetched[i] = true;
		Self.doScroll();
		Self.updateStyles();
	};
	
	/**
	 * @private
	 */
	this.selected_cb = function(count) {
		Self.updateStyles();
		Self.events.trigger("Selected", count);
	};
	
	/**
	 * Events triggered by this LiveGrid.
	 * <dl><dt>Selected</dt><dd>The selection has changed. Triggered only when
	 * this grid is enabled. Parameters:
	 * <ul><li>Number of selected elements.</li></ul></dd>
	 * <dt>Activated</dt><dd>One or more elements have been activated.
	 * Parameters:
	 * <ul><li>Array with object IDs of activated objects.</li></ul></dd>
	 * <dt>Delete</dt><dd>One or more elements should be deleted.
	 * Parameters:
	 * <ul><li>Array with object IDs of activated objects.</li></ul></dd>
	 * </dl>
	 */
	this.events = new Events();
}

function getRowIndex(e) {
	var n = e.target || e.srcElement;
	while (n.oxRowIndex == undefined && n.parentNode) n = n.parentNode;
	return n.oxRowIndex;
}

LiveGrid.prototype = {
	/**
	 * How many additional invisible rows to create to allow flicker-free
	 * scrolling at faster speeds.
	 */
	preview: 20,
	
	/**
	 * Granularity of row fetches.
	 */
	granularity: 1,
	
	/**
	 * Creates a DOM tree for the headers of the LiveGrid.
	 * The returned DOM tree must be inserted in its place in the document.
	 * @type Object
	 * @return A DOM tree which displays the headers of the LiveGrid.
	 * @see #getTable
	 */
	getHeader: function() {
		var Self = this;
		this.headers = new Array(this.columns.length);
		this.outer_headers = new Array(this.columns.length);
		this.arrows = new Array(this.columns.length);
		this.arrow_events = new Array(this.columns.length);
		for (var i in this.columns) this.arrow_events[i] = sortHandler(i);
		function sortHandler(i) {
			return function(e) {
				Self.setSort(Self.columns[i].id,
					(Self.sort_id == Self.columns[i].id &&
					 Self.sort_order == "asc") ? "desc" : "asc");
				stopEvent(e);
			}
		}
		this.ids = {};
		for (var i in this.columns) this.ids[this.columns[i].id] = i;
		var colgroup = this.getColGroup();
		colgroup.appendChild(this.ruCorner = newnode("col", {width: "0"}));
		var tbody = newnode("tbody",{verticalAlign : "top"});
		var table = newnode("table", {tableLayout: "fixed"},
		                    {cellPadding: "0", cellSpacing: "0", className: "livegrid"},
		                    [colgroup, tbody]);
		var row = newnode("tr", {height: this.lineHeight + "em"},
		                  {className: "th"});
		for (var i = 0; i < this.columns.length; i++) {
			var outerCell, innerCell;
            var headerClassName = this.columns[i].name
                ? "header livegrid-column-" +
                      this.columns[i].name.join(" livegrid-column-")
                : "header";
			if (this.columns[i].sortable) {
				outerCell = this.outer_headers[i] =
					newnode("th", this.columns[i].style, {className: headerClassName}, [
						newnode("table", 0, {cellPadding: "0", cellSpacing: "0"}, [
							newnode("tbody", 0, 0, [
								newnode("tr", 0, 0, [
									innerCell = this.headers[i] = newnode("th", {whiteSpace: "nowrap"}),
									newnode("th", 0, 0, [
										this.arrows[i] = newnode("img", {verticalAlign: "middle"}, {src: getFullImgSrc("img/dummy.gif"), width: "16", height: "16"})
									])
								])
							])
						])
					]);
			} else
				innerCell = outerCell = this.headers[i] = this.outer_headers[i] =
					newnode("th", this.columns[i].style, {className: headerClassName});
			if (this.columns[i].i18n) {
				var node = new I18nNode(function() {
                    return pgettext(this.context, this.text);
                });
				node.text = this.columns[i].text;
                node.context = this.columns[i].i18n;
                if (node.context === true) node.context = ""; 
				node.parent = innerCell;
				node.update = function() {
					this.parent.title = this.node.data = this.callback();
				};
				innerCell.appendChild(node.node);
				node.update();
			} else switch (typeof this.columns[i].text) {
                case "string":
				    innerCell.innerHTML = this.columns[i].text;
                    break;
                case "function":
                    innerCell.appendChild(addTranslated(this.columns[i].text));
                    break;
                default:
                    innerCell.appendChild(this.columns[i].text);
            }
			row.appendChild(outerCell);
		}
		row.appendChild(newnode("th"));
        var index = this.ids[this.sort_id];
		if (this.sort_id && this.arrows[index]) {
			this.arrows[index].src = 
				this.sort_order == "asc" ? getFullImgSrc("img/arrows/sort_up.gif") :
				this.sort_order == "desc" ? getFullImgSrc("img/arrows/sort_down.gif") :
				getFullImgSrc("img/dummy.gif"); // just in case
            LiveGrid.setSortClass(this.outer_headers[index], true);
        }
		tbody.appendChild(row);
		return table;
	},
	
	/**
	 * Adds the scrollable area of the LiveGrid to the specified parent node.
	 * The parent node should have an explicit size and "overflow: auto".
	 */
	getTable: function(parent) {
		this.rows = {};
		var last = this.columns.length - 1;
		this.parent = parent;
		this.heightDiv = newnode("div", {height: "0em"}, {className: "livegrid"});
		parent.appendChild(this.heightDiv);
		parent.tabIndex = 0;
		parent.style.outline = "none";
		parent.style.overflowX = "hidden";
		parent.hideFocus = true;
		var emptyText = addTranslated(this.emptylivegridtext);
		this.emptyMessage = newnode("div",
			{display: "none", position: "absolute", textAlign: "center", 
			 width: "100%", marginTop: "5px"}, 0,
			[emptyText]);
		parent.appendChild(this.emptyMessage);
		var tr = newnode("tr");
		for (var i = 0; i < this.columns.length; i++)
			tr.appendChild(newnode("td", this.columns[i].style,
			                       {className: "cell"}));
		this.rowTemplate = newnode("table", 0,
			{cellPadding: "0", cellSpacing: "0", className: "tr"},
			[this.getColGroup(), newnode("tbody", 0, 0, [tr])]);
	},
	
	/**
	 * Adds a hover to the LiveGrid.
	 * This method should be called between getTable() and the first enable(). 
	 * @param {Object} hover A DOM node which serves as a hover. It should be
	 * attached directly to the body and have the CSS style
	 * "position: absolute".
	 * @param {Function} callback A callback function which is called when
	 * the hover is shown. As its only parameter, it receives the ID of
	 * the entry over which the mouse is hovering.
	 * @type {Object}
	 * @return A new Hover object. The enable and disable methods of the hover
	 * object will be called automatically by the LiveGrid.
	 */
	addHover: function(hover, callback) {
		this.hover = new Hover(this.heightDiv, hover);
		var Self = this;
		this.hover.onShow = function(node) {
			var i = node.oxRowIndex;
			if (i !== undefined)
				callback(Self.storage.ids[i]);
			else
				return false;
		}
		return this.hover;
	},
	
	/**
	 * @private
	 */
	getColGroup: function() {
		var colgroup = newnode("colgroup");
		for (var i = 0; i < this.columns.length; i++) {
			var w = this.columns[i].width;
			colgroup.appendChild(newnode("col", w ? {width: w} : 0));
		}
		return colgroup;
	},
	
	/**
	 * Appends the contents either of the entire grid or of selected rows to
	 * the specified node.
	 * @param {Object} parent The node to append the data to.
	 * @param {Function} cb The callback to call after all data is appended.
	 * @param {Boolean} selected True if only selected rows should be appended.
	 * Otherwise, all rows are appended.
	 */
	print: function(parent, cb, selected) {
		var doc = parent.ownerDocument;
		var tr = doc.createElement("tr");
		for (var i = 0; i < this.columns.length; i++) {
			var th = doc.createElement("th");
			if (this.columns[i].i18n)
				th.appendChild(doc.createTextNode(_(this.columns[i].text)) || "\u00a0");
			else
				th.innerHTML = this.columns[i].text || "\u00a0";
			tr.appendChild(th);
		}
		var table = newnode("table", {width: "100%"}, {border: 1},
			[newnode("thead", 0, 0, [tr], doc)], doc);
		var trs = [];
		var Self = this;
		function set(n, obj) {
			trs[n] = doc.createElement("tr");
			for (var i = 0; i < Self.columns.length; i++) {
				var td = doc.createElement("td");
				trs[n].appendChild(td);
				var col = Self.columns[i];
				col.set(td, col.index ? obj[col.index] : obj);
			}
		}
		function final_() {
			var tbody = newnode("tbody", 0, 0, 0, doc);
			for (var i = 0; i < trs.length; i++) 
				if (trs[i]) tbody.appendChild(trs[i]);
			table.appendChild(tbody);
			parent.appendChild(table);
			if(trs.length==0) {
				return; 
			} 
			cb();
		}
		this.storage.newIterate(
			selected ? this.selection.getSelected() : this.storage.ids,
			emptyFunction, set, final_);
	},
	
	/**
	 * Displays the specified sort order as an arrow in the headers.
	 * @param {String} id The column ID used for sorting, or null for no
	 * sorting.
	 * @param {String} order The sort order. One of "asc", "desc" or null for no
	 * sorting.
	 */
	setSort: function(id, order) {
		if (this.sort_id && id != this.sort_id) {
            var index = this.ids[this.sort_id];
			this.arrows[index].src =
                getFullImgSrc("img/dummy.gif");
            LiveGrid.setSortClass(this.outer_headers[index], false);
        }
        var index = this.ids[id];
        if (index || index === 0) {
    		this.arrows[index].src = 
    			order == "asc" ? getFullImgSrc("img/arrows/sort_up.gif") :
    			order == "desc" ? getFullImgSrc("img/arrows/sort_down.gif") :
    			getFullImgSrc("img/dummy.gif"); // just in case
            LiveGrid.setSortClass(this.outer_headers[index], true);
        }
		this.sort_id = id;
		this.sort_order = order;
		if (this.storage) this.sort(id, order);
	},
	
	/**
	 * Callback function which changes the displayed data.
	 * Should be overridden in each instance.
	 * @param {String} id The column ID used for sorting, or null for no
	 * sorting.
	 * @param {String} order The sort order. Either "asc" or "desc".
	 */
	sort: emptyFunction,

	getSelectedIDs: function() {
		return this.selection.getSelected();
	},
	
	/**
	 * Deletes items with specified object IDs from the storage and from
	 * the selection. If the entire selection was deleted, the first unselected
	 * item at or after the focus is selected.
	 */
	deleteIDs: function(ids) {
		var sids = {};
		for (var i = 0; i < ids.length; i++)
			sids[this.storage.serialize(ids[i])] = true;
		while (this.storage.getSID(this.focus) in sids) this.focus++;
		var newID = this.storage.ids[this.focus];
		this.storage.removeIDs(ids);
		this.selection.deselectSIDs(sids);
		this.focus = newID ? this.storage.getIndex(newID) : 0;
		if (!this.selection.count)
			this.selection.click(this.focus);
	},
	
	updateSelected: function(callback) {
		if (this.storage)
			this.storage.localUpdate(this.selection.getSelected(), callback);
	},

	/**
	 * Initializes the LiveGrid.
	 * The DOM tree must have been created by {@link #getHeader} and
	 * {@link #getTable}.
	 * @see #getHTML
	 */
	enable: function(storage) {
		if (this.storage) {
			this.storage.events.unregister("Changed", this.change_cb);
		} else {
			for (var i in this.arrows)
				addDOMEvent(this.outer_headers[i], "click", this.arrow_events[i]);
			addDOMEvent(this.parent, "scroll", this.scroll_cb);
			addDOMEvent(this.parent, "keydown", this.keydown_cb);
			addDOMEvent(this.parent, "keypress", this.keypress_cb);
			addDOMEvent(this.parent, "mousedown", this.mousedown_cb);
			addDOMEvent(this.parent, "click", this.click_cb);
			addDOMEvent(this.parent, "dblclick", this.dblclick_cb);
			addDOMEvent(this.parent, "focus", this.focus_cb);
			addDOMEvent(this.parent, "blur", this.blur_cb);
			if (this.hover && !this.disablehover) this.hover.enable();
			resizeEvents.register("Resized", this.scroll_cb);
			register("LanguageChanged",this.language_cb);
			if (this.module && (this.module != "links")) {
				var Self = this;
				var disabled;
				var validfield;
				switch(this.module) {
					case "calendar": disabled=calendardefaultdisabled;	validfield=5; break;
					case "tasks":	disabled=taskdefaultdisabled; validfield=3; break;
					case "contacts": disabled=contactdefaultdisabled; validfield=2; break;
					case "infostore": disabled=infostoredefaultdisabled; validfield=3; break;
					case "mail": disabled=maildefaultdisabled; validfield=5; break;
				}
				this.contextmenu = registerContext(this.heightDiv,this.module);
				this.dragEvent = registerSource(this.heightDiv, this.module, function() {
					var myelements=clone(Self.selection.getSelected());
					if(myelements.length==1) {
						Self.storage.newIterate(myelements,emptyFunction,function (index,id) {
							myelements[0].livegridfield=id[validfield];
						})
					}
					objMoveCopy.begin(Self.module,myelements);
					return objMoveCopy;
					
				},null,null,disabled,defaultdisabledremove);
			}
		}
		storage.events.register("Changed", this.change_cb);
		this.storage = storage;
		this.rowCount = storage.ids.length;
		this.selection.setStorage(storage);
		this.selection.events.register("Selected", this.selected_cb);
		this.scroll(true);
	},
	
	disable: function() {
		if (!this.storage) return;
		this.selection.events.unregister("Selected", this.selected_cb);
		this.storage.events.unregister("Changed", this.change_cb);
		this.storage = null;
		this.selection.setStorage(null);
		for (var i in this.arrows)
			removeDOMEvent(this.outer_headers[i], "click", this.arrow_events[i]);
		removeDOMEvent(this.parent, "scroll", this.scroll_cb);
		removeDOMEvent(this.parent, "keydown", this.keydown_cb);
		removeDOMEvent(this.parent, "keypress", this.keypress_cb);
		removeDOMEvent(this.parent, "mousedown", this.mousedown_cb);
		removeDOMEvent(this.parent, "click", this.click_cb);
		removeDOMEvent(this.parent, "dblclick", this.dblclick_cb);
		removeDOMEvent(this.parent, "focus", this.focus_cb);
		removeDOMEvent(this.parent, "blur", this.blur_cb);
		if (this.hover) this.hover.disable();
		resizeEvents.unregister("Resized", this.scroll_cb);
		unregister("LanguageChanged",this.language_cb);
		if (this.module) {
			unregisterSource(this.parent, this.dragEvent);
			unregisterContext(this.heightDiv, this.contextmenu);
		}
	},
	
	/**
	 * @private
	 */
	createRow: function() {
		var last = this.columns.length - 1;
		var row = {tr: this.rowTemplate.cloneNode(true),
		           cells: new Array(this.columns.length)};
		for (var i = 0, cell = row.tr.lastChild.firstChild.firstChild; cell;
		     i++, cell = cell.nextSibling)
		{
			row.cells[i] = cell;
			this.columns[i].clear(cell);
		}
		this.heightDiv.appendChild(row.tr);
		if (!this.rowHeight) this.rowHeight = row.tr.offsetHeight;
		return row;
	},

	/**
	 * To be removed.
	 * @private
	 */
	updateStyles: function() {
		for (var i in this.rows) {
			var newclass = this.selection.get(i) ? "tr selected" : i%2 ? "tr " : "tr_h ";
			if (focusedElement == this.parent && i == this.focus)
				newclass += "focused";
			var tr = this.rows[i].tr;
			if (tr.className != newclass) tr.className = newclass;
		}
	},

	/**
	 * @private
	 */
	deleteRow: function(index) {
		var row = this.rows[index];
		row.tr.style.display = "none";
		this.rowcache.push(row);
		delete this.rows[index];
		delete this.unfetched[index];
	},

	/**
	 * @private
	 */
	insertRow: function(index) {
		var row = this.rows[index] =
			this.rowcache.length ? this.rowcache.pop() : this.createRow();
		row.tr.className = this.selection.get(index) ? "tr selected" : index%2 ? "tr" : "tr_h";
		row.tr.style.display = "";
		row.tr.style.top = (index * this.rowHeight) + "px";
		row.tr.oxRowIndex = index;
		this.unfetched[index] = true;
	},

	/**
	 * @private
	 */
	scroll: function(fullRedraw) {
		if (fullRedraw) this.fullRedraw = true;
		if (this.scrollTimeout) clearTimeout(this.scrollTimeout);
		var Self = this;
		this.scrollTimeout = setTimeout(function() {
			Self.scrollTimeout = null;
			Self.doScroll();
		});
	},
	
	/**
	 * @private
	 */
	calculateRowHeight: function() {
		var row = this.createRow();
		row.tr.style.display = "none";
		this.rowcache.push();
	},

	/**
	 * @private
	 */
	doScroll: function() {
		var newruWidth = this.parent.offsetWidth - this.parent.clientWidth;
		if (newruWidth > 0 && newruWidth != this.ruWidth) {
			this.ruCorner.style.width = newruWidth + "px";
			this.ruWidth = newruWidth;
		}
		if (!this.rowHeight) this.calculateRowHeight();
		this.height = Math.floor(this.parent.clientHeight / this.rowHeight);
		var len = this.storage ? this.storage.ids.length : 0;
		var newHeight = (len * this.rowHeight) + "px"
		var hDivStyle = this.heightDiv.style;
		if (newHeight != hDivStyle.height) hDivStyle.height = newHeight;
		var Self = this;
		if (this.heightDiv.offsetWidth != this.parent.clientWidth)
			hDivStyle.width = Self.parent.clientWidth + "px";
		var scrolltop = this.parent.scrollTop;
		var from2 = Math.max(0, Math.floor(                           scrolltop  / this.rowHeight / this.granularity) * this.granularity - this.preview);
		var to2 = Math.min(len, Math.ceil((this.parent.clientHeight + scrolltop) / this.rowHeight / this.granularity) * this.granularity + this.preview);
		var Self = this;
		if (this.fullRedraw || from2 != this.from || to2 != this.to) {
			if (this.fullRedraw || from2 > this.to || this.from > to2) {
				for (var i = this.from; i < this.to; i++) this.deleteRow(i);
				for (var i = from2; i < to2; i++) this.insertRow(i);
			} else {
				for (var i = this.from; i < from2; i++) this.deleteRow(i);
				for (var i = to2; i < this.to; i++) this.deleteRow(i);
				for (var i = from2; i < this.from; i++) this.insertRow(i);
				for (var i = this.to; i < to2; i++) this.insertRow(i);
			}
			this.from = from2;
			this.to = to2;
		}
		this.fullRedraw = false;
        this.emptyMessage.style.display = (!this.storage || len) ? "none"
                                                                : "block";
		var ids = [];
		for (var i in this.unfetched) ids.push(this.storage.ids[i]);
		if (ids.length) {
			if (this.iterateRequest) this.storage.cancel(this.iterateRequest);
			this.iterateRequest = this.storage.newIterate(ids, function(n) {
				if (!(n in Self.unfetched)) return;
				var cellrow = Self.rows[n].cells;
				for (var i in cellrow) Self.columns[i].clear(cellrow[i]);
			}, function(n, obj) {
				if (!(n in Self.unfetched)) return;
				delete Self.unfetched[n];
                var cellrow = Self.rows[n].cells;
				for (var i in cellrow) {
					var col = Self.columns[i];
					col.set(cellrow[i], col.index ? obj[col.index] : obj);
				}
			}, function() { Self.iterateRequest = null; }, 20);
		}
	},
	
	/**
	 * @private
	 */
	scrollTo: function(y) {
		var parent = this.parent;
		setTimeout(function() { parent.scrollTop = y; }, 0);
	},
	
	/**
	 * Scrolls if necessary to display the currently focused row.
	 */
	showFocus: function() {
		var scrolltop = this.parent.scrollTop;
		if (!this.rowHeight) this.calculateRowHeight();
		if (this.focus < scrolltop / this.rowHeight)
			this.scrollTo(this.focus * this.rowHeight);
		else {
			var height = this.parent.clientHeight;
			if (this.focus + 1 > (scrolltop + height) / this.rowHeight)
				this.scrollTo((this.focus + 1) * this.rowHeight - height);
		}
	},

	/**
	 * @private
	 */
	key: function(e) {
		if (!this.storage) return;
		switch (e.keyCode || e.which) {
		case 13: // enter
			this.events.trigger("Activated", this.selection.getSelected());
			break;
		case 32: // spacebar
			this.selection.click(this.focus, e.ctrlKey, e.shiftKey);
			this.showFocus();
			break;
		case 46: // delete
		case 108: // numpad delete
			this.events.trigger("Delete", this.selection.getSelected());
			break;
		case 38: // arrow up
			if (this.focus > 0) this.focus--;
			if (!e.ctrlKey)
				this.selection.click(this.focus, e.ctrlKey, e.shiftKey);
			this.showFocus();
			break;
		case 40: // arrow down
			if (this.focus < this.rowCount - 1) this.focus++;
			if (!e.ctrlKey)
				this.selection.click(this.focus, e.ctrlKey, e.shiftKey);
			this.showFocus();
			break;
		case 33: // page up
			this.focus -= this.height;
			if (this.focus < 0) this.focus = 0;
			if (!e.ctrlKey)
				this.selection.click(this.focus, e.ctrlKey, e.shiftKey);
			this.showFocus();
			break;
		case 34: // page down
			this.focus += this.height;
			if (this.focus >= this.rowCount) this.focus = this.rowCount - 1;
			if (!e.ctrlKey)
				this.selection.click(this.focus, e.ctrlKey, e.shiftKey);
			this.showFocus();
			break;
		case 35: // end
			this.focus = this.rowCount - 1;
			if (!e.ctrlKey)
				this.selection.click(this.focus, e.ctrlKey, e.shiftKey);
			this.showFocus();
			break;
		case 36: // home
			this.focus = 0;
			if (!e.ctrlKey)
				this.selection.click(this.focus, e.ctrlKey, e.shiftKey);
			this.showFocus();
			break;
		case 65: // A
		case 97: // a
			if (e.ctrlKey)
				this.selection.select(0, this.storage.ids.length);
			break;
		default:
			return;
		}
		this.updateStyles();
		stopEvent(e);
		return false;
	},
	
	/**
	 * @private
	 */
	mousedown: function(e) {
		this.clickprocessed = false;
		var focus = getRowIndex(e);
		if (this.storage.ids.length < this.height) {
			if (focus === undefined) focus = -1; else this.focus = focus;
			if(!this.selection.get(focus)) {
				this.clickprocessed=true;
				this.selection.click(focus, e.ctrlKey, e.shiftKey);
			}
		} else if (focus !== undefined) {
			this.focus = focus;
			if(!this.selection.get(focus)) {
				this.clickprocessed=true;
				this.selection.click(focus, e.ctrlKey, e.shiftKey);
			}
		}
//		cancelDefault(e);
		if (!IE6) setFocus(this.parent);
	},
	
	/**
	 * @private
	 */
	click: function(e) {
		if (this.clickprocessed) return;
		var focus = getRowIndex(e);
		if (this.storage.ids.length < this.height) {
			if (focus === undefined) focus = -1; else this.focus = focus;
			this.selection.click(focus, e.ctrlKey, e.shiftKey);
		} else if (focus !== undefined) {
			this.focus = focus;
			this.selection.click(focus, e.ctrlKey, e.shiftKey);
		}
//		cancelDefault(e);
		if (!IE6) setFocus(this.parent);
	},
	
	/**
	 * @private
	 */
	dblclick: function(e) {
		if (!this.storage) return;
		this.focus = getRowIndex(e) || 0;
		this.events.trigger("Activated", this.selection.getSelected());
	}
};

LiveGrid.setSortClass = classNameSetter("livegrid-th-sorted");


/**
 * Default set function for use in the second parameter of the constructor.
 * This function adds a text node in the specified DIV element and sets text
 * to the specified content value.
 * @param {Object} div The HTML DIV element to fill with content.
 * @param {String} text The text to set in the DIV element.
 */
LiveGrid.defaultSet = function(div, text) {
	text = text || "\u00a0";
	if (div.firstChild)
		div.firstChild.data = text;
	else
		div.appendChild(document.createTextNode(text));
};

/**
 * Set function for use in formatting dates in the live grid.
 * This function adds a text node in the specified DIV element and sets text
 * to the specified date value.
 * @param {Object} div The HTML DIV element to fill with content.
 * @param {String} utc The utc to format in the DIV element.
 */
LiveGrid.dateSet = function(div, date) {
	var dateString = "\u00a0";
	if (null != date) dateString = formatDate(date, "date");
	if (div.firstChild)
		div.firstChild.data = dateString;
	else
		div.appendChild(document.createTextNode(dateString));
};

/**
 * Set function for use in formatting dates in the live grid.
 * This function adds a text node in the specified DIV element and sets text
 * to the specified date value.
 * @param {Object} div The HTML DIV element to fill with content.
 * @param {String} utc The utc to format in the DIV element.
 */
LiveGrid.dateSetTime = function(div, date) {
	if (div.firstChild)
		div.firstChild.data = formatDate(date, "datetime");
	else
		div.appendChild(document.createTextNode(formatDate(date, "datetime")));
};

/**
 * Creates a clear function for use in the second parameter of the constructor.
 * This function creates a function which sets all DIV elements of the LiveGrid
 * to the text which is specified as parameter to this function.
 * @param {String} text The text to set in DIV elements.
 * @type Function
 * @return A function which takes a HTML DIV element as parameter, adds a text
 * node to it, and sets the text to a constant value.
 */
LiveGrid.makeClear = function(text) {
	return function(div) {
		if (div.firstChild)
			div.firstChild.data = text;
		else
			div.appendChild(document.createTextNode(text));
	};
};

















function LiveGrid2(columns, selection, module) {
    this.disablehover=false;
    this.emptylivegridtext="This folder is empty";/*i18n*/

    /**
     * @private
     */
    this.columns = columns;
    
    /**
     * @private
     */
    this.clickprocessed=false;

    /**
     * The selection displayed in this LiveGrid2.
     */
    this.selection = selection || console.error("LiveGrid2 without Selection");
    
    /**
     * @private
     */
    this.module = module;

    /**
     * Index of the anchor row.
     * The anchor is used for multiple-selects with the Shift key.
     * 0&nbsp;&lt;=&nbsp;anchor&nbsp;&lt;&nbsp;rowCount.
     * @private
     */
    this.anchor = 0;
    
    /**
     * Index of the focus row.
     * The focus is used for all selects.
     * 0&nbsp;&lt;=&nbsp;focus&nbsp;&lt;&nbsp;rowCount.
     * @private
     */
    this.focus = 0;
    
    /**
     * @private
     * Sort order. Either "asc" or "desc".
     */
    this.sort_order = "asc";
    
    /**
     * @private
     * ID of the column used for sorting. null if sorting is disabled.
     */
    this.sort_id = null;
    
    for (var i = 0; i < columns.length; i++)
        if (!this.sort_id && columns[i].sortable) this.sort_id = columns[i].sort;

    this.lineHeight = 1.6;
    
    /**
     * @private
     */
    this.from = 0;

    /**
     * @private
     */
    this.to = 0;
    
    /**
     * @private
     */
    this.rowcache = [];
    
    /**
     * @private
     * {Int: true}
     */
    this.unfetched = {};

    var Self = this;

    /**
     * @private
     */
    this.fullRedraw = true;
    
    /**
     * @private
     */
    this.scroll_cb = function() { Self.scroll(); };
    
    /**
     * @private
     */
    this.key_cb = function(e) { return Self.key(e); };

    /**
     * @private
     */
    this.keydown_cb = function(e) {
        this.keydown = true;
        return Self.key(e);
    };
    
    /**
     * @private
     */
    this.keypress_cb = function(e) {
        if (this.keydown)
            this.keydown = false;
        else
            return Self.key(e);
    };
    
    /**
     * @private
     */
    this.mousedown_cb = function(e) { Self.mousedown(e); };
    
    /**
     * @private
     */
    this.click_cb = function(e) { Self.click(e); };
    
    /**
     * @private
     */
    this.dblclick_cb = function(e) { Self.dblclick(e); };
    
    /**
     * @private
     */
    this.focus_cb = function() {
        focusedElement = Self.parent;
        Self.updateStyles();
    };
    
    this.language_cb = function(e) { Self.scroll(true); }
    
    /**
     * @private
     */
    this.blur_cb = function() {
        if (focusedElement == Self.parent)
            focusedElement = null;
        Self.updateStyles();
    };
    
    /**
     * @private
     */
    this.change_cb = function(from, to) {
        Self.rowCount = Self.collection.objects.length;
        from = Math.max(from, Self.from);
        to = Math.min(to, Self.to);
        for (var i = from; i < to; i++) Self.unfetched[i] = true;
        Self.doScroll();
        Self.updateStyles();
    };
    
    /**
     * @private
     */
    this.selected_cb = function(count) {
        Self.updateStyles();
        Self.events.trigger("Selected", count);
    };
    
    /**
     * Events triggered by this LiveGrid2.
     * <dl><dt>Selected</dt><dd>The selection has changed. Triggered only when
     * this grid is enabled. Parameters:
     * <ul><li>Number of selected elements.</li></ul></dd>
     * <dt>Activated</dt><dd>One or more elements have been activated.
     * Parameters:
     * <ul><li>Array with object IDs of activated objects.</li></ul></dd>
     * <dt>Delete</dt><dd>One or more elements should be deleted.
     * Parameters:
     * <ul><li>Array with object IDs of activated objects.</li></ul></dd>
     * </dl>
     */
    this.events = new Events();
}

LiveGrid2.prototype = {
    constructor : "LiveGrid2",
    /**
     * How many additional invisible rows to create to allow flicker-free
     * scrolling at faster speeds.
     */
    preview: 15,
    
    /**
     * Granularity of row fetches.
     */
    granularity: 15,
    
    /**
     * Creates a DOM tree for the headers of the LiveGrid2.
     * The returned DOM tree must be inserted in its place in the document.
     * @type Object
     * @return A DOM tree which displays the headers of the LiveGrid2.
     * @see #getTable
     */
    getHeader: function() {
        var Self = this;
        this.headers = new Array(this.columns.length);
        this.outer_headers = new Array(this.columns.length);
        this.arrows = new Array(this.columns.length);
        this.arrow_events = new Array(this.columns.length);
        for (var i in this.columns) this.arrow_events[i] = sortHandler(i);
        function sortHandler(i) {
            return function(e) {
                Self.setSort(Self.columns[i].sort,
                    (Self.sort_id == Self.columns[i].sort &&
                     Self.sort_order == "asc") ? "desc" : "asc");
                stopEvent(e);
            }
        }
        this.ids = {};
        for (var i in this.columns) {
			if(this.columns[i].sort) {
		        this.ids[this.columns[i].sort] = i;		
			}
		}
        var colgroup = this.getColGroup();
        colgroup.appendChild(this.ruCorner = newnode("col", {width: "0"}));
        var tbody = newnode("tbody",{verticalAlign : "top"});
        var table = newnode("table", {tableLayout: "fixed"},
                            {cellPadding: "0", cellSpacing: "0", className: "livegrid"},
                            [colgroup, tbody]);
        var row = newnode("tr", {height: this.lineHeight + "em"},
                          {className: "th"});
        for (var i = 0; i < this.columns.length; i++) {
            var outerCell, innerCell;
            var headerClassName = this.columns[i].name
                ? "header livegrid-column-" +
                      this.columns[i].name.join(" livegrid-column-")
                : "header";
            if (this.columns[i].sortable) {
                outerCell = this.outer_headers[i] =
                    newnode("th", this.columns[i].style, {className: headerClassName}, [
                        newnode("table", 0, {cellPadding: "0", cellSpacing: "0"}, [
                            newnode("tbody", 0, 0, [
                                newnode("tr", 0, 0, [
                                    innerCell = this.headers[i] = newnode("th", {whiteSpace: "nowrap"}),
                                    newnode("th", 0, 0, [
                                        this.arrows[i] = newnode("img", {verticalAlign: "middle"}, {src: getFullImgSrc("img/dummy.gif"), width: "16", height: "16"})
                                    ])
                                ])
                            ])
                        ])
                    ]);
            } else
                innerCell = outerCell = this.headers[i] = this.outer_headers[i] =
                    newnode("th", this.columns[i].style, {className: headerClassName});
            if (this.columns[i].i18n) {
                var node = new I18nNode(function() {
                    return pgettext(this.context, this.text); });
                node.text = this.columns[i].text;
                node.context = this.columns[i].i18n;
                if (node.context === true) node.context = "";
                node.parent = innerCell;
                node.update = function() {
                    this.parent.title = this.node.data = this.callback();
                };
                innerCell.appendChild(node.node);
                node.update();
            } else switch (typeof this.columns[i].text) {
                case "string":
                    innerCell.innerHTML = this.columns[i].text;
                    break;
                case "function":
                    innerCell.appendChild(addTranslated(this.columns[i].text));
                    break;
                default:
                    innerCell.appendChild(this.columns[i].text);
            }
            row.appendChild(outerCell);
        }
        row.appendChild(newnode("th"));
        var index = this.ids[this.sort_id];
        if (this.sort_id && index && this.arrows[index]) {
            this.arrows[index].src = 
                this.sort_order == "asc" ? getFullImgSrc("img/arrows/sort_up.gif") :
                this.sort_order == "desc" ? getFullImgSrc("img/arrows/sort_down.gif") :
                getFullImgSrc("img/dummy.gif"); // just in case
            LiveGrid2.setSortClass(this.outer_headers[index], true);
         }
        tbody.appendChild(row);
        return table;
    },
    
    /**
     * Adds the scrollable area of the LiveGrid2 to the specified parent node.
     * The parent node should have an explicit size and "overflow: auto".
     */
    getTable: function(parent) {
        this.rows = {};
        var last = this.columns.length - 1;
        this.parent = parent;
        this.heightDiv = newnode("div", {height: "0em"}, {className: "livegrid"});
        parent.appendChild(this.heightDiv);
        parent.tabIndex = 0;
        parent.style.outline = "none";
        parent.style.overflowX = "hidden";
        parent.hideFocus = true;
        var emptyText = addTranslated(this.emptylivegridtext);
        this.emptyMessage = newnode("div",
            {display: "none", position: "absolute", textAlign: "center", 
             width: "100%", marginTop: "5px"}, 0,
            [emptyText]);
        parent.appendChild(this.emptyMessage);
        var tr = newnode("tr");
        for (var i = 0; i < this.columns.length; i++)
            tr.appendChild(newnode("td", this.columns[i].style,
                                   {className: "cell"}));
        this.rowTemplate = newnode("table", 0,
            {cellPadding: "0", cellSpacing: "0", className: "tr"},
            [this.getColGroup(), newnode("tbody", 0, 0, [tr])]);
    },
    
    /**
     * Adds a hover to the LiveGrid2.
     * This method should be called between getTable() and the first enable(). 
     * @param {Object} hover A DOM node which serves as a hover. It should be
     * attached directly to the body and have the CSS style
     * "position: absolute".
     * @param {Function} callback A callback function which is called when
     * the hover is shown. As its only parameter, it receives the ID of
     * the entry over which the mouse is hovering.
     * @type {Object}
     * @return A new Hover object. The enable and disable methods of the hover
     * object will be called automatically by the LiveGrid2.
     */
    addHover: function(hover, callback) {
		this.hover = new Hover(this.heightDiv, hover);
        var Self = this;
        this.hover.onShow = function(node) {
		    var i = node.oxRowIndex;
            if (i !== undefined)
                callback(Self.collection.objects[i]);
            else
                return false;
        }
        return this.hover;
    },
    
    /**
     * @private
     */
    getColGroup: function() {
        var colgroup = newnode("colgroup");
        for (var i = 0; i < this.columns.length; i++) {
            var w = this.columns[i].width;
            colgroup.appendChild(newnode("col", w ? {width: w} : 0));
        }
        return colgroup;
    },
    
    /**
     * Appends the contents either of the entire grid or of selected rows to
     * the specified node.
     * @param {Object} parent The node to append the data to.
     * @param {Function} cb The callback to call after all data is appended.
     * @param {Boolean} selected True if only selected rows should be appended.
     * Otherwise, all rows are appended.
     */
    print: function(parent, cb, selected) {
        var doc = parent.ownerDocument;
        var tr = doc.createElement("tr");
        for (var i = 0; i < this.columns.length; i++) {
            var th = doc.createElement("th");
            if (this.columns[i].i18n)
                th.appendChild(doc.createTextNode(_(this.columns[i].text) || "\u00a0"));
            else
                th.innerHTML = this.columns[i].text || "\u00a0";
            tr.appendChild(th);
        }
        var table = newnode("table", {width: "100%"}, {border: 1},
            [newnode("thead", 0, 0, [tr], doc)], doc);
        var trs = [];
        var Self = this;
        function set(n, obj) {
            trs[n] = doc.createElement("tr");
            for (var i = 0; i < Self.columns.length; i++) {
                var td = doc.createElement("td");
                trs[n].appendChild(td);
                var col = Self.columns[i];
                col.set(td, col.index ? obj[col.index] : obj);
            }
        }
        function final_() {
            var tbody = newnode("tbody", 0, 0, 0, doc);
            for (var i = 0; i < trs.length; i++) 
                if (trs[i]) tbody.appendChild(trs[i]);
            table.appendChild(tbody);
            parent.appendChild(table);
            if(trs.length==0) {
                return; 
            }
            cb();
        }
		var selection = (selected) ? this.selection.getSelected() : this.collection.objects;
		var tmpColumns=[];
        for(var i=0; i<this.columns.length;i++) {
            for(var i2=0;i2<this.columns[i].name.length;i2++) {
                tmpColumns.push(this.columns[i].name[i2]);  
            }
        }
        OXCache.newRequest(null,this.module,{
                objects : selection,
                columns : tmpColumns
            },null,function callback(data) {
				if (!data || !data.objects) {
					return;
				}
				for(var i=0;i<data.objects.length;i++) {
					var obj=data.objects[i];
					var tr = doc.createElement("tr");
					for (var i2 = 0; i2 < Self.columns.length; i2++) {
                        var td = doc.createElement("td");
                        tr.appendChild(td);
					    var col = Self.columns[i2];
                        if(col.name.length==1) {
						  col.set(td,  obj[col.name[0]]);	
						} else {
					      col.set(td,  obj);
						}
  				    }
				    trs.push(tr); 
                 }
				 var tbody = newnode("tbody", 0, 0, 0, doc);
                 for (var i = 0; i < trs.length; i++) {
				 	if (trs[i]) {
						tbody.appendChild(trs[i]);
					}
				 }
                 table.appendChild(tbody);
                 parent.appendChild(table);
                 if(trs.length==0) {
                    return; 
                 }
                 cb();
            }
		);    
    },
    
    /**
     * Displays the specified sort order as an arrow in the headers.
     * @param {String} id The column ID used for sorting, or null for no
     * sorting.
     * @param {String} order The sort order. One of "asc", "desc" or null for no
     * sorting.
     */
    setSort: function(name, order) {
        if (this.sort_id && name != this.sort_id) {
            var index = this.ids[this.sort_id];
			if(index || index === 0) {
			    this.arrows[index].src = getFullImgSrc("img/dummy.gif");
                LiveGrid2.setSortClass(this.outer_headers[index], false);
			}
		}
        var index = this.ids[name];
        if(index || index === 0) {
            this.arrows[index].src = 
                order == "asc" ? getFullImgSrc("img/arrows/sort_up.gif") :
                order == "desc" ? getFullImgSrc("img/arrows/sort_down.gif") :
                getFullImgSrc("img/dummy.gif"); // just in case
            LiveGrid2.setSortClass(this.outer_headers[index], true);
        }
		this.sort_id = name;
        this.sort_order = order;
        if (this.collection) this.sort(name, order);
    },
    
    /**
     * Callback function which changes the displayed data.
     * Should be overridden in each instance.
     * @param {String} id The column ID used for sorting, or null for no
     * sorting.
     * @param {String} order The sort order. Either "asc" or "desc".
     */
    sort: function() {
		if(!this.collection) {
			return
		}
		this.enable(this.collection.criteria, this.collection.search);
	},
    getSelectedIDs: function() {
        return this.selection.getSelected();
    },
    
    /**
     * Deletes items with specified object IDs from the collection and from
     * the selection. If the entire selection was deleted, the first unselected
     * item at or after the focus is selected.
     */
    deleteIDs: function(ids) {
		/*
        var sids = {};
        for (var i = 0; i < ids.length; i++)
            sids[this.storage.serialize(ids[i])] = true;
        while (this.storage.getSID(this.focus) in sids) this.focus++;
        var newID = this.storage.ids[this.focus];
        this.storage.removeIDs(ids);
        this.selection.deselectSIDs(sids);
        this.focus = newID ? this.storage.getIndex(newID) : 0;
        if (!this.selection.count)
            this.selection.click(this.focus);
        */
    },
    
    updateSelected: function(callback) {
        //if (this.collection)
            //this.storage.localUpdate(this.selection.getSelected(), callback);
    },

    /**
     * Initializes the LiveGrid2.
     * The DOM tree must have been created by {@link #getHeader} and
     * {@link #getTable}.
     * @see #getHTML
     */
	enable : function(criteria,search,force) {
		var Self=this;
		var collectiondata = {
             "criteria" : criteria,
			 "order" : [{sort : this.sort_id , order : this.sort_order}],
             columns : []
        };
		if(search) {
			collectiondata["search"]=search;
		}
        if(this.collection) {
			if(this.uniqueName) {
			    OXCache.unregister(this.uniqueName);
                delete this.uniqueName;
            }
			if(this.listRequest) {
                OXCache.unregister(this.listRequest.uniqueName);
                delete this.listRequest;
            }   
		} else {
			for (var i in Self.arrows) {
                addDOMEvent(Self.outer_headers[i], "click", Self.arrow_events[i]);
			}
            addDOMEvent(Self.parent, "scroll", Self.scroll_cb);
            addDOMEvent(Self.parent, "keydown", Self.keydown_cb);
            addDOMEvent(Self.parent, "keypress", Self.keypress_cb);
            addDOMEvent(Self.parent, "mousedown", Self.mousedown_cb);
            addDOMEvent(Self.parent, "click", Self.click_cb);
            addDOMEvent(Self.parent, "dblclick", Self.dblclick_cb);
            addDOMEvent(Self.parent, "focus", Self.focus_cb);
            addDOMEvent(Self.parent, "blur", Self.blur_cb);
            if (Self.hover && !Self.disablehover) Self.hover.enable();
            resizeEvents.register("Resized", Self.scroll_cb);
            register("LanguageChanged",Self.language_cb);
            if (Self.module && (Self.module != "links")) {
                var disabled;
                var validfield;
                switch(Self.module) {
                    case "calendar": disabled=calendardefaultdisabled;  validfield="title"; break;
                    case "tasks":   disabled=taskdefaultdisabled; validfield="title"; break;
                    case "contacts": disabled=contactdefaultdisabled; validfield="display_name"; break;
                    case "infostore": disabled=infostoredefaultdisabled; validfield="title"; break;
                    case "mail": disabled=maildefaultdisabled; validfield="subject"; break;
                }
                Self.contextmenu = registerContext(Self.heightDiv,Self.module);
                Self.dragEvent = registerSource(Self.heightDiv, Self.module, function() 
                {
                    var myelements=clone(Self.selection.getSelected());
                    objMoveCopy.begin(Self.module,myelements);
                    return objMoveCopy;
                },null,null,disabled,defaultdisabledremove);
            }
        }
		var returnObject=OXCache.newRequest(null,this.module,collectiondata,
			function modifiedCallback(data) {
			  Self.collection=data;
	          Self.rowCount = data.objects.length;
	          Self.selection.setCollection(Self.collection);
	          Self.change_cb(0,data.objects.length); 
			  Self.events.trigger("Changed", data, true);
			},
			function callback(data) {
				Self.collection=data;
	            Self.rowCount = data.objects.length;
	            Self.selection.setCollection(Self.collection, Self.selection.selectedIndex);
	            Self.scroll(true);
				Self.events.trigger("Changed", data, false);
			},force);
		this.uniqueName=returnObject.uniqueName;
		Self.selection.events.register("Selected", Self.selected_cb);
	},
    disable: function() {
        if (!this.collection) return;
        this.selection.events.unregister("Selected", this.selected_cb);
        if(this.uniqueName) {
			OXCache.unregister(this.uniqueName);
            delete this.uniqueName;
		}
		if(this.listRequest) {
            OXCache.unregister(this.listRequest.uniqueName);
            delete this.listRequest;
        }
		this.collection = null;
        this.selection.setCollection(null);
        for (var i in this.arrows)
            removeDOMEvent(this.outer_headers[i], "click", this.arrow_events[i]);
        removeDOMEvent(this.parent, "scroll", this.scroll_cb);
        removeDOMEvent(this.parent, "keydown", this.keydown_cb);
        removeDOMEvent(this.parent, "keypress", this.keypress_cb);
        removeDOMEvent(this.parent, "mousedown", this.mousedown_cb);
        removeDOMEvent(this.parent, "click", this.click_cb);
        removeDOMEvent(this.parent, "dblclick", this.dblclick_cb);
        removeDOMEvent(this.parent, "focus", this.focus_cb);
        removeDOMEvent(this.parent, "blur", this.blur_cb);
        unregister("LanguageChanged",this.language_cb);
        if (this.hover) this.hover.disable();
        resizeEvents.unregister("Resized", this.scroll_cb);
        if (this.module) {
            unregisterSource(this.parent, this.dragEvent);
            unregisterContext(this.heightDiv, this.contextmenu);
        }
    },
    
    /**
     * @private
     */
    createRow: function() {
        var last = this.columns.length - 1;
        var row = {tr: this.rowTemplate.cloneNode(true),
                   cells: new Array(this.columns.length)};
        for (var i = 0, cell = row.tr.lastChild.firstChild.firstChild; cell;
             i++, cell = cell.nextSibling)
        {
            row.cells[i] = cell;
            this.columns[i].clear(cell);
        }
        this.heightDiv.appendChild(row.tr);
        if (!this.rowHeight) this.rowHeight = row.tr.offsetHeight;
        return row;
    },

    /**
     * To be removed.
     * @private
     */
    updateStyles: function() {
        for (var i in this.rows) {
            var newclass = this.selection.get(i) ? "tr selected" : i%2 ? "tr " : "tr_h ";
            if (focusedElement == this.parent && i == this.focus)
                newclass += "focused";
            var tr = this.rows[i].tr;
            if (tr.className != newclass) tr.className = newclass;
        }
    },

    /**
     * @private
     */
    deleteRow: function(index) {
        var row = this.rows[index];
        row.tr.style.display = "none";
        this.rowcache.push(row);
        delete this.rows[index];
        delete this.unfetched[index];
    },

    /**
     * @private
     */
    insertRow: function(index) {
        var row = this.rows[index] =
            this.rowcache.length ? this.rowcache.pop() : this.createRow();
        row.tr.className = this.selection.get(index) ? "tr selected" : index%2 ? "tr" : "tr_h";
        row.tr.style.display = "";
        row.tr.style.top = (index * this.rowHeight) + "px";
        row.tr.oxRowIndex = index;
        this.unfetched[index] = true;
    },

    /**
     * @private
     */
    scroll: function(fullRedraw) {
        if (fullRedraw) this.fullRedraw = true;
        if (this.scrollTimeout) {
			clearTimeout(this.scrollTimeout);
		}
        var Self = this;
        this.scrollTimeout = setTimeout(function() {
            Self.scrollTimeout = null;
            Self.doScroll();
        });
    },
    
    /**
     * @private
     */
    calculateRowHeight: function() {
        var row = this.createRow();
        row.tr.style.display = "none";
        this.rowcache.push();
    },

    /**
     * @private
     */
    doScroll: function() {
		var Self=this;       
	    var newruWidth = this.parent.offsetWidth - this.parent.clientWidth;
        if (newruWidth > 0 && newruWidth != this.ruWidth) {
            this.ruCorner.style.width = newruWidth + "px";
            this.ruWidth = newruWidth;
        }
        if (!this.rowHeight) this.calculateRowHeight();
        this.height = Math.floor(this.parent.clientHeight / this.rowHeight);
        var len = this.collection ? this.collection.objects.length : 0;
        var newHeight = (len * this.rowHeight) + "px"
        var hDivStyle = this.heightDiv.style;
        if (newHeight != hDivStyle.height) hDivStyle.height = newHeight;
        var Self = this;
        if (this.heightDiv.offsetWidth != this.parent.clientWidth)
            hDivStyle.width = Self.parent.clientWidth + "px";
        var scrolltop = this.parent.scrollTop;
        var from2 = Math.max(0, Math.floor(                           scrolltop  / this.rowHeight / this.granularity) * this.granularity - this.preview);
        var to2 = Math.min(len, Math.ceil((this.parent.clientHeight + scrolltop) / this.rowHeight / this.granularity) * this.granularity + this.preview);
		if (this.fullRedraw || from2 != this.from || to2 != this.to) {
            if (this.fullRedraw || from2 > this.to || this.from > to2) {
                for (var i = this.from; i < this.to; i++) this.deleteRow(i);
                for (var i = from2; i < to2; i++) this.insertRow(i);
            } else {
                for (var i = this.from; i < from2; i++) this.deleteRow(i);
                for (var i = to2; i < this.to; i++) this.deleteRow(i);
                for (var i = from2; i < this.from; i++) this.insertRow(i);
                for (var i = this.to; i < to2; i++) this.insertRow(i);
            }
            this.from = from2;
            this.to = to2;
        }
        this.fullRedraw = false;
        this.emptyMessage.style.display = (!this.collection || len) ? "none"
                                                                    : "block";
        var ids = [];
        for (var i in this.unfetched) ids.push(this.collection.objects[i]);
        if (ids.length) {        	
			var tmpColumnsObj=new Object();
            for(var i=0; i<this.columns.length;i++) {
                for(var i2=0;i2<this.columns[i].name.length;i2++) {
                    tmpColumnsObj[this.columns[i].name[i2]] = this.columns[i].name[i2];  
                }
            }
            var tmpColumns=[];
            for (var i in tmpColumnsObj) {
            	  tmpColumns.push(i);
            }
            if(this.listRequest) {
				OXCache.unregister(this.listRequest.uniqueName);
				delete this.listRequest;
			}
            var collection = this.collection;
		    this.listRequest=OXCache.newRequest(null,this.module,{
                objects : ids,
                columns : tmpColumns
            },null,function cb(data) {
                for(var i=0;i<data.objects.length;i++) {
                    var objIndex=collection.map_objects.get(data.objects[i]);
                    if(!Self.rows[objIndex-1]) return;
                    delete Self.unfetched[objIndex-1];
                    var cellrow = Self.rows[objIndex-1].cells;
                    for (var i2 in cellrow) {
                        var col = Self.columns[i2];
                        if(col.name.length==1) {
                          col.set(cellrow[i2], data.objects[i][col.name[0]]);   
                        } else {
                          col.set(cellrow[i2], data.objects[i]);
                        }
                    }
                }
                delete this.listRequest;
            },false);
			if(this.listRequest) {
				for(var i in this.listRequest.clear) {
                    var objIndex=this.collection.map_objects.get(this.listRequest.clear[i]);
                    var cellrow = Self.rows[objIndex-1].cells;
                    for (var i in cellrow) Self.columns[i].clear(cellrow[i]);
                }
				for(var i in this.listRequest.set) {
                	var objIndex=this.collection.map_objects.get(this.listRequest.set[i]);
					var cellrow = Self.rows[objIndex-1].cells;
                    for (var i2 in cellrow) {
                        var col = Self.columns[i2];
                        if(col.name.length==1) {
                          col.set(cellrow[i2], this.listRequest.set[i][col.name[0]]);   
                        } else {
                          col.set(cellrow[i2], this.listRequest.set[i]);
                        }
                    }
                }
            }
        }
    },
    
    /**
     * @private
     */
    scrollTo: function(y) {
        var parent = this.parent;
        setTimeout(function() { parent.scrollTop = y; }, 0);
    },
    
    /**
     * Scrolls if necessary to display the currently focused row.
     */
    showFocus: function() {
        var scrolltop = this.parent.scrollTop;
        if (!this.rowHeight) this.calculateRowHeight();
        if (this.focus < scrolltop / this.rowHeight)
            this.scrollTo(this.focus * this.rowHeight);
        else {
            var height = this.parent.clientHeight;
            if (this.focus + 1 > (scrolltop + height) / this.rowHeight)
                this.scrollTo((this.focus + 1) * this.rowHeight - height);
        }
    },

    /**
     * @private
     */
    key: function(e) {
        if (!this.collection) return;
        switch (e.keyCode || e.which) {
        case 13: // enter
            this.events.trigger("Activated", this.selection.getSelected());
            break;
        case 32: // spacebar
            this.selection.click(this.focus, e.ctrlKey, e.shiftKey);
            this.showFocus();
            break;
        case 46: // delete
        case 108: // numpad delete
            this.events.trigger("Delete", this.selection.getSelected());
            break;
        case 38: // arrow up
            if (this.focus > 0) this.focus--;
            if (!e.ctrlKey)
                this.selection.click(this.focus, e.ctrlKey, e.shiftKey);
            this.showFocus();
            break;
        case 40: // arrow down
            if (this.focus < this.rowCount - 1) this.focus++;
            if (!e.ctrlKey)
                this.selection.click(this.focus, e.ctrlKey, e.shiftKey);
            this.showFocus();
            break;
        case 33: // page up
            this.focus -= this.height;
            if (this.focus < 0) this.focus = 0;
            if (!e.ctrlKey)
                this.selection.click(this.focus, e.ctrlKey, e.shiftKey);
            this.showFocus();
            break;
        case 34: // page down
            this.focus += this.height;
            if (this.focus >= this.rowCount) this.focus = this.rowCount - 1;
            if (!e.ctrlKey)
                this.selection.click(this.focus, e.ctrlKey, e.shiftKey);
            this.showFocus();
            break;
        case 35: // end
            this.focus = this.rowCount - 1;
            if (!e.ctrlKey)
                this.selection.click(this.focus, e.ctrlKey, e.shiftKey);
            this.showFocus();
            break;
        case 36: // home
            this.focus = 0;
            if (!e.ctrlKey)
                this.selection.click(this.focus, e.ctrlKey, e.shiftKey);
            this.showFocus();
            break;
        case 65: // A
        case 97: // a
            if (e.ctrlKey)
                this.selection.select(0, this.collection.objects.length);
            break;
        default:
            return;
        }
        this.updateStyles();
        stopEvent(e);
        return false;
    },
    
    /**
     * @private
     */
    mousedown: function(e) {
        this.clickprocessed = false;
        var focus = getRowIndex(e);
        if (this.collection.objects.length < this.height) {
            if (focus === undefined) focus = -1; else this.focus = focus;
            if(!this.selection.get(focus)) {
                this.clickprocessed=true;
                this.selection.click(focus, e.ctrlKey, e.shiftKey);
            }
        } else if (focus !== undefined) {
            this.focus = focus;
            if(!this.selection.get(focus)) {
                this.clickprocessed=true;
                this.selection.click(focus, e.ctrlKey, e.shiftKey);
            }
        }
        cancelDefault(e);
        if (!IE6) setFocus(this.parent);
    },
    
    /**
     * @private
     */
    click: function(e) {
        if (this.clickprocessed) return;
        var focus = getRowIndex(e);
        if (this.collection.objects.length < this.height) {
            if (focus === undefined) focus = -1; else this.focus = focus;
            this.selection.click(focus, e.ctrlKey, e.shiftKey);
        } else if (focus !== undefined) {
            this.focus = focus;
            this.selection.click(focus, e.ctrlKey, e.shiftKey);
        }
        cancelDefault(e);
        if (!IE6) setFocus(this.parent);
    },
    
    /**
     * @private
     */
    dblclick: function(e) {
        if (!this.collection) return;
        this.focus = getRowIndex(e) || 0;
        this.events.trigger("Activated", this.selection.getSelected());
    }
};

LiveGrid2.setSortClass = classNameSetter("livegrid-th-sorted");

/**
 * Default set function for use in the second parameter of the constructor.
 * This function adds a text node in the specified DIV element and sets text
 * to the specified content value.
 * @param {Object} div The HTML DIV element to fill with content.
 * @param {String} text The text to set in the DIV element.
 */
LiveGrid2.defaultSet = function(div, text) {
    text = text || "\u00a0";
    if (div.firstChild)
        div.firstChild.data = text;
    else
        div.appendChild(document.createTextNode(text));
};

/**
 * Set function for use in formatting dates in the live grid.
 * This function adds a text node in the specified DIV element and sets text
 * to the specified date value.
 * @param {Object} div The HTML DIV element to fill with content.
 * @param {String} utc The utc to format in the DIV element.
 */
LiveGrid2.dateSet = function(div, date) {
    var dateString = "\u00a0";
    if (null != date) dateString = formatDate(date, "date");
    if (div.firstChild)
        div.firstChild.data = dateString;
    else
        div.appendChild(document.createTextNode(dateString));
};

/**
 * Set function for use in formatting dates in the live grid.
 * This function adds a text node in the specified DIV element and sets text
 * to the specified date value.
 * @param {Object} div The HTML DIV element to fill with content.
 * @param {String} utc The utc to format in the DIV element.
 */
LiveGrid2.dateSetTime = function(div, date) {
    if (div.firstChild)
        div.firstChild.data = formatDate(date, "datetime");
    else
        div.appendChild(document.createTextNode(formatDate(date, "datetime")));
};

/**
 * Creates a clear function for use in the second parameter of the constructor.
 * This function creates a function which sets all DIV elements of the LiveGrid2
 * to the text which is specified as parameter to this function.
 * @param {String} text The text to set in DIV elements.
 * @type Function
 * @return A function which takes a HTML DIV element as parameter, adds a text
 * node to it, and sets the text to a constant value.
 */
LiveGrid2.makeClear = function(text) {
    return function(div) {
        if (div.firstChild)
            div.firstChild.data = text;
        else
            div.appendChild(document.createTextNode(text));
    };
};




fileloaded();
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2007 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Viktor Pracht <viktor.pracht@open-xchange.com>
 * @author Jan Finsel <jan.finsel@open-xchange.com>
 * 
 */
 
/* TODO:
 * popup
 * static context menu
 * dynamic context menu
 * submenu (context menu)
 * pseudo-window: header, moving
 */
/**
 * Displays a popup.
 * @param {Object} content The DOM node which contains the popup.
 * @param {Int} x Optional horizontal position of the left edge of the popup.
 * @param {Int} y Optional vertical position of the top edge of the popup.
 */
displayAt = function(content, x, y, cb) {
	if (x && y) {
		content.style.left = x;
		content.style.top = y;
	}
	fade(content,0,100,500,cb);
};
var popupenabled=false;
/**
 * Displays a popup which disappears when the user clicks anywhere else.
 * @param {Object} content The DOM node which contains the popup.
 * @param {Int} x Optional horizontal position of the left edge of the popup.
 * @param {Int} y Optional vertical position of the top edge of the popup.
 */
var popup;
var popupHide;

(function() {

	var popups = [];	

	register("Loaded", function (){
		var moving = null;
		var offsetX, offsetY;
		var body = $("body");
		popupHide = function() {
			for (var i in popups) {
				if(popups[i].onhide) {
					popups[i].onhide();
				}
                fade_setOpacity(popups[i].popup, 0, false);
                showIFrames();
                popupenabled=false;
                popups.splice(i,1);
			}
		}
		addDOMEvent(body, "click", popupHide);
		addDOMEvent(body, "contextmenu", function(e) {
			popupHide();
			var DOMTarget = (e.target) ? e.target : e.srcElement;
			if (DOMTarget != undefined && DOMTarget.nodeName != "TEXTAREA") {				
				stopEvent(e);
				delete(DOMTarget);
				return false;
			}
			delete(DOMTarget);
		});
		for (var i in init.movable) {
			var node = $(init.movable[i]);
			(function(node, parent) {
				addDOMEvent(node, "mousedown", function(e) {
					if (e.button != LeftButton) return;
					moving = parent;
					offsetX = parent.offsetLeft - e.clientX;
					offsetY = parent.offsetTop - e.clientY;
					parent.style.cursor = "move";
					function moveHandler(e) {
						var topposition=(e.clientY + offsetY);
						var leftposition=(e.clientX + offsetX);
						if(topposition<0) {
							topposition=0;
						}
						else if(topposition+parent.offsetHeight > $("body").offsetHeight) {
							topposition=$("body").offsetHeight-parent.offsetHeight;
						}
						if(leftposition<0) {
							leftposition=0;
						}
						else if(leftposition+parent.offsetWidth > $("body").offsetWidth) {
							leftposition=$("body").offsetWidth-parent.offsetWidth;
						}
						moving.style.left = leftposition + "px";
						moving.style.top = topposition + "px";
						stopEvent(e);
					}
					function upHandler(e) {
						if (moving && e.button == LeftButton) {
							moving = null;
							parent.style.cursor = "auto";
							removeDOMEvent(body, "mousemove", moveHandler);
							removeDOMEvent(body, "mouseup", upHandler);
							//removeDOMEvent(body, "mouseout", upHandler);
							stopEvent(e);
						}
					}
					addDOMEvent(body, "mousemove", moveHandler);
					addDOMEvent(body, "mouseup", upHandler);
					//addDOMEvent(body, "mouseout", upHandler);
					cancelDefault(e);
				});
				addDOMEvent(node.firstChild, "click", function() {
					parent.style.display = "none";
				});
			})(node, node.parentNode);
		}
	});
	popup = function(content, x, y, onhide, type, leave_open) {
		/*
		 * @todo  now we have the same behaviour like the context menus under a well known OS.
		 *        only the panel menus doesn't behave as they should. a click should
		 *        enable / disable such menus. right now a click re-enables it all the time. 
		 */
		popupHide();
        hideIFrames();
        popupenabled=true;
        displayAt(content, x, y);
        setTimeout(function() {
            popups.push({popup: content, onhide: onhide, type : type, leave_open : leave_open }); 
        }, 0);
	};
	
})();
function centerPopupWindow(popupwindow) {
	var windowheight=$("body").offsetHeight;
	var windowwidth=$("body").offsetWidth;
	var tmpnode=popupwindow.cloneNode(true);
	tmpnode.style.display="block";
	tmpnode.style.zIndex=-1;
	body.appendChild(tmpnode);
	var popupwindowheight=tmpnode.offsetHeight;
	var popupwindowwidth=tmpnode.offsetWidth;
	body.removeChild(tmpnode);
	popupwindow.style.left =((windowwidth/2) -(popupwindowwidth/2)) +"px";
	popupwindow.style.top =((windowheight/2) -(popupwindowheight/2)) +"px";
}

/**
 * Adds a DOM event listener to the onclick event of the "Close" icon in the
 * right top corner of a popup window.
 * @param {Object} popupwindow The DOM node of the popup window.
 * @param {Function} onclose The callback function which will be called when
 * the popup window is closed by clicking on the "Close" icon.
 */
function addOnClose(popupwindow, onclose) {
	addDOMEvent(getElement(popupwindow.firstChild).firstChild, "click", onclose);
}

/**
 * Removes a previously added DOM event listener from the onclick event of
 * the "Close" icon in the rihgt top corner of a popup window.
 * @param {Object} popupwindow The DOM node of the popup window.
 * @param {Function} onclose The callback function which was supplied in
 * a previous call to {@link addOnClose}.
 */
function removeOnClose(popupwindow, onclose) {
	removeDOMEvent(getElement(popupwindow.firstChild).firstChild, "click", onclose);
}

/**
 * Adds a context menu to an element. the context menu must be a child of
 * the document body.
 * @param {Object} parent The DOM element to which the context menu is added.
 * @param {Object} menu The DOM element which contains the context menu.
 */
function addContextMenu(parent, menu,contxParam) {
	
	addDOMEvent(parent, "contextmenu", function(e) {
		if (e && e.button && e.button != RightButton) return;
		var newRefer  = contxParam.node;
		menu.context = newRefer;		
		popup(menu.div, e.clientX + "px", e.clientY + "px",function (){contxParam.onhide(newRefer)},"contextmenu2",true);
		menu.events.trigger('OX_Tree_Popup_Menu_Down',menu,parent);
		stopEvent(e);		
	});
}

/**
 * @param {Object} parent Optional absolutely positioned DOM element which
 * serves as the parent node for the popup menu. Defaults to the document body.
 */
function Popup(parent, contDiv) {
	this.entries = [];
	this.context;
	this.parent = parent || $("body");
	this.div = contDiv || newnode("div", {display: "none", position: "absolute"});
	this.parent.appendChild(this.div);
	this.events = new Events();
}

/**
 * @private
 */
Popup.makeDiv = function(entry) {
	return typeof(entry) == "string" ? newnode('div',0,{className:"PopupEntryDiv"},[addTranslated(entry)])
	                                 : entry;
}

Popup.prototype = {

	/**
	 * @param {Object or String} entry The new menu entry either as a DOM node
	 * or as a string.
	 */
	append: function(entry, callback, idOpt, disabled) {
		var div = Popup.makeDiv(entry);
		var Self = this;
		function modifiedCallBack(evt)
		{
			//stopEvent(evt);
			callback(evt,Self.context);
			Self.div.style.display = "none";
		}
		var entryobject = {div: div, entry_id: idOpt, indx: -1, callback: modifiedCallBack, disabled: disabled || false};
		addDOMEvent(div,'mouseover',function (e) {
			         div.className = (entryobject.disabled)?'PopupEntryDivDisabled font-color-disabled':"PopupEntryDivOver border-color-PMG-selection-elements";
			         //stopEvent(e);
			     });		
		addDOMEvent(div,'mouseout',function (e) {
			         div.className = (entryobject.disabled)?'PopupEntryDivDisabled font-color-disabled':"PopupEntryDiv";
			         //stopEvent(e);
			     });				
		if (entryobject.disabled) {
			div.className='PopupEntryDivDisabled font-color-disabled';
		} else {
			addDOMEvent(div,'click',modifiedCallBack);
		}
		this.entries.push(entryobject);
		entryobject.indx = this.entries.length - 1;		
		this.div.appendChild(div);
		return this.entries.length - 1;
	},
	
	clear: function() {
		for (var i in this.entries) {
			var entry = this.entries[i];
			if (entry.callback)
				removeDOMEvent(entry.div, "click", entry.callback);
		}
		this.entries = [];
		this.div.innerHTML = "";
	},

	remove: function(index) {
		this.div.removeChild(this.entries[i]);
		this.entries.splice(i, 1);
	},
	
	disableEntry: function (nEntry)
	{		
		var entry = this.entries[nEntry];
		removeDOMEvent(entry.div, "click", entry.callback);
		entry.div.className = 'PopupEntryDivDisabled font-color-disabled';
		entry.disabled = true;
	}
	,
	
	enableEntry: function (nEntry)
	{
		var entry = this.entries[nEntry];
		addDOMEvent(entry.div, "click", entry.callback);
		entry.div.className = '';
		entry.disabled = false;
	}
};



function ContextMenu() {
	this.entries = [];
	this.context;
	this.div = document.createElement("div");
	this.fillin;	
	this.div.style.display="none";
	this.div.className="PopupMenuDiv background-color-popup-menu border-color-popup-menu font-size-small";
	this.div.style.position="absolute";
	this.div.style.zIndex=1000;
}

ContextMenu.makeDiv = function(entry) {
	return typeof(entry) == "string" ? newnode('div',0,0,[addTranslated(entry)])
	                                 : entry();
}
ContextMenu.makeLine = function () {
	return function() {
		return newnode('hr',0,0,[]);     	
	}
}

ContextMenu.prototype = {
	append: function(object,optparam) {
				
		var Self = this;
		var entryobject=new Object();				
		var myentry=ContextMenu.makeDiv(object.entry);
		if(object.status) {
			if(object.conditionvisible(entryobject,optparam)) {
				this.fillin.appendChild(myentry);	
			}
					
			return;
		}
		object.disabled(myentry);
		
		function fn_mouseover(e) {
			object.mouseover(e,entryobject,optparam);	
		}
		function fn_mouseout(e) {
			object.mouseout(e,entryobject,optparam);
		}
		function fn_callback(e) {
			if(object.conditionactive(entryobject,optparam)) {
				popupHide();
				object.callback(e,entryobject,optparam);
			} else if (object.callback_disabled) {
			    popupHide();
                object.callback_disabled(e,entryobject,optparam);
			}
		}

		entryobject.fn_mouseover=fn_mouseover;
		entryobject.fn_mouseout=fn_mouseout;
		entryobject.fn_callback=fn_callback;
		entryobject.disabled=object.disabled;
		entryobject.enabled=object.enabled
		entryobject.div=myentry;
		entryobject.conditionvisible=object.conditionvisible;
		entryobject.conditionactive=object.conditionactive;
		entryobject.optionalParam=object.optionalParam;
		this.entries.push(entryobject);
		object.loadingFunction(entryobject,optparam);
		this.fillin.appendChild(myentry);				
	},
	clear: function() {
		for (var i in this.entries) {
			var entryobject = this.entries[i];
			if (entryobject.fn_mouseover) { removeDOMEvent(entryobject.div, "mouseover", entryobject.fn_mouseover); }
			if (entryobject.fn_mouseout) { removeDOMEvent(entryobject.div, "mouseout", entryobject.fn_mouseout); }
			if (entryobject.fn_callback) { removeDOMEvent(entryobject.div, "click", entryobject.fn_callback); }
		}
		this.entries = new Array();
		removeChildNodes(this.div);
		this.div.appendChild(newnode("table",{ width: "160px" },{ cellPadding: 0, cellSpacing : 0, border : 0} , [
			newnode("tbody",null,null,[
				newnode("tr",null,null,[
					newnode("td",null,null,[])
				])
			])
		])
		);
		this.fillin=this.div.getElementsByTagName("td")[0];
	}	
};

var registerContext;
var unregisterContext;
var addContextNode;
var addContextLine;
var createContextMenu;
(function() {
    var contexts=new Object();
    var contextMenu = new ContextMenu();
    createContextMenu = function(type,optparam,css){
    	contextMenu.clear();
		contextMenu.div.className= (css) ? css : "PopupMenuDiv background-color-popup-menu border-color-popup-menu font-size-small";
    	if(contexts[type] && contexts[type].length>0) {
	    	for(var i=0;i<contexts[type].length;i++) {
    			var myobject=contexts[type][i];
    			contextMenu.append(myobject,optparam);
    		}
    	}
    	return contextMenu.div;
    }
	clearContext = function(type) {
		if(contexts[type]) { delete contexts[type]; }	
	}
	getContext = function(type) {
		if(type) { return contexts[type]; }
		return contexts;
	}
	registerContext = function (node,type,callback,optparam,fn_onhide,event,posObj,leave_open,cssContext,startbefore) {
		function showContentMenu(e,mynode,optparam,fn_onhide) {
			if(document.getElementById("default_context")) {
				document.getElementById("default_context").parentNode.removeChild(document.getElementById("default_context")); 
			}
			popup(mynode,null,null,function() { 
				if(fn_onhide) { fn_onhide(optparam); }
				if(window.setHoverActive) { setHoverActive(true); }
			},"contextmenu",leave_open);
			if(window.setHoverActive) {	
				setHoverActive(false);
			}
			var mychild=$("everything");
			if(!mychild) mychild=$("loading_data_complete");
			mychild.appendChild(mynode);			
			mynode.id="default_context";		
			if(posObj) {
				mynode.style.top=getAbsolutePositionTop(posObj)+posObj.offsetHeight +"px";
				mynode.style.left=getAbsolutePositionLeft(posObj)+"px";
				mynode.getElementsByTagName("table")[0].style.width=(posObj.offsetWidth-5)+"px";
			} else {
				var coorX = e.clientX;	
				var coorY = e.clientY;
				var tmpnode=mynode.cloneNode(true);
				tmpnode.style.zIndex=-1;
				tmpnode.style.display="block";
				body.appendChild(tmpnode);
				var contextheight=tmpnode.offsetHeight
				var contextwidth=tmpnode.offsetWidth;
				body.removeChild(tmpnode);
				if(parseInt(coorY + contextheight + 10) < parseInt($("body").offsetHeight)) mynode.style.top = (coorY + 10) +'px';
				else if(coorY - contextheight -10 < 0) mynode.style.top = 0 + 'px';
				else mynode.style.top = ((coorY -10 )- contextheight) + 'px';
				if((coorX + contextwidth + 10) < $("body").offsetWidth) mynode.style.left = (coorX + 10) +'px';
				else if(coorX - contextwidth - 10 < 0) mynode.style.left = 0 + 'px';
				else mynode.style.left = (coorX - contextwidth- 10) + 'px';				
			}

		}
		function myfunc() {
			return function(e) {
				if(startbefore) {
					startbefore();
				}
				//Hover not defined in Mail Detail 10365
				try {
					if(Hover.current) { Hover.current.hide(); }
				} catch (exc) {}
				var contextdiv=createContextMenu(type,optparam,cssContext);
				if(callback) {
					callback(e,contextdiv,optparam);			
				} else {
					showContentMenu(e,contextdiv,optparam,fn_onhide);
				}
				stopEvent(e);
			}
		}
		if(!event) {
			addDOMEvent(node,"contextmenu",myfunc());
		} else {
			addDOMEvent(node,event,myfunc());
		}
		return myfunc();
	}
	unregisterContext = function (node,func) {
		removeDOMEvent(node,"contextmenu",func);
	}
	addContextLine = function(entry, typearray,conditionvisible) {
		if(!entry) {
			entry= ContextMenu.makeLine();
		}
		var retobj=new Array();
		for(var i=0;i<typearray.length;i++) {
			var type=typearray[i];
			if(!contexts[type]) contexts[type]=new Array();
			var myobject=new Object();
				
			myobject.entry=entry;
			myobject.type=type;
			myobject.status="line";
			myobject.conditionvisible= (conditionvisible) ? conditionvisible : function () { return true };
			contexts[type].push(myobject);
			retobj.push(myobject);
		}
		return retobj;	
	}
	addContextNode = function(entry,typearray,callback,conditionactive,conditionvisible,setDisabled,optionalMouseOver,optionalMouseOut,loadingFunction,setEnabled,optionalParam,callback_disabled) {
		if(typearray && typearray.length>0) {
			var retobj=new Array();
			for(var i=0;i<typearray.length;i++) {
				var type=typearray[i];
				if(!contexts[type]) contexts[type]=new Array();
				var myobject=new Object();
				function defaultMouseOver(e,entrycomplete,optparam) {
					if(myobject.conditionactive(entrycomplete,optparam)) {
						entrycomplete.div.className = 'PopupEntryDivOver border-color-PMG-selection-elements';	
					}
				}
				function defaultMouseOut(e,entrycomplete,optparam) {
					if(myobject.conditionactive(entrycomplete,optparam)) {
						entrycomplete.div.className = 'PopupEntryDiv';	
					}
				}
				function defaultSetDisabled(myentry) {
					myentry.className = 'PopupEntryDivDisabled font-color-disabled';
				}
				function defaultSetEnabled(myentry) {
					myentry.className = '';
					myentry.style.cursor="pointer";	
				}
				function defaultloading(entrycomplete,optparam) {	
					entrycomplete.params=null;		
					
					if(myobject.conditionvisible(entrycomplete,optparam,true)) {	
						addDOMEvent(entrycomplete.div,'mouseover',entrycomplete.fn_mouseover);
						addDOMEvent(entrycomplete.div,'mouseout',entrycomplete.fn_mouseout);				
						addDOMEvent(entrycomplete.div,'click',entrycomplete.fn_callback);
						if(!myobject.conditionactive(entrycomplete,optparam,true)) {
							myobject.disabled(entrycomplete.div);
						} else {
							myobject.enabled(entrycomplete.div);
						}
					} else {
						entrycomplete.div.style.display="none";
					}
				}
				myobject.optionalParam=optionalParam;
				myobject.entry=entry;
				myobject.type=type;
				myobject.callback=callback || function() {};
				myobject.callback_disabled=callback_disabled;
				myobject.loadingFunction= (loadingFunction) ? loadingFunction : defaultloading;
				myobject.conditionactive=  (conditionactive) ? conditionactive : function () { return true };
				myobject.conditionvisible= (conditionvisible) ? conditionvisible : function () { return true };
				myobject.disabled=  (setDisabled) ? setDisabled   : defaultSetDisabled;
				myobject.enabled=   (setEnabled) ? setEnabled : defaultSetEnabled;
				myobject.mouseover= (optionalMouseOver) ? optionalMouseOver : defaultMouseOver;
				myobject.mouseout= (optionalMouseOut) ? optionalMouseOut : defaultMouseOut;
				contexts[type].push(myobject);
				retobj.push(myobject);
			}
			return retobj;	
		}
	}
	
})();
function contextmenu_createInputDiv(text) {
	return function() {
		var mynode = newnode("table",{ verticalAlign : "middle" , width: "100%" },{ className: "PopupEntryDiv", cellPadding : "0" , cellSpacing : "0" , border : "0"},[ 	
			newnode("tbody",null,null,[
 				newnode("tr",null,null,[
 					newnode("td",{ width : "20px" },null,[
						newnode("input",null,{ type : "checkbox" ,className: "noborder"},[])
					]),
					newnode("td",null, { className: "height16 padding7px"},[
						newnode("div",null,null, [addTranslated(text)])	 
					])
				])
	 		])
		]);
		addDOMEvent(mynode.getElementsByTagName("td")[1],"click",function() {
			mynode.getElementsByTagName("input")[0].checked=(!mynode.getElementsByTagName("input")[0].checked);
		});
	 	return mynode;	
	}
		
}

function contextmenu_createImgDiv(text,image,image_d,image_sub,image_sub_d) {
	return function() {
		var tablenode = newnode("table",{ verticalAlign : "middle" , width: "100%" },{ className: "PopupEntryDiv", cellPadding : "0" , cellSpacing : "0" , border : "0"}); 	
		var imageparentnode = newnode("div",{ height: "16px" },{});
		var image1node =newnode("img",{display:"none" , height: "16px" , width: "16px" },{ src : image , alt : ""})
 	 	imageparentnode.appendChild(image1node);
 	 	if(image_d) { 
 	 		var image2node = newnode("img", {display:"none", height: "16px" , width: "16px" }, { src : image_d , alt : "" }); 
 	 		imageparentnode.appendChild(image2node);
 	 	}
 	 	 	 	
 	 	var imgSubParent = newnode("div",null,null);
 	 	var imgSubNode = newnode("img",{display:"none"},{src: image_sub || getFullImgSrc("img/dummy.gif"), alt:"", align:"absmiddle" })
 	 	imgSubParent.appendChild(imgSubNode);
 	 	if (image_sub_d) {
 	 		var imgSubNode_d = newnode("img",{display:"none"},{src: image_sub_d, alt:"", align:"absmiddle"}); 
            imgSubParent.appendChild(imgSubNode_d);
 	 	}
 	 	
 	 	var tablebody = newnode("tbody",null,null,[
 	 		newnode("tr",null,null,[
	 		 	newnode("td",{width: "20px"},{},[imageparentnode]),
	 		 	newnode("td",null,null,[
					newnode("div",{whiteSpace: "nowrap"},{ className : "paddingleft7px"}, [addTranslated(text)])	 	 	 	
		 	 	]),
		 	 	newnode("td",{width: "20px"},{align:"right"},[imgSubParent]),
 		 	])
 		 ]);
	 	 tablenode.appendChild(tablebody); 
 		 return tablenode;
	}
}
function contextmenu_createOnlyImgDiv(image,image_d) {
	return function() {
		var tablenode = newnode("table",{ verticalAlign : "middle" , width: "100%" },{ className: "PopupEntryDiv", cellPadding : "0" , cellSpacing : "0" , border : "0"}); 	
		var imageparentnode = newnode("div",{textAlign:"center" , height:"16px" },{});
		var image1node =newnode("img",{display:"none" },{ src : image , alt : ""})
 	 	imageparentnode.appendChild(image1node);
 	 	if(image_d) { 
 	 		var image2node = newnode("img", {display:"none" , height: "16px" , width: "16px" }, { src : image_d , alt : "" }); 
 	 		imageparentnode.appendChild(image2node);
 	 	}
 	 	var tablebody = newnode("tbody",null,null,[
 	 		newnode("tr",null,null,[
	 		 	newnode("td",{ textAlign : "center" },{},[imageparentnode])
	 	 	])
 		 ]);
	 	 tablenode.appendChild(tablebody); 
 		 return tablenode;
	}
}
function contextmenu_createOnlyDiv(text) {
	return function() {
		var tablenode = newnode("table",{ verticalAlign : "middle" , width: "100%" },{ className: "PopupEntryDiv", cellPadding : "0" , cellSpacing : "0" , border : "0"}); 	
		 var tablebody = newnode("tbody",null,null,[
 	 		newnode("tr",null,null,[
	 		 	newnode("td",null,null,[
					newnode("div",{ whiteSpace : "nowrap"},null, [addTranslated(text)])	 	 	 	
		 	 	])
	 	 	])
 		 ]);
	 	 tablenode.appendChild(tablebody); 
 		 return tablenode;
	}
}

function contextmenu_inputenabled(entryobject,optparam,first) {
	if(first) {
		var path=currentpath2.join("/");
		if(dynamiccontents[currentpath2[0]]) {
			for(var i=0;i<dynamiccontents[currentpath2[0]].length;i++) {
				var el=dynamiccontents[currentpath2[0]][i];
				if(path.indexOf(el.view)==0) {
					if(el.display!="none") {
						if(!entryobject.div.getElementsByTagName("input")[0].checked) {
							entryobject.div.getElementsByTagName("input")[0].checked= entryobject.div.getElementsByTagName("input")[0].defaultChecked = true;
						}
					} else {
						if(entryobject.div.getElementsByTagName("input")[0].checked) {
							entryobject.div.getElementsByTagName("input")[0].checked= entryobject.div.getElementsByTagName("input")[0].defaultChecked = false;
						}
					}
					break;
				}
			}
		}
	}
	return true;
}
function contextmenu_setDisabled(myentry) {
	myentry.className = 'PopupEntryDivDisabled font-color-disabled';
	var enadiv=myentry.getElementsByTagName("img")[0];
	var disdiv=myentry.getElementsByTagName("img")[1];
	if(enadiv) { if(enadiv.style.display!="none") { enadiv.style.display="none"; } }
	if(disdiv) { if(disdiv.style.display!="inline") { disdiv.style.display="inline"; }}
	
    var enadiv_sub=myentry.getElementsByTagName("img")[2];
    var disdiv_sub=myentry.getElementsByTagName("img")[3];
    if(enadiv_sub) { if(enadiv_sub.style.display!="none") { enadiv_sub.style.display="none"; } }
    if(disdiv_sub) { if(disdiv_sub.style.display!="inline") { disdiv_sub.style.display="inline"; }}
}
function contextmenu_setActive(myentry) {
	myentry.className = 'PopupEntryDivActive border-color-PMG-selection-elements';
	var enadiv=myentry.getElementsByTagName("img")[0];
	var disdiv=myentry.getElementsByTagName("img")[1];
	if(enadiv) { if(enadiv.style.display!="none") { enadiv.style.display="none"; } }
	if(disdiv) { if(disdiv.style.display!="inline") { disdiv.style.display="inline"; }}
	
    var enadiv_sub=myentry.getElementsByTagName("img")[2];
    var disdiv_sub=myentry.getElementsByTagName("img")[3];
    if(enadiv_sub) { if(enadiv_sub.style.display!="none") { enadiv_sub.style.display="none"; } }
    if(disdiv_sub) { if(disdiv_sub.style.display!="inline") { disdiv_sub.style.display="inline"; }}	
} 
function contextmenu_setEnabled(myentry) {
	myentry.className = 'PopupEntryDiv';
	myentry.style.cursor="pointer";	
	var enadiv=myentry.getElementsByTagName("img")[0];
	var disdiv=myentry.getElementsByTagName("img")[1];
	if(enadiv) { if(enadiv.style.display!="inline") { enadiv.style.display="inline"; } }
	if(disdiv) { if(disdiv.style.display!="none") { disdiv.style.display="none"; }}

    var enadiv_sub=myentry.getElementsByTagName("img")[2];
    var disdiv_sub=myentry.getElementsByTagName("img")[3];
    if(enadiv_sub) { if(enadiv_sub.style.display!="inline") { enadiv_sub.style.display="inline"; } }
    if(disdiv_sub) { if(disdiv_sub.style.display!="none") { disdiv_sub.style.display="none"; }}
}
fileloaded();
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2007 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Andreas Mayer <andreas.mayer@open-xchange.com>
 * 
 */

  //////////////
 //   Tree   //
//////////////

/**
 * Creates a tree widget inside the specified DOM element.
 * @class A basic tree widget.
 * @param {Object} container The DOM element to which the root nodes of the tree
 * @param {Object] state Optional state object previously retrieved with
 * {@link getState}.
 * will be added.
 */
function Tree(container, state, id) {
	/**
	 * @private
	 */
	this.container = container;

	/**
	 * @private
	 */
	this.state = state || new Object();
	
	/**
	 * @private
	 */
	this.children = {};
	
	/**
	 * @private
	 */
	this.id = id;
	
	/**
	 * @private
	 */
	this.hasChildren = false;
	this.events = new Events();
}

Tree.prototype = {
	/**
	 * Removes a previously added root node from the tree.
	 * @param {TreeNode} node The root node to remove.
	 */
	removeChild: function(node) {
		delete this.children[node.id];
		delete this.state[node.id];
		this.container.removeChild(node.div);
	},
	
	/**
	 * Returns the open/colsed state of all nodes in the tree as a JavaScript
	 * object. Returns the original state if no root nodes were added.
	 * @type Object 
	 * @return A state object which can be supplied to the constructor later.
	 */
	getState: function() {
		if (this.hasChildren) {
			this.state = new Object();
			for (var i in this.children) {
				var childState = this.children[i].getState();
				if (childState) this.state[i] = childState;
			}
		}
		return this.state;
	},
	
	/**
	 * @private
	 */
	setState: function(state) {
		for(var i in this.state)	
		{
			if(this.children[i] && this.state[this.children[i].id])
			{	
				this.children[i].setState(this.state[this.children[i].id]);
			}
		}
	},
	replaceChild: function(node,oldnode) {
		this.container.replaceChild(node.div,oldnode.div);
		node.hasChildren = oldnode.hasChildren;
		node.tree_id = oldnode.tree_id;
		node.tree = oldnode.tree;
		this.children[node.id] = node;			
		if (oldnode.open) 
		{				
			node.open = true;			
			node.setOpen(true);
		}
	},
	/**
	 * @private
	 */
	addChild: function(node) {
		this.hasChildren = true;
		node.tree_id = this.id;
		node.tree = this;
		this.children[node.id] = node;
		if (this.state[node.id]) 
		{		
			node.setState(this.state[node.id]);			
			node.setOpen(true);
		}
		this.container.appendChild(node.div);		
	},
	setOpen:function ()
	{
		
	}
};

/**
 * Adds a root node to a tree or a child node to another node.
 * The new node is appended as the last immediate child of the specified parent
 * object.
 * @class A node for the tree widget.
 * @param {Object} parent Either a Tree or a TreeNode object to which the new
 * node will be added.
 * @param {Boolean} hasChildren Specifies whether to display a plus sign before
 * the node. For trees which are built on-demand, this parameter can be used to
 * specify whether the node has children without actually adding any. If this
 * parameter is false and a child is added later, the plus sign is displayed
 * automatically.
 * @param {String} colsedIcon URI of the icon to display when the children are
 * not displayed.
 * @param {String} openIcon URI of the icon to display when the children are
 * displayed.
 * @param {String or Object} text Text to display in the node as a string or as
 * an inline-level DOM node.
 * @param {String} id Unique ID of the node. Used to restore state.
 * @param {Object} data Any additional data to store in the node. the data is
 * later accessible via {@link #data}.
 */
function TreeNode(parent, hasChildren, closedIcon, openIcon, text, id, data,afterlabel) {
	/**
	 * Specifies whether this node has children.
	 */
	this.hasChildren = hasChildren;
	this.link = false;
	/**
	 * URI of the icon to display when the children are not displayed.
	 */
	this.closedIcon = closedIcon;

	/**
	 * URI of the icon to display when the children are displayed.
	 */
	this.openIcon = openIcon;
	
	/**
	 * Unique ID of the node.
	 */
	this.id = id;
	
	/**
	 * Additional data specified as the last parameter of the constructor.
	 */
	this.data = data;
	
	/**
	 * Specifies whether the children are displayed.
	 */
	this.open = false;
	
	/**
	 * @private
	 */
	this.children = {};
	
	/**
	* @parent node
	*/
	this.parent = parent;
	/**
	 * @private
	 */
	this.count = 0;

	this.events = new Events();
	
	/**
	 * @private
	 */
	this.state = new Object();
	this.tree_id;	
	this.tree;		

	var afterlabeltext = afterlabel;
	/**
	 * This and further fields can be used to customize the behaviour of
	 * the tree.
	 */

	this.table_Node = newnode("table",{borderCollapse:"collapse",borderWidth:"0",borderSpacing:"0",width:"100%"}
									,0,[
							newnode("tbody",{width:"100%"},0,[
										newnode("tr",{width:"100%"},0,[
													newnode("td",0,0,[
		newnode("div",0,0,[this.plus = newnode("img", 0, {className: hasChildren ? "tree-children" : "tree-nochildren",
				src: hasChildren ? getFullImgSrc("img/plus.gif") : getFullImgSrc("img/noplus.gif")})])
																		]),
													newnode("td",0,0,[
		newnode("div",0,0,[this.icon = newnode("img", 0,
				{className: "tree-icon", src: this.closedIcon})])]),
													newnode("td",{paddingLeft:"0",width:"100%"},0,[				
		this.text = newnode("div", 0, {className: "tree-text border-color-content-default"}, [
				this.textnode = typeof(text) != "string" ? text
					: this.textnode = document.createTextNode(text),											
					this.afterLabel = newnode("span", 0, {className: "tree-afterlabel"}, [
							this.afterlabel_textnode = typeof(afterlabeltext) != "string" ? afterlabeltext
								: this.afterlabel_textnode = document.createTextNode(afterlabeltext)])
					])				
													/*, newnode("td",0,0,[						
		this.afterLabel = newnode("span", 0, {className: "tree-afterlabel"}, [
				this.afterlabel_textnode = typeof(afterlabeltext) != "string" ? afterlabeltext
					: this.afterlabel_textnode = document.createTextNode(afterlabeltext)])]) */
																			])
		
																	])																																								

								  							])
								  							
													]);

	this.div = newnode("div", 0, {className: "tree-div"}, [
		this.line = newnode("span", 0, {className: "tree-line"}, [
			this.table_Node
			,this.container = newnode("div", {marginLeft: "18px", whiteSpace: "nowrap", display: "none"})])]);
	var Self = this;
	
	addDOMEvent(this.plus, "click", function(e) {
		Self.setOpen(!Self.open);
		cancelDefault(e);		
		Self.tree.events.trigger("OX_Tree_State_Changed",Self.tree_id);		
	});

	addDOMEvent(this.icon, "click", function(e) {
		cancelDefault(e);
		Self.tree.events.trigger("OX_Tree_Node_Click",Self,Self.tree_id);
	});

	addDOMEvent(this.text, "click", function(e) {
		cancelDefault(e);
		Self.tree.events.trigger("OX_Tree_Node_Click",Self,Self.tree_id);
	});
	
}

TreeNode.prototype = {
	/**
	 * Returns the current text of the node.
	 * @type String or Object
	 * @return The current text of the node as a string or DOM node.
	 */
	getText: function() {
		return this.textnode.nodeType == 3 ? this.textnode.data : this.textnode;
	},
	
	/**
	 * Sets the text of the node.
	 * @param {String or Object} text The new text of the node as a string or
	 * as an inline-level DOM node.
	 */
	setText: function(text) {
		if (typeof(text) != "string") {
			this.text.replaceChild(text, this.textnode);
			this.textnode = text;
		} else if (this.textnode.nodeType == 3)
		{
			this.textnode.data = text;
		}
		else {
			var textNode = document.createTextNode(text);
			this.text.removeChild(this.textnode);
			this.text.appendChild(textNode);
			this.textnode = textNode;
		}
	},

	updateIcon: function () { this.icon.src = open ? this.openIcon : this.closedIcon;},
	
	/**
	 * Removes a previously added child node from the tree.
	 * @param {TreeNode} node The child node to remove.
	 */
	removeChild: function(node) {
		if (!this.children[node.id]) return;		
		if (this.children[node.id].tree_id != this.tree_id) return;				
		if (!--this.count) {
			this.open = false;
			this.hasChildren = false;
			this.plus.src = getFullImgSrc("img/noplus.gif");
		}
		delete this.children[node.id];
		if(this.state!=undefined)
			delete this.state[node.id];
		this.container.removeChild(node.div);
	},
	
	/**
	 * Displays or hides all children.
	 * @param {Boolean} open Specifies whether to display children.
	 */
	setOpen: function(open,fn_opt) {
		var Self = this;
		if(open && !this.open && this.hasChildren)
		{
			this.tree.events.trigger("OX_Tree_Expand",Self,fn_opt);
		}		
		else if (fn_opt) 
				fn_opt();
		this.open = open;
		if(open)
			this.events.trigger("NodeOpen",Self);
		else
			this.events.trigger("NodeClose",Self);		
		this.plus.src = this.hasChildren ? (open ? getFullImgSrc("img/minus.gif")
		                                         : getFullImgSrc("img/plus.gif"))
		                                 : getFullImgSrc("img/noplus.gif");
		this.plus.className = this.hasChildren ? "tree-children" : "tree-nochildren";
		this.icon.src = open ? this.openIcon : this.closedIcon;
		this.container.style.display = open ? "block" : "none";
	},
	
	
	/**
	 * @private
	 */
	addChild: function(node) {
		if (!this.count++) {
			this.hasChildren = true;
			this.plus.src = this.open ? getFullImgSrc("img/minus.gif") : getFullImgSrc("img/plus.gif");
		}
		node.tree_id = this.tree_id;
		node.tree = this.tree;		
		this.children[node.id] = node;		
		if (this.state!=undefined && this.state[node.id])
		{
			node.setState(this.state[node.id]);	
			node.setOpen(true);
		}
		this.container.appendChild(node.div);
	},
	replaceChild: function(node,oldnode) {
		try {
		  this.container.replaceChild(node.div,oldnode.div);
		} catch (e) {
            // for some reason this sometimes leads into an error
        }
		node.tree_id = oldnode.tree_id;
		node.tree = oldnode.tree;	
		node.parent = this;
		this.children[node.id] = node;
	},
	/**
	 * @private
	 */
	setState: function(state) {
		this.state = state;
		for(var i in this.state)	
		{
			if(this.children[i] && this.state[this.children[i].id])
			{				
				this.children[i].setState(this.state[this.children[i].id]);
				this.children[i].setOpen(true);												
			}
		}
	},
	
	/**
	 * @private
	 */
	getState: function() {
		if (this.count || !this.hasChildren) {
			this.state = new Object();
			for (var i in this.children) {
				var childState = this.children[i].getState();
				if (childState) this.state[i] = childState;
			}
		}
		return this.open ? this.state : undefined;
	}
};
fileloaded();
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2007 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Suphi Basdemir <suphi.basdemir@open-xchange.com>
 * 
 */

var globalclickdisable2=true;
var activeComboBox = null;

function changePopUpMenuText3(obj, n,noevent)
{
	var myValues = new Array();
	myValues = obj.elements[n];
	
	obj.currentI18nNode.callback = myValues.callback;
	obj.currentI18nNode.update();
        	
    obj.selected = n;
 	var key =  myValues.key;
    
	if(obj.targetDiv) {
		if(obj.form) {
			obj.targetDiv.value = key;
		} else {
			if(!obj.valueIsHTML) {
				if(obj.targetDiv && obj.targetDiv.firstChild)
				obj.targetDiv.firstChild.data = key;
			}
			else {
				removeChildNodes(obj.targetDiv);
				if(key)
					obj.targetDiv.appendChild(key);
			}
				
		}		
	}
    if (obj.onChange && !noevent) obj.onChange(key);
//	if(obj.eventname && obj.eventname != "" && !noevent) 
//		triggerEvent(obj.eventname, key, obj.eventparam);
		
	obj.currentResult = key;
	obj.myboxComboDiv.style.display="none";
	activeComboBox = null;
}

/**
 * Make ComboBox
 * @param {object} current Window where the combobox is called
 * @param {String} id of rootDiv, to append the combobox
 * @param {String} width from the combobox
 * @param {int} index of selected item from the elements
 * @param {boolean} default Style, should be used ore not
 * @param {String} eventname to trigger an event onChange
 * @param {int} scrollbar should be shown after n items
 * example:
 * new ComboBox3(window, "start_time", "15em", myElements, 0, false, changetime, 6);
 */

function ComboBox3(window, elementid, width, selected, style, eventname, maxscrollbar,eventParam) 
{
    return new ComboBox3.impl(window, $ALL(elementid), width, selected, style,
        maxscrollbar,
        function(key) { triggerEvent(eventname, key, eventParam); } );
}

ComboBox3.impl = function(window, rootDiv, width, selected, style, maxscrollbar, onchange, onshow, onhide)
{
	this.window = window;
	this.selected = selected;
	this.elements = new Array();
	this.width=width;
	this.maxscrollbar=maxscrollbar;
	this.style = style;
	this.targetDiv = null;
	this.valueIsHTML = false; //the value is a node
	this.form = false;
	this.viewValue = false;
	this.disabled = false;		
	this.rowHeight;	
	this.childOffsetTop;
	this.childOffsetTopValues = new Array();
	var Self = this;
	this.rootDiv = rootDiv;		
	this.rootDiv.tabIndex = 0;
	this.rootDiv.style.outline = "none";
	this.keydown_cb = function(e) {
		return Self.key(e);
	};
	addDOMEvent(this.rootDiv, "keydown", this.keydown_cb);
	this.rootDiv.innerHTML = "";
	this.headTable = newnode('table',0,{
			         cellPadding: 0, cellSpacing: 0, border: 0});
	this.headTable.style.tableLayout="fixed";
	this.headTableBody = document.createElement("tbody");
	this.headTableRow = document.createElement("tr");
	this.headTableColumn1 = document.createElement("td");	
	this.headTableColumn1.setAttribute("nowrap","nowrap");
	this.headTableColumn1.style.whiteSpace = "nowrap";
	this.headTableColumn1div = document.createElement("div");	
	this.headTableColumn1.appendChild(this.headTableColumn1div);
	this.headTableColumn1div.style.width = "92%";	
	this.headTableColumn1div.style.paddingRight = "3px";		
	this.headTableColumn1div.style.overflow = "hidden";	
	this.headTableColumn2 = document.createElement("td");
	this.headTableColumn2.setAttribute("width","14");
	this.myboxComboDiv = document.createElement("div");
	this.keySearch = {};
	
	if(this.style) {
		this.headTable.className="smallpopupborder border-color-design smallpopupnotactive background-color-content font-color-default";
		this.headTableColumn1.className="smallpopupborderright border-color-design comboboxColumn1";
		this.headTableColumn2.className="comboboxColumn2 borderLeftDefault border-color-design";
	}
	this.rootDiv.style.width=this.width;
	this.rootDiv.className="dropdownbox";
	
	this.currentValues = new Array();
	
	this.currentResult = "";
	
    this.onChange = onchange;
    this.onShow = onshow;
    this.onHide = onhide;
		
	if(maxscrollbar)	
		this.scrollbar=maxscrollbar;
	else 
		this.scrollbar=null;
		
	this.setFirstDiv();
	
	
}
ComboBox3.impl.prototype = {
	sort: function ()
	{
		function fn(a,b)
		{
			if(a.value < b.value)
				return -1;
			else
				return 1;
		}
		this.elements.sort(fn);
	},
	sorti18nValue: function ()
	{
		function fn(a,b)
		{
			//if(a.i18nNode.node.nodeValue < b.i18nNode.node.nodeValue)
			if(a.key == "" ) 
				return -1;
			else if(b.key == "" ) 
				return 1;
			else
				return a.i18nNode.node.nodeValue.localeCompare( b.i18nNode.node.nodeValue ) ;
		}
		this.elements.sort(fn);
	},
	sortKeys: function ()
	{
		function fn(a,b)
		{
			if(a.key < b.key)
				return -1;
			else
				return 1;
		}
		this.elements.sort(fn);
	},
	getValue : function()
	{
		this.currentValues = this.elements[this.selected];
		this.currentValue = this.currentValues.value;
		return this.currentValue;
	} ,
	setEnabled : function(value) {
		//TODO DISABLED;
	} ,
	getKey : function()
	{
		this.currentValues = this.elements[this.selected];
		this.currentResult = this.currentValues.key;
		return this.currentResult;
	} , 
	getSelectedIndex : function()
	{
		return this.selected;
	} ,
	setSelectedIndex : function(n, noevent)
	{
		changePopUpMenuText3(this, n, noevent);
	},
	disable : function() {
		var Self = this;
		Self.headTable.className = Self.headTable.className.replace(/font-color-default/, "font-color-disabled");
		Self.disabled = true;
		Self.headTableColumn2.style.cursor = "default";
		//Self.window.removeDOMEvent(Self.headTable,"click", Self.clickOnCombobox);
	},
	enable : function() {
		var Self = this;
		Self.disabled = false;			
		Self.headTable.className = Self.headTable.className.replace(/font-color-disabled/, "font-color-default");
		Self.headTableColumn2.style.cursor = "pointer";
		//Self.window.addDOMEvent(Self.headTable,"click", Self.clickOnCombobox);  
	},
	setKey : function(key,noevent)
	{
		var myElValues = new Array();
		var newIndex = key;
		var bFound = false;
		for(i=0; i<this.elements.length;i++) {
			myElValues = this.elements[i];
   			 	if (key == myElValues.key) {
   			 		key = myElValues.key;
   			 		newIndex = i;
   			 		bFound = true;
   			 		break;
   			 	}       		
		}
		if(!bFound)
			return false;
		if(this.elements[newIndex]) {
			changePopUpMenuText3(
					this,
					newIndex,
					noevent
			);
		}					
		return true;		
	},	
	setValue : function(value,noevent)
	{
		var myElValues = new Array();
		var newIndex = value;
		var bFound = false;
		for(i=0; i<this.elements.length;i++) {
			myElValues = this.elements[i];
			
			if(myElValues.i18nNode && myElValues.i18nNode.node && myElValues.i18nNode.node.nodeValue)
				myi18nValue = myElValues.i18nNode.node.nodeValue;
			else
				myi18nValue = this.elements[i];
				
		 	if (value == myElValues.value || value == myi18nValue) {
		 		value = myElValues.value;
		 		newIndex = i;
		 		bFound = true;
		 		break;
		 	} 		
		}
		if(!bFound)
			return false;
		if(this.elements[newIndex]) {
			changePopUpMenuText3(
					this,
					newIndex,
					noevent
			);
		}					
		return true;		
	},	
	setUnknownKey : function(key) {					
		this.targetDiv.firstChild.data = key;
	},
	scrollToSelected : function(tmpIndex) {
		if(this.scrollbar) {
				
			if(tmpIndex || tmpIndex == 0)
				var gotoItem = tmpIndex;
			else
				var gotoItem = this.selected;
			var Self = this;
			if(this.childOffsetTopValues[gotoItem] || this.childOffsetTopValues[gotoItem] == 0) {
				window.setTimeout(function() {
					Self.myboxComboDiv.scrollTop = Self.childOffsetTopValues[gotoItem];
				},0);
			}
			else
				this.myboxComboDiv.scrollTop = (gotoItem * this.rowHeight);
		}			
	},
	
	key: function(e) {
		switch (e.keyCode || e.which) {
		case 13: // enter
			break;
		case 32: // spacebar
			break;
		case 38: // arrow up
			break;
		case 40: // arrow down
			break;
		case 65: // A
		case 97: // a		
			if(this.keySearch["a"])
			this.scrollToSelected(this.keySearch["a"][0]);
			break;
		case 66: // B
		case 98: // b	
			if(this.keySearch["b"])	
			this.scrollToSelected(this.keySearch["b"][0]);
			break;
		case 67: // C
		case 99: // c	
			if(this.keySearch["c"])	
			this.scrollToSelected(this.keySearch["c"][0]);
			break;
		case 68: // D
		case 100: // d		
			if(this.keySearch["d"])
			this.scrollToSelected(this.keySearch["d"][0]);
			break;
		case 69: // E
		case 101: // e		
			if(this.keySearch["e"])
			this.scrollToSelected(this.keySearch["e"][0]);
			break;
		case 70: // F
		case 102: // f		
			if(this.keySearch["f"])
			this.scrollToSelected(this.keySearch["f"][0]);
			break;
		case 71: // G
		case 103: // g		
			if(this.keySearch["g"])
			this.scrollToSelected(this.keySearch["g"][0]);
			break;
		case 72: // H
		case 104: // h		
			if(this.keySearch["h"])
			this.scrollToSelected(this.keySearch["h"][0]);
			break;
		case 73: // I
		case 105: // i		
			if(this.keySearch["i"])
			this.scrollToSelected(this.keySearch["i"][0]);
			break;
		case 74: // J
		case 106: // j		
			if(this.keySearch["j"])
			this.scrollToSelected(this.keySearch["j"][0]);
			break;
		case 75: // K
		case 107: // k		
			if(this.keySearch["k"])
			this.scrollToSelected(this.keySearch["k"][0]);
			break;
		case 76: // L
		case 108: // l		
			if(this.keySearch["l"])
			this.scrollToSelected(this.keySearch["l"][0]);
			break;
		case 77: // M
		case 109: // m		
			if(this.keySearch["m"])
			this.scrollToSelected(this.keySearch["m"][0]);
			break;
		case 78: // N
		case 110: // n		
			if(this.keySearch["n"])
			this.scrollToSelected(this.keySearch["n"][0]);
			break;
		case 79: // O
		case 111: // o	
			if(this.keySearch["o"])	
			this.scrollToSelected(this.keySearch["o"][0]);
			break;
		case 80: // P
		case 112: // p		
			if(this.keySearch["p"])
			this.scrollToSelected(this.keySearch["p"][0]);
			break;
		case 81: // Q
		case 113: // q		
			if(this.keySearch["q"])
			this.scrollToSelected(this.keySearch["q"][0]);
			break;
		case 82: // R
		case 114: // r		
			if(this.keySearch["r"])
			this.scrollToSelected(this.keySearch["r"][0]);
			break;
		case 83: // S
		case 115: // s		
			if(this.keySearch["s"])
			this.scrollToSelected(this.keySearch["s"][0]);
			break;
		case 84: // T
		case 116: // t		
			if(this.keySearch["t"])
			this.scrollToSelected(this.keySearch["t"][0]);
			break;
		case 85: // U
		case 117: // u		
			if(this.keySearch["u"])
			this.scrollToSelected(this.keySearch["u"][0]);
			break;
		case 86: // V
		case 118: // v		
			if(this.keySearch["v"])
			this.scrollToSelected(this.keySearch["v"][0]);
			break;
		case 87: // W
		case 119: // w		
			if(this.keySearch["w"])
			this.scrollToSelected(this.keySearch["w"][0]);
			break;
		case 88: // X
		case 120: // x		
			if(this.keySearch["x"])
			this.scrollToSelected(this.keySearch["x"][0]);
			break;
		case 89: // Y
		case 121: // y		
			if(this.keySearch["y"])
			this.scrollToSelected(this.keySearch["y"][0]);
			break;
		case 90: // Z
		case 122: // z		
			if(this.keySearch["z"])
			this.scrollToSelected(this.keySearch["z"][0]);
			break;
		default:
			return;
		}
		stopEvent(e);
		return false;
	},
	
	setFirstDiv : function()
	{
		var Self = this;
						
		Self.clickOnCombobox = function (e) { 
			if(!Self.disabled)
				showHideBox3(Self,e);
			stopEvent(e);
		}
		Self.headTable.style.width=this.width;	
		setTimeout(function() {
			var tmpW = Self.headTable.offsetWidth - 16;
				
			Self.headTableColumn2.style.width="16px";
			Self.headTableColumn2.style.textAlign="center";
		
			Self.image=document.createElement("img");
			Self.image.className="cursorPointer";
			Self.image.setAttribute("src",getFullImgSrc("img/arrows/arrow_gray_down.gif"));
			Self.headTableColumn2.appendChild(Self.image);
			Self.headTableRow.appendChild(Self.headTableColumn1);	
			Self.headTableRow.appendChild(Self.headTableColumn2);
			Self.headTableBody.appendChild(Self.headTableRow);
			Self.headTable.appendChild(Self.headTableBody);
   			 
   			Self.window.addDOMEvent(document.getElementById("body"),
   			 "click", hideComboBox3); 		   			    			
			
			Self.window.addDOMEvent(Self.headTable,
   			 "click", Self.clickOnCombobox);  
   			 
   			Self.window.addDOMEvent(Self.headTable,
   			 "mouseover", function() { globalclickdisable2=true; }); 
   			 
   			Self.window.addDOMEvent(Self.headTable,
   			 "mouseout", function() { globalclickdisable2=false; });  
		}, 0);		
		
		Self.rootDiv.appendChild(Self.headTable);
		
	} ,
	
	clickOnCombobox : function(e) { 
		showHideBox3(this,e) 
	},
	
	addElement: function(value, key, callback) {
		if(typeof value == "string") {
			if(value.length > 0) {
				var indexKeySearchSingle = value.substr(0,1).toLowerCase();
				if(typeof this.keySearch[indexKeySearchSingle] == "undefined") this.keySearch[indexKeySearchSingle] = new Array();
				this.keySearch[indexKeySearchSingle].push(this.elements.length);
			}
			/*
			 * for later implemantation
			if(key.length > 1) {
				var indexKeySearchDuo = key.substr(0,2).toLowerCase();
				if(typeof this.keySearch[indexKeySearchDuo] == "undefined") this.keySearch[indexKeySearchDuo] = new Array();
				this.keySearch[indexKeySearchDuo].push(this.elements.length);
			}
			*/
		}
		this.elements.push({
			value: value,
			key: key,
			callback: callback || function() { return _(value); }
		});
	},
	
	getDomNode : function(sortedAfterTranslation) {	
		
		while(this.myboxComboDiv.childNodes.length>0)		
			this.myboxComboDiv.removeChild(this.myboxComboDiv.firstChild);

		this.currentValues = this.elements[this.selected];
		this.currentValue = this.currentValues.value;
		this.currentResult = this.currentValues.key;	
		while(this.headTableColumn1div.childNodes.length>0)		
			this.headTableColumn1div.removeChild(this.headTableColumn1div.firstChild);			
		this.currentI18nNode = new I18nNode(this.currentValues.callback);
		this.headTableColumn1div.appendChild(this.currentI18nNode.node);

		var Self = this;			
		Self.myboxComboDiv.className="smallpopupborder smallpopuplist background-color-content font-weight-default font-color-default border-color-design";
		Self.myboxComboDiv.style.display="none";
		Self.myboxComboDiv.style.textAlign="left";
        Self.myboxComboDiv.style.whiteSpace = "nowrap";	
		
		for(var el1 in Self.elements)
		{
			(function(n) {
				var myValues = new Array();           			 			 
       			myValues = Self.elements[n];       			 
				myValues.i18nNode = new I18nNode(myValues.callback);				
			})(el1);
		}
		
		if(sortedAfterTranslation) {
			this.sorti18nValue();	
       		Self.keySearch = {};
       		Self.indexForKeySearch = 0;		
		}
		
		for(var el in Self.elements)
		{
			(function(n) {
				var childDiv = null;
				var myValues = new Array();
				childDiv = newnode("div",0,{className:"height16 background-color-content"});
				childDiv.style.overflow="hidden";
				childDiv.style.paddingLeft="4px";
				childDiv.style.paddingTop="3px";
				childDiv.style.width="97%";
				Self.window.addDOMEvent(childDiv,
       			 "click", function() { 
	 						changePopUpMenuText3(Self,n);	
	 						Self.rootDiv.blur();          			 						
 						}
       			);  	 
       			
				Self.window.addDOMEvent(childDiv,
       			 "mouseover", function() { changePopUpMenuStatus3(childDiv, true, Self) });           			 
       			 
				Self.window.addDOMEvent(childDiv,
       			 "mouseout", function() { changePopUpMenuStatus3(childDiv, false, Self) });      
       			 			 
       			 myValues = Self.elements[n];
       			 
       			 if(sortedAfterTranslation) {
					if(typeof myValues.i18nNode.node.nodeValue == "string" && myValues.value != "please select") {
						if(myValues.i18nNode.node.nodeValue.length > 0) {
							var indexKeySearchSingle = myValues.i18nNode.node.nodeValue.substr(0,1).toLowerCase();
							if(typeof Self.keySearch[indexKeySearchSingle] == "undefined") Self.keySearch[indexKeySearchSingle] = new Array();
							Self.keySearch[indexKeySearchSingle].push(Self.indexForKeySearch);
						}
					}
					Self.indexForKeySearch++;
       			 }
       			 
       			 if(Self.viewValue) {
       			 	if(myValues.key != "")
						childDiv.className="height16 font-color-default background-color-content";
       			 	else					
						childDiv.className="height16 font-color-disabled background-color-content";
       			}
				myValues.i18nNode = new I18nNode(myValues.callback);
				childDiv.appendChild(myValues.i18nNode.node);
				
				if(!Self.rowHeight) {
					var tmpnode=childDiv.cloneNode(true);
					tmpnode.style.display="block";
					tmpnode.style.zIndex=-1;
					body.appendChild(tmpnode);				
					Self.rowHeight=tmpnode.offsetHeight;
					body.removeChild(tmpnode);
				}
       			 
				Self.myboxComboDiv.appendChild(childDiv);
			})(el);
		}
		if(this.scrollbar && this.scrollbar != "" && typeof this.scrollbar != "undefined") {
			var tempHeight = Math.floor(this.scrollbar*this.rowHeight);
			Self.myboxComboDiv.style.height=tempHeight+"px";
		}
		Self.rootDiv.appendChild(Self.myboxComboDiv);
		if(this.maxscrollbar)
			Self.myboxComboDiv.style.overflowX="hidden";
		else
			Self.myboxComboDiv.style.overflow="hidden";
			
		var tmpnode2=Self.myboxComboDiv.cloneNode(true);
		tmpnode2.style.display="block";
		tmpnode2.style.zIndex=-1;
		body.appendChild(tmpnode2);
		for (var j = 0; j < tmpnode2.childNodes.length; j++) {
			this.childOffsetTopValues[j] = tmpnode2.childNodes[j].offsetTop;
		}
		body.removeChild(tmpnode2);
		
	}
}
function ComboBoxSpecial(window, elementid, width, selected, style, eventname, maxscrollbar,eventParam) 
{
	this.window = window;
	this.selected = selected;
	this.elements = new Array();
	this.width=width;
	this.elementid=elementid;
	this.style = style;
	this.targetDiv = null;
	this.valueIsHTML = false; //the value is a node
	this.form = false;
	this.viewValue = false;
	this.disabled = false;			
	this.optWidth;
	this.show_buttons = false;
	this.special = true;
	this.childOffsetTopValues = [];
	this.keySearch = {};
	
	this.rootDiv = $ALL(elementid);		
	this.rootDiv.innerHTML = "";
	this.headTable = newnode('table',0,{
					 cellPadding: 0, cellSpacing: 0, border: 0});
					 
	this.headTable.style.tableLayout="fixed";
	this.headTableBody = document.createElement("tbody");
	this.headTableRow = document.createElement("tr");
	this.headTableColumn1 = document.createElement("td");	
	this.headTableColumn1.setAttribute("nowrap","nowrap");
	this.headTableColumn1.style.whiteSpace = "nowrap";
	this.headTableColumn1div = newnode("div",{textAlign:"center"});	
	this.headTableColumn1.appendChild(this.headTableColumn1div);
	this.headTableColumn1div.style.overflow = "hidden";	
	this.myboxComboDiv = newnode("div");

	if(this.style) {
		this.headTable.className="smallpopupborder smallpopupnotactive background-color-content font-color-default";
		this.headTableColumn1.className="smallpopupborderright border-color-design comboboxColumn1";
	}
	this.rootDiv.style.width=this.width;
	this.rootDiv.className="dropdownbox";
	
	this.currentValues = new Array();
	
	this.currentResult = "";
	
	if(eventname)
        this.onChange = function(key) {
            triggerEvent(eventname, key, eventParam);
        };
	
	if(maxscrollbar)
		this.scrollbar=maxscrollbar;
	else 
		this.scrollbar=null;
		
	this.setFirstDiv();

//extends.. Combobox3
	this.scrollToSelected = ComboBox3.impl.prototype.scrollToSelected;	
	this.setKey = ComboBox3.impl.prototype.setKey;		
	this.addElement = ComboBox3.impl.prototype.addElement;	
	this.sort = ComboBox3.impl.prototype.sort;		
	
}
ComboBoxSpecial.prototype = {
	setDynamicKey : function(key,noevent)
	{
		this.elements = new Array();
		for(var nIndx=-5;nIndx< 5;nIndx++)
		{
			this.addElement((nIndx+key),(nIndx+key));
		}
		this.getDomNode();		

		var myElValues = new Array();
		var newIndex = key;
		var bFound = false;
		for(i=0; i<this.elements.length;i++) {
			myElValues = this.elements[i];
   			 	if (key == myElValues.key)	
   			 	{
   			 		key = myElValues.key;
   			 		newIndex = i;
   			 		bFound = true;
   			 		break;
   			 	}       		
		}
		if(!bFound)
			return false;
		if(this.elements[newIndex]) {
			changePopUpMenuText3(
					this,
					newIndex,
					noevent
			);
		}					
		return true;		
	},		
	setFirstDiv:function (){
		var Self = this;
								
		Self.clickOnCombobox = function (e) { 
					if(!Self.disabled)
						showHideBox3(Self,e);
					stopEvent(e);
		}
		Self.headTable.style.width=this.width;	
		setTimeout(function() {
			var tmpW = Self.headTable.offsetWidth - 16;
			Self.headTableRow.appendChild(Self.headTableColumn1);	
			Self.headTableBody.appendChild(Self.headTableRow);
			Self.headTable.appendChild(Self.headTableBody);
			 
			Self.window.addDOMEvent(document.getElementById("body"),
			 "click", function() { 
                    hideComboBox3();                    
                    if (Self.onHide) Self.onHide();
                }); 
			    			
			
			Self.window.addDOMEvent(Self.headTable,
			 "click", Self.clickOnCombobox);  
			 
			 Self.window.addDOMEvent(Self.headTable,
			 "mouseover", function() { globalclickdisable2=true; }); 
			 
			 Self.window.addDOMEvent(Self.headTable,
			 "mouseout", function() { globalclickdisable2=false; });  
		}, 0);		
		
		Self.rootDiv.appendChild(Self.headTable);		
	},	
	scrollElements : function (sDirection){
		if(sDirection == "up")
		{
			var nOldVal = this.elements[0].value-10;
			this.elements = new Array();
			for(var nIndx=0;nIndx< 10;nIndx++)
			{
				this.addElement((nOldVal+nIndx),(nOldVal+nIndx));
			}
			this.getList();				
		}
		else		
		{
			var nOldVal = this.elements[this.elements.length-1].value;
			this.elements = new Array();
			for(var nIndx=1;nIndx<= 10;nIndx++)
			{
				this.addElement((nOldVal+nIndx),(nOldVal+nIndx));
			}
			this.getList();							
		}			
	}
	,
	getDomNode: function (){
		
		this.currentValues = this.elements[this.selected];
		this.currentValue = this.currentValues.value;
		this.currentResult = this.currentValues.key;	
		while(this.headTableColumn1div.childNodes.length>0)		
			this.headTableColumn1div.removeChild(this.headTableColumn1div.firstChild);			
		this.currentI18nNode = new I18nNode(this.currentValues.callback);
		this.headTableColumn1div.appendChild(this.currentI18nNode.node);

		var Self = this;		
		if(this.optWidth)
			this.myboxComboDiv.style.width=this.optWidth;	
		
		Self.myboxComboDiv.className="smallpopupborder smallpopuplist background-color-content font-color-default";
		Self.myboxComboDiv.style.display="none";
		Self.myboxComboDiv.style.textAlign="left";			
		
		Self.getList();
		Self.rootDiv.appendChild(Self.myboxComboDiv);
		Self.myboxComboDiv.style.overflowX="hidden";
						
	},
	getList : function (){
		var Self = this;
		if(this.scrollbar && this.scrollbar != "" && typeof this.scrollbar != "undefined") {
			if(Self.show_buttons)			
				var nfactor = 17;
			else
				var nfactor = 19.5;
			var tempHeight = Math.floor(this.scrollbar*nfactor);

		}		
		while(this.myboxComboDiv.childNodes.length>0)		
			this.myboxComboDiv.removeChild(this.myboxComboDiv.firstChild);
		if(Self.show_buttons) {
			Self.myboxComboDiv.style.overflow = "hidden";
			Self.myboxComboDiv.style.height=(tempHeight + 22)/pxPerEm + "em";		
			var oDomUp = newnode("div",0,{className:"height10 background-color-content"});
			oDomUp.style.paddingBottom ="2px";			
			oDomUp.style.overflow="hidden";
			oDomUp.style.textAlign = "center";	
			oDomUp.style.height = "10px";			
			oDomUp.style.lineHeight = "10px";			
			oDomUp.appendChild(newnode('img',{marginBottom:"2px"},{src : getFullImgSrc("img/arrows/sort_up.gif")}));
			//select on mouse over
			Self.window.addDOMEvent(oDomUp,
   			 "mouseover", function() { changePopUpMenuStatus3(oDomUp, true, Self) });           			 
   			 
			Self.window.addDOMEvent(oDomUp,
   			 "mouseout", function() { changePopUpMenuStatus3(oDomUp, false, Self) }); 			
   			
			Self.rowHeight = oDomUp.offsetHeight;
			Self.window.addDOMEvent(oDomUp,"click",function (e){Self.scrollElements("up");stopEvent(e);return false;});
			Self.myboxComboDiv.appendChild(oDomUp);
			
		}
		else
			Self.myboxComboDiv.style.height=tempHeight + "px";
		for(var el in Self.elements)
		{
			(function(n) {
				var childDiv = null;
				var myValues = new Array();
				childDiv = newnode("div", {overflow : "hidden"});
				childDiv.style.paddingLeft ="2px";		
				childDiv.style.paddingRight ="2px";		
				childDiv.style.width="96%";
				if(Self.show_buttons)
					childDiv.className="height14 background-color-content";
				else
					childDiv.className="height16 background-color-content";
				
				Self.rowHeight = childDiv.offsetHeight;
				Self.window.addDOMEvent(childDiv,
       			 "click", function() { 
	 						changePopUpMenuText3(Self,n);	           			 						
 						}
       			);           			 
       			 
				Self.window.addDOMEvent(childDiv,
       			 "mouseover", function() { changePopUpMenuStatus3(childDiv, true, Self) });           			 
       			 
				Self.window.addDOMEvent(childDiv,
       			 "mouseout", function() { changePopUpMenuStatus3(childDiv, false, Self) });      
       			         			 
       			myValues = Self.elements[n];
       			if(Self.viewValue) {
       			 	if(myValues.key != "") 
						childDiv.className="height16 font-color-default background-color-content";
       			 	else							
						childDiv.className="height16 font-color-disabled background-color-content";
       			}
				myValues.i18nNode = new I18nNode(myValues.callback);
				childDiv.appendChild(myValues.i18nNode.node);
       			 
				Self.myboxComboDiv.appendChild(childDiv);
				 
			})(el);
		}
		if(Self.show_buttons) {				
			var oDomDown = newnode("div",{textAlign:"center"}, {className : "background-color-content"});
			oDomDown.style.paddingBottom ="2px";		
			oDomDown.style.height = "10px";																	
			oDomDown.style.lineHeight = "10px";						
			oDomDown.appendChild(newnode('img',{marginBottom:"2px",marginTop:"2px"},{src : getFullImgSrc("img/arrows/arrow-down.gif")}));			
			//select on mouse over
			Self.window.addDOMEvent(oDomDown,
   			 "mouseover", function() { changePopUpMenuStatus3(oDomDown, true, Self) });           			 
   			 
			Self.window.addDOMEvent(oDomDown,
   			 "mouseout", function() { changePopUpMenuStatus3(oDomDown, false, Self) }); 			
   				
			Self.window.addDOMEvent(oDomDown,"click",function (e){Self.scrollElements("down");stopEvent(e);return false;});		
			Self.myboxComboDiv.appendChild(oDomDown);
		}		
	}
	
}

function showHideBox3(Self,e)
{	
	if(activeComboBox) {		
		Self.myboxComboDiv.style.display="none";
		fade(activeComboBox,100,0,500, 
			function() {
				activeComboBox = null;				
                if (Self.onHide) Self.onHide();
				//Self.rootDiv.blur();
			}
		);
	} else {
		Self.myboxComboDiv.style.width = Self.rootDiv.offsetWidth+"px";
		Self.myboxComboDiv.style.width = Self.rootDiv.offsetWidth-2+"px";
		if(!document.all)
			Self.myboxComboDiv.style.marginLeft = -1+"px";
			
		var tmpnode=Self.myboxComboDiv.cloneNode(true);
		tmpnode.style.display="block";
		tmpnode.style.zIndex=-1;
		body.appendChild(tmpnode);
		
		var tmpHeight=tmpnode.offsetHeight;
		body.removeChild(tmpnode);
		
		if (body.offsetHeight < (e.clientY + tmpHeight + 45) && !Self.special)
			Self.myboxComboDiv.style.marginTop="-"+(tmpHeight + Self.rootDiv.offsetHeight)+"px";
		else
			Self.myboxComboDiv.style.marginTop="-1px";			
		
		fade(Self.myboxComboDiv,0,100,500);
		Self.myboxComboDiv.style.zIndex="21";		
															
		activeComboBox=Self.myboxComboDiv;
		Self.scrollToSelected();
		
        if (Self.onShow) Self.onShow();
	}
}

function hideComboBox3() {	
	if (activeComboBox && !globalclickdisable2) {
		if (activeComboBox.onHide) activeComboBox.onHide();	
		fade(activeComboBox,100,0,500, function() { 
			activeComboBox = null
		});		
	}
}

function changePopUpMenuStatus3(element,activated, Self) {
	//if(!IE6) return;
	if(activated) 	
		element.className = element.className.replace(/background-color-content/,"background-color-PMG-selection-elements");
	else 
		element.className = element.className.replace(/background-color-PMG-selection-elements/,"background-color-content");
}

var globalclickdisable=false;

function changePopUpMenuStatus(elementid,activated) {
	var d=document.getElementById(elementid);
	if(activated)
		d.className="background-color-PMG-selection-elements border-color-PMG-selection-elements font-color-default";
	else
		d.className="smallpopupnotactive background-color-content font-color-default";
}
fileloaded();
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2003-2007 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Jan Finsel <jan.finsel@open-xchange.com>
 * 
 */

register("Loaded",function(){
});

/**
 * Object ID of the currently displayed folder, or <code>null</code> if no
 * specific folder is displayed.
 */
var activefolder = null;
/**
 * -1 if Configuration Folder Edit
 * [parentfolder] if Configuration Folder New
 *
 */
var configparentfolder=-1;
var superzaehler=1;
/*ID des Reiters im Modul */
// Remove this
var menuarrows=new Object();


register("OX_Direct_Linking",
	function(modulename,object)
	{
		menufollowLink(modulename,object);
	}	
);
var blockremoveerrors=false;
register("OX_GLOBAL_CLICK",
	function ()
	{
		if(!blockremoveerrors) {
			menuremoveErrors(4);
		}
		blockremoveerrors=false;
	});


register("OX_Show_Help_Panel",showPanelHelp);
register("OX_menu_Change_Height",menuHeightChange);
register("OX_menu_Change_Height",menuHeightChangePicture);
resizeEvents.register("Resized", reloadDynamicContent);
register("OX_menu_After_Change_Height",reloadDynamicContent);
register("OX_menu_RESIZE",resizeMenuSplit);
var errorlist=new MRUList(15);

function showErrorinPopup() {
	var d=$("error_popup_messages");
	removeChildNodes(d);
	var hasErrors=false;
	for(var i in corewindow.errorlist.cache) {
		hasErrors=true;
		var element=corewindow.errorlist.get(i);
		var newnode=$("error_messages_template").cloneNode(true);
		newnode.id="";
		newnode.getElementsByTagName("td")[0].firstChild.data=formatDate(element.time,"datetime")+"-->"+formatError(element);
		d.appendChild(newnode);
		newnode.style.display="";
	}
	if(hasErrors) {
		
	} else {
		var newnode=$("error_messages_template").cloneNode(true);
		newnode.id="";
		newnode.getElementsByTagName("td")[0].firstChild.data=_("No error occured");
		d.appendChild(newnode);
		newnode.style.display="";
	}
	ErrorPopup.openWindow();
}
function newServerError(args,level) {
	if(!level) { level=4; }
	if(args.error) {
		var sizes = {
			"MSG-0028": [0, 2], "MSG-0029": [0], "NON-0005": [0, 1],
			"AJP-0006": [0], "FLS-0003": [0, 1, 2], "CON-0101": [1, 2],
			"AJP-0020": [0, 1]
		}[args.code];
		if (sizes) {
			for (var i = 0; i < sizes.length; i++)
				args.error_params[sizes[i]] = bytesToString(args.error_params[sizes[i]]);
		}
		corewindow.storeError(clone(args),level);
		triggerEvent("OX_New_Error",level,formatError(args));		
	}
}
function storeError(args1,level) {
	var myel=clone(args1);
	args1.time=new Date();
	myel.time=new Date(Date.UTC(args1.time.getFullYear(),args1.time.getMonth(),args1.time.getDate(),args1.time.getHours(),args1.time.getMinutes(),0,0));
	errorlist.set(myel.error_id,myel);
}
register("OX_New_Error",
	function (errorprio,errormessage)
	{
		addErrorMessage(errorprio,errormessage);
	});
register("OX_New_Info",
	function (errorprio,errormessage)
	{
		addInfoMessage(errorprio,errormessage);
	});
register("Loaded",loadmenu);    
/** Globale Variablen
 *
 * 0 - 6 Layouts
 */

menuconfig=new Object();
var itemsselected=0;
oxlayout=0;

/**
 * 0 day
 * 1 workweek
 * 2 month
 * 3 week
 */


var taskview=0;
	
var lang="DE";			/*Sprache die Aktiviert ist*/
var helpmessage;		/*Derzeitige Helpmessage*/
// I18N
var errormessages=new Array();
var infomessages=new Array();


/* Beschreibung Trigger */
/* OX_Email_Change_Structure		Struktur Ansicht einer EMail hat sich ge?ndert
 * 
 */
/*menunvariablen */
var debugit=false;
var dynamicdatacontent      	= new Array(); /* Alle dynamischen Contents */
var menu_complete				= 0;  /*Komplette Gr????e menu*/
var menu_dynamic_scroller		= 0;  /*Gr????e dynamischer Scroller*/

var menu_dynamic_rightscroll	= 0;  /*dynamisch Rechts Gr??e*/
var menu_dynamic				= 0;  /*menu Dynamisch Gr??e mit Links und Rechts*/
var menu_dynamic_content		= 0;  /*menu Dynamisch Content Gr????e*/
var status_content				= 0;  /*derzeitiger Status*/


var dynamiccontents=new Object();			 /* Alle dynamischen Contents */
var corecontents=new Array();
var allcontentspicturepopup=new Array();     /* Alle Contents der Picture Popups */

var menu_modules=new Array();

var menuminiteration = 3;
var menuiterationheight=1.6;
var menupadding=0.4;
var menumaxiteration=6;		
var menuiteration=4;

var menupictureiterationmin=3;
var menupictureiterationheight=menupictureiterationmin-1;

var headerheight=1.5;		 /* Mindesth?he menu in Zeilen */
var arrowheight=0.7;
var erroriteration=2;
var menuerrorheight=1.6;
var menuhelpheight=3.6;
var menunhelphoehemin=1;

var newmodule=-1;

var errorid=0;
var popshow=false;
var clickedinpopup=false;
var showhelp=false;

/********************************** New Registrations ************************************************************/
var menulastviews = new Object();
register("OX_Configuration_Set_Empty",loadDefaultModules);
function loadDefaultModules() {
	for (var i in modules) {
		if(configContainsKey("gui."+modules[i].name+".view")) 
		{
			if(modules[i].name == "calendar" && !configGetKey("modules.calendar.calendar_teamview") &&(configGetKey("gui."+modules[i].name+".view").indexOf("calendar/team") == 0 )) {
				configSetKey("gui."+modules[i].name+".view","calendar/calendar/day");
			}
			menulastviews[modules[i].name] = configGetKey("gui."+modules[i].name+".view");
			
		} else {
			if(modules[i].name== "configuration") {
				menulastviews[modules[i].name] = "configuration/settings";
			}
			else {
				menulastviews[modules[i].name] = modules[i].name;
			}
		}
	}
}

register("OX_Switch_Module",switchtoModule);
function switchtoModule(newmodule,param,folder) {
	menuremoveErrors(2);
	if (newmodule == "mail" && folder == configGetKey("mail.folder.spam")) {
		changeView(configGetKey("gui.mail.view_spam") || "mail/list/unthreaded", param);
	} else if (menulastviews[newmodule]) {
		changeView(menulastviews[newmodule],param);
	} else {
		alert("Unknown Module view "+newmodule);
	}
	status_content=0;  // Neues Modul -- Dynamischer Content auf Start
	reloadDynamicContent();
	triggerEvent("OX_Switched");
}

register("OX_Switch_View",switchtoView);
function switchtoView(newview,param) {
	var newmodule=newview.split("/")[0];
	(newmodule!=activemodule) ? menuremoveErrors(2) : menuremoveErrors(3);
	if(isDefaultableView(newview)) {
		menulastviews[newmodule]=newview;
	}
	changeView(newview,param);
	reloadDynamicContent();
	triggerEvent("OX_Switched");
}

function initDisplay(view,elementid,setwidth) {

	var views=view.split("/");
	var tmp=new Object();
	if(!setwidth) {
		tmp.content=$(elementid);
		tmp.content.style.display="";
		tmp.width=tmp.content.offsetWidth;
		tmp.content.style.display="none";
	} else {
		tmp.width=setwidth;
	}
	
	tmp.id=tmp.content.id;
	tmp.view=view;
	// LOAD FROM CONFIG (TODO)
	tmp["display"]="block";
	if(!dynamiccontents[views[0]]) {
		dynamiccontents[views[0]]= new Array();
	}
	dynamiccontents[views[0]].push(tmp);
}
function changeDisplay(name,id) {
	registerView(name,function() {  
			showNode(id);
		},null,null,
		function() {  
			hideNode(id);
		});
	}
/*** Global ***/
changeDisplay("portal","menu_picture");
changeDisplay("portal","menu_portal_quick_settings");

changeDisplay("mail","menu_picture");
changeDisplay("mail","menu_mail");
changeDisplay("mail","menu_mail2");
changeDisplay("mail","menu_mail_view");
changeDisplay("mail","menu_global_tags");

changeDisplay("mail/list","menu_global_attachments");
changeDisplay("mail/hsplit","menu_global_attachments");
changeDisplay("mail/vsplit","menu_global_attachments");

function menuswitchMailView(param) {
	
	var newpath=currentpath2.slice(0);
	newpath[1]=param;
	switch (param)	{
		case "list":
		case "vsplit":
		case "hsplit":
			if(newpath[2]) {
				if(newpath[2]!="unthreaded" && newpath[2]!="threaded") {
					newpath.length=3;
					newpath[2]="unthreaded";
					break;
				}
			} else {
				newpath[2]="unthreaded";
			}
			break;
	}
	switchtoView(newpath.join("/"));
}
function menuswitchStructureView(param) {
	var newpath=currentpath2.slice(0);
	newpath[2]=param;
	newpath.length=3;
	switchtoView(newpath.join("/"));
}
changeDisplay("mail_new","menu_save_mail");
changeDisplay("mail_new","menu_new_mail");
changeDisplay("mail_new","menu_new_mail_view");
changeDisplay("mail_new","menu_dyn_window");

changeDisplay("mail_detail","menu_picture");
changeDisplay("mail_detail","menu_mail");
changeDisplay("mail_detail","menu_mail2");
changeDisplay("mail_detail","menu_global_tags");
changeDisplay("mail_detail","menu_global_attachments");
changeDisplay("mail_detail","menu_dyn_window");

changeDisplay("calendar","menu_picture");
changeDisplay("calendar","menu_calendar");
changeDisplay("calendar","menu_calendar_views");
changeDisplay("calendar","menu_global_tags");
changeDisplay("calendar","menu_linking");

changeDisplay("calendar/calendar","menu_calendar_timerange");
changeDisplay("calendar/team","menu_calendar_timerange");
changeDisplay("calendar/team","menu_calendar_teammember");
changeDisplay("calendar/list","menu_calendar_timerange");
changeDisplay("calendar/detail/attachment","menu_global_attachments");
changeDisplay("calendar/detail","menu_global_confirmation");

function menuswitchCalendarView(param) {
	var newpath=currentpath2.slice(0);
	newpath[1]=param;
	if(!newpath[2]) {
	}
	switch (param)	{
		case "list":
			if(newpath[2]) {
				if(newpath[2]!="day" && newpath[2]!="workweek" && newpath[2]!="month" && newpath[2]!="week") {
					newpath.length=3;
					var oldview=menulastviews["calendar"].split("/");
					if(!oldview[2]) {
						newpath[2]="day";
					}
					else if(oldview[2]!="day" && oldview[2]!="workweek" && oldview[2]!="month" && oldview[2]!="week") {
						newpath[2]="day";
					}
					else {
						newpath[2]=oldview[2];
					}
					break;
				}
			} else {
				newpath[2]="day";
			}
			break;
		case "calendar":
			if(newpath[2]) {
				if(newpath[2]!="day" && newpath[2]!="workweek" && newpath[2]!="month" && newpath[2]!="week" && newpath[2]!="custom") {
					newpath.length=3;
					var oldview=menulastviews["calendar"].split("/");
					if(!oldview[2]) {
						newpath[2]="day";
					}
					else if(oldview[2]!="day" && oldview[2]!="workweek" && oldview[2]!="month" && oldview[2]!="week" && oldview[2]!="custom") {
						newpath[2]="day";
					}
					else {
						newpath[2]=oldview[2];
					}
					break;
				}
			} else {
				newpath[2]="day";
			}
			break;		
		case "team":
			if(newpath[2]) {
				if(newpath[2]!="day" && newpath[2]!="workweek") {
					newpath.length=3;
					var oldview=menulastviews["calendar"].split("/");
					if(!oldview[2]) {
						newpath[2]="day";
					}
					else if(oldview[2]!="day" && oldview[2]!="workweek") {
						newpath[2]="day";
					}
					else {
						newpath[2]=oldview[2];
					}
					break;
				}
			} else {
				newpath[2] = "day";
			}
			break;
		case "detail":
			newpath[2]="overview";
			newpath.length=3;
			break;
	}
	switchtoView(newpath.join("/"));
}
function menuswitchCalendarTimerange(param) {
	var newpath=currentpath2.slice(0);
	newpath[2]=param;
	newpath.length=3;
	switchtoView(newpath.join("/"));
	
}
function menuswitchCalendarDetail(param) {
	var newpath=currentpath2.slice(0);
	newpath[2]=param;
	newpath.length=3;
	switchtoView(newpath.join("/"));
}

changeDisplay("calendar_new","menu_save");
changeDisplay("calendar_new/participant","menu_calendar_new_participant");
changeDisplay("calendar_new/attachment","menu_calendar_new_attachment");
changeDisplay("calendar_new","menu_dyn_window");

function menuswitchCalendarNew(param) {
	var newpath=currentpath2.slice(0);
	newpath[1]=param;
	newpath.length=2;
	switchtoView(newpath.join("/"));
}


changeDisplay("contacts","menu_picture");
changeDisplay("contacts","menu_contact");
changeDisplay("contacts","menu_contact2");
changeDisplay("contacts","menu_contacts_views");
changeDisplay("contacts/detail/attachment","menu_global_attachments");
changeDisplay("contacts","menu_global_tags");
changeDisplay("contacts","menu_linking");
function menuswitchContactView(param) {
	var newpath=currentpath2.slice(0);
	newpath[1]=param;
	switch (param)	{
		case "phonelist":
		case "cards":
			newpath.length=2;
			break;
		case "detail":
			newpath[2]="overview";
			newpath.length=3;
			break;
	}
	switchtoView(newpath.join("/"));
}
function menuswitchContactDetail(param) {
	var newpath=currentpath2.slice(0);
	newpath[2]=param;
	newpath.length=3;
	switchtoView(newpath.join("/"));
}


changeDisplay("contact_new","menu_save");
changeDisplay("contact_new/personal","menu_contact_new_picture");
changeDisplay("contact_new/distributionlist","menu_new_contact_distributionlist");
changeDisplay("contact_new/attachment","menu_calendar_new_attachment");
changeDisplay("contact_new","menu_dyn_window");

function menuswitchContactNew(param) {
	var newpath=currentpath2.slice(0);
	newpath[1]=param;
	newpath.length=2;
	switchtoView(newpath.join("/"));
}

changeDisplay("infostore","menu_picture");
changeDisplay("infostore","menu_infostore");
changeDisplay("infostore","menu_infostore2");
changeDisplay("infostore","menu_infostore_views");
changeDisplay("infostore","menu_infostore_send");
changeDisplay("infostore","menu_global_tags");
changeDisplay("infostore","menu_linking");
changeDisplay("infostore/split/version","menu_infostore_version");
function menuswitchInfoStoreView(param) {
	var newpath=currentpath2.slice(0);
	newpath[1]=param;
	switch (param)	{
		case "list":
		case "split":
			newpath.length=2;
			break;
		case "detail":
			newpath[2]="overview";
			newpath.length=3;
			break;
	}
	switchtoView(newpath.join("/"));
}
function menuswitchInfoStoreDetail(param) {
	var newpath=currentpath2.slice(0);
	newpath[2]=param;
	newpath.length=3;
	switchtoView(newpath.join("/"));
}


changeDisplay("infostore_new","menu_save");
changeDisplay("infostore_new","menu_dyn_window");

changeDisplay("tasks","menu_picture");
changeDisplay("tasks","menu_tasks");
changeDisplay("tasks","menu_tasks_views");
changeDisplay("tasks","menu_global_tags");
changeDisplay("tasks","menu_linking");

changeDisplay("tasks/split/attachment","menu_global_attachments");
changeDisplay("tasks/split","menu_global_confirmation");
function menuswitchTaskView(param) {
	var newpath=currentpath2.slice(0);
	newpath[1]=param;	
	switch (param)	{
	case "list":
		case "split":
			newpath.length=2;
			break;
		case "detail":
			newpath[2]="overview";
			newpath.length=3;
			break;
	}
	switchtoView(newpath.join("/"));
}
function menuswitchTaskDetail(param) {
	var newpath=currentpath2.slice(0);
	newpath[2]=param;
	newpath.length=3;
	switchtoView(newpath.join("/"));
}

changeDisplay("task_new","menu_save");
changeDisplay("task_new/participant","menu_task_new_participant");
changeDisplay("task_new/attachment","menu_calendar_new_attachment");
changeDisplay("task_new","menu_dyn_window");

function menuswitchTaskNew(param) {
	var newpath=currentpath2.slice(0);
	newpath[1]=param;
	newpath.length=2;
	switchtoView(newpath.join("/"));
}

//changeDisplay("configuration","menu_save");
changeDisplay("configuration/settings","menu_save");
changeDisplay("configuration/import","menu_import");
changeDisplay("configuration/tags","menu_save");
changeDisplay("configuration/portal","menu_save");
changeDisplay("configuration/mail","menu_save");
changeDisplay("configuration/calendar","menu_save");
changeDisplay("configuration/contacts","menu_save");
changeDisplay("configuration/tasks","menu_save");

changeDisplay("configuration/infostore","menu_save");
changeDisplay("configuration/folder","menu_save");
changeDisplay("configuration/folder/overview","menu_configuration_folder");
changeDisplay("configuration/folder/rights","menu_configuration_user");
changeDisplay("configuration/folder/rights","menu_configuration_folderright_folder");
changeDisplay("configuration/folder/rights","menu_configuration_objectright_read");
changeDisplay("configuration/folder/rights","menu_configuration_objectright_write");
changeDisplay("configuration/folder/rights","menu_configuration_objectright_delete");
changeDisplay("configuration/folder/rights","menu_configuration_adminright");
changeDisplay("configuration/mail/signatures","menu_config_mail_signatures");
changeDisplay("configuration/portal/external","menu_config_portal_external");
changeDisplay("configuration/portal/external","menu_config_portal_external2");

changeDisplay("configuration/calendar/teams","menu_configuration_teams_teams");
changeDisplay("configuration/tags","menu_configuration_settings_categories");

function menuswitchConfigurationFolderView(param) {
	var newpath=currentpath2.slice(0);
	newpath[2]=param;
	newpath.length=3;
	switchtoView(newpath.join("/"));
}
/********************************** First of all *****************************************************************/
function menufollowLink(mymodule,object) {
	if(mymodule=="task") { mymodule="tasks" } 
	
	//triggerEvent("OX_Switch_View",mymodule);
	var view=mymodule+"/detail/overview";
	activefolder=object.folder;
	if(!object.folder_id && object.folder) {
		object.folder_id=object.folder;
	}
	if(!object.module) {
		object.module=mymodule;
	}
	menucountselected=1;
	menucountsubselected=0;
	oArray=new Array();
	oArray.push(object);
	switch(mymodule) {
		case "infostore":
		    view = menulastviews[mymodule] || mymodule+"/split/overview";
		    if (object.id) {
		    	view = mymodule+"/split/overview";
		    	object["directLink"] = true;
		    	oLastSelectedInfoObject = new Array();
		    	oLastSelectedInfoObject[0] = object;
		    }		
			break;
		case "contacts":		    
		    if (object.id) {
		    	object["directLink"] = true;
		    	contactDetailId = object;
		    } else {
		    	view = menulastviews[mymodule] || configGetKey("gui.contacts.view");
		    }
			break;
		case "calendar":
		    if (object.id) {
		    	object["directLink"] = true;
			    selectedAppointment = object;
		    } else {
		        view = menulastviews[mymodule] || configGetKey("gui.calendar.view");
		    }
			break;
		case "tasks":
		    view = menulastviews[mymodule] || configGetKey("gui.tasks.view");
		    if (object.id) {
	    	    view = mymodule+"/split/overview";
			    object["directLink"] = true;
			    activeTaskId = new Array();
			    activeTaskId[0]=object;
		    }	
			break;
		case "mail":
		    view = menulastviews[mymodule] || configGetKey("gui.mail.view");
		    if (object.id) {
			    if (view.match(/mail\/list/gi)) view = mymodule + "/hsplit/unthreaded";
			    object["directLink"]=true;
                mail_lastSelectedId=object;
		    }
	}
	var folderChanged = (activefolder == object.folder);
	setActiveFolder(object.folder,function() {
		triggerEvent("OX_Switch_View",view, (folderChanged ? "samefolder" : "newfolder"));
		triggerEvent("Selected",oArray);
	},null,true);			
}
function cloneMenuforPopup(clonemodule) {
	var mynode=$("all_of_menu").cloneNode(true);
	var allattributes=mynode.getElementsByTagName("td");
	var removeNode = new Array();
	var len=allattributes.length;
	for (i=0;i<len;i++) {
		var tmptd=allattributes[i];
		if(tmptd.attributes) {
			if(tmptd.getAttribute("oxmodule"))
			{
				var mymodules=tmptd.getAttribute("oxmodule");
				var splitmodules=mymodules.split(",");
				var ismodule=false;
				for(zaehler=0;zaehler<splitmodules.length;zaehler++) {
					if(splitmodules[zaehler]==clonemodule) {
						ismodule=true;
					}
				}
				if(!ismodule) {
					removeNode.push(tmptd);
				}
			}
		}
	}
	for (i=0;i<removeNode.length;i++){
		removeNode[i].parentNode.removeChild(removeNode[i]);
	}
}	
/********************************** Menu Init Functions ***************************************************************************/
function loadmenu()
{
	menuinit(activemodule);
	addDOMEvent($("menu_error"),"mousedown",function() {
		blockremoveerrors=true;
	});
}
var menuselectstart=false;
var menumousedown=false;
function menuinit(newmodule)
{
	if(newmodule)
	{
		activemodule=newmodule;
	}
	$("all_of_menu").onselectstart = function () { var tmp=menuselectstart ; menuselectstart = false; return tmp; }  
	$("all_of_menu").onmousedown = function () { var tmp=menumousedown ; menumousedown = false; return tmp; }
	if($("menu_search_mail_searchfield")) {
		$("menu_search_mail_searchfield").onselectstart = function () {
			menuselectstart=true;
		}
		$("menu_search_mail_searchfield").onmousedown = function () {
			menumousedown=true;
		}
	} 
	if($("menu_search_global_searchfield")) {
		$("menu_search_global_searchfield").onselectstart = function () {
			menuselectstart=true;
		}
		$("menu_search_global_searchfield").onmousedown = function () {
			menumousedown=true;
		}
	} 
	
	register("OX_Configuration_Loaded", function(){
		if(currentpath.length==0) {
			if(configContainsKey("gui.menu.menuiteration")) {
				menuiteration=configGetKey("gui.menu.menuiteration");
			}
			menuHeightChange(true);
			
			var module = location.hash.match(/[#&]m=([^#&]+)/);
			var folder = location.hash.match(/[#&]f=([^#&]+)/);
			var id = location.hash.match(/[#&]i=([^#&]+)/);
			
			activemodule = "portal"; // fallback to portal			
			if (module && folder && id) {
				// direct link, nothing to do here ...
                activemodule = module;
                activefolder = folder;
                return;
            }
			
			// iterating through all available modules after login and 
			// active the first module we found which is enabled. we also try
			// to select the standard folder of this module
			for (var i=0; i < modules.length; i++) {
				var module = modules[i];
				if (module.disable) continue;
				activemodule = module["name"];
				switch(activemodule) {
					case "portal": 
					   activefolder=null; 
					   break;
					case "mail":
					   activefolder=configGetKey("mail.folder.inbox");
					   // checking the default folder of mail
					   // if it's null or empty somethings broken in the 
					   // mail backend and we can't connect to the mail server 
					   if (!activefolder) {
					   	   // to prevent further errors we disable the mail module
					   	   // and keep on iterating
					   	   module.disable = true;
					   	   configSetKey("modules.mail.module", false);
					   	   addModuleIcon({ name: module.name, 
					   	       text: module.text, prio: module.priority, 
					   	       disable: true });

					   	   newAlert(_("The E-Mail module is not available"), 
					   	       _("Unable to establish a connection to the E-Mail server. Possible reasons: the mail server is (temporarily) down or there are network connection problems. To prevent further errors the module has been disabled. Please contact your administrator."), /**i18n**/
					   	       null);
					   	   continue;
					   }
					   break;
					default: 
					   activefolder=configGetKey("folder."+module["name"]); 
					   break;
				}
                break;
			}

            if (activefolder && oMainFolderTree) setActiveFolder(activefolder);
            triggerEvent("OX_Switch_Module",activemodule);
            triggerEvent("OX_Selected_Attachments",0);
		}
	});
	
}
/*** Search Functions ***/
var menumailsearchcontents=["from","to","cc","subject","mailtext"];
var mailsearchcontent=0;
function menu_setSearchField(field) {
	mailsearchcontent=-1;
	for(var i=0;i<menumailsearchcontents.length;i++) {
		if(menumailsearchcontents[i]==field) {
			mailsearchcontent=i; break;
		}
	}
}

function showMailFields() {
	var elements=$("menu_search_mail_context").getElementsByTagName("td");
	for(var i=0;i<elements.length;i++) {
		if(i==mailsearchcontent) {
			elements[i].className="PopupEntryDivActiveNoPadding border-color-PMG-selection-elements";
		} else {
			elements[i].className="";
		}
	}
	popup($("menu_search_mail_context"));
}							
function menudoSearchMail() {
	var param=$('menu_search_mail_searchfield').value;
	triggerEvent("OX_New_Search","mail",param,menumailsearchcontents[mailsearchcontent]); 
}
function menuclearSearchMail() {
	if($("menu_search_mail_searchfield")) $('menu_search_mail_searchfield').value="";
}

function menudoSearch() {
	var param=$('menu_search_global_searchfield').value;
	triggerEvent("OX_New_Search",activemodule,param); 
}
function menuclearSearch() {
	if($("menu_search_global_searchfield")) $('menu_search_global_searchfield').value="";
}

function menuSearchClicked(event) {
	if(event.keyCode == 13) {
		menudoSearch();
	}
}
function menuSearchClickedMail(event) {
	if(event.keyCode == 13) {
		menudoSearchMail();
	}
}

/*** Options Menu ******************************************************************************************************************/
function initContextOptionsMenu() {
	addContextNode(contextmenu_createInputDiv("Search"),["menu_options_content_main"],
		function (e,entryobject) {
			changeDisplayDynamic("menu_search_global",entryobject.div.getElementsByTagName("input")[0].checked);
		},contextmenu_inputenabled,function() {
			var elem=currentpath2.join("/");
			if(!elem.indexOf("tasks/list")) { return true; }
			if(!elem.indexOf("tasks/split")) { return true; }
			if(!elem.indexOf("tasks/search")) { return true; }
			if(!elem.indexOf("infostore/list")) { return true; }
			if(!elem.indexOf("infostore/split")) { return true; }
			if(!elem.indexOf("calendar/calendar")) { return true; }
			if(!elem.indexOf("calendar/list")) { return true; }
			if(!elem.indexOf("contacts/cards")) { return true; }
			if(!elem.indexOf("contacts/phonelist")) { return true; }
			return false;
		},contextmenu_setDisabled,null,null,null,contextmenu_setEnabled
	);					
	addContextNode(contextmenu_createInputDiv("Search"),["menu_options_content_main"],
		function (e,entryobject) {
			changeDisplayDynamic("menu_search_mail",entryobject.div.getElementsByTagName("input")[0].checked);
		},contextmenu_inputenabled,function() {
			var elem=currentpath2.join("/");
			if(!elem.indexOf("mail/list")) { return true; }
			if(!elem.indexOf("mail/hsplit")) { return true; }
			if(!elem.indexOf("mail/vsplit")) { return true; }
			
			return false;
		},contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);
					
	addContextLine(null,["menu_options_content_main"],function () {
		var elem=currentpath2.join("/");
		if(!elem.indexOf("mail/list")) { return true; }
		if(!elem.indexOf("mail/hsplit")) { return true; }
		if(!elem.indexOf("mail/vsplit")) { return true; }
		if(!elem.indexOf("tasks/list")) { return true; }
		if(!elem.indexOf("tasks/split")) { return true; }
		if(!elem.indexOf("tasks/search")) { return true; }
		if(!elem.indexOf("infostore/list")) { return true; }
		if(!elem.indexOf("infostore/split")) { return true; }
		if(!elem.indexOf("calendar/calendar")) { return true; }
		if(!elem.indexOf("calendar/list")) { return true; }
		if(!elem.indexOf("contacts/cards")) { return true; }
		if(!elem.indexOf("contacts/phonelist")) { return true; }
		return false;				
	});
	addContextNode(contextmenu_createInputDiv("Show InfoBox"),["menu_options_content_main"],
		function (e,entryobject) {
			triggerEvent("OX_Show_Mail_Panel",entryobject.div.getElementsByTagName("input")[0].checked);
		},	
		function (entryobject,optparam,first) {
			if(first) {
				entryobject.div.getElementsByTagName("input")[0].checked= entryobject.div.getElementsByTagName("input")[0].defaultChecked =menu_showmail;
			}
			return true;			
		},null,contextmenu_setDisabled,null,null,null,contextmenu_setEnabled
	);
	addContextNode(contextmenu_createInputDiv("Show QuickInfo"),["menu_options_content","menu_options_content_main"],
		function (e,entryobject) {
			triggerEvent("OX_Show_Help_Panel",entryobject.div.getElementsByTagName("input")[0].checked);
		},function (entryobject,optparam,first) {
			if(first) {
				entryobject.div.getElementsByTagName("input")[0].checked= entryobject.div.getElementsByTagName("input")[0].defaultChecked =menu_showhelp;
			}
			return true;			
		},null,contextmenu_setDisabled,null,null,null,contextmenu_setEnabled
	);
	addContextNode(contextmenu_createInputDiv("Show MiniCalendar"),["menu_options_content_main"],
		function (e,entryobject) {
			triggerEvent("OX_Show_MiniCalendar",entryobject.div.getElementsByTagName("input")[0].checked);
		},function (entryobject,optparam,first) {
			if(first) {
				entryobject.div.getElementsByTagName("input")[0].checked= entryobject.div.getElementsByTagName("input")[0].defaultChecked =menu_showmini;
			}
			return true;			
		},null,contextmenu_setDisabled,null,null,null,contextmenu_setEnabled
	);
}	

function changeDisplayDynamic(dynamiccontent,bool)
{
	for (zaehler in dynamiccontents[activemodule])
	{
		if(dynamiccontents[activemodule][zaehler]["content"].id == dynamiccontent && currentpath2.join("/").indexOf(dynamiccontents[activemodule][zaehler]["view"])==0) {
			if(bool) {
				dynamiccontents[activemodule][zaehler]["display"]="";
			} else {
				dynamiccontents[activemodule][zaehler]["display"]="none";
			}
			break;
		}	
	}
	reloadDynamicContent();
}

function optionsmenuchangestatus(elementid,mystatus)
{
	if(mystatus)
	{
		var mytr = elementid;
		mytr.className="smallpopupactive border-color-PMG-selection-elements font-color-default background-color-content";
	}
	else
	{
		var mytr=elementid;
		mytr.className="smallpopupnotactive background-color-content font-color-default background-color-content";
	}
}
/******************** End of Options ****************************************************************************************** */
/************************** Panelhelp ********************************************************************************************/
function showPanelHelp(myboolean)
{
	menu_showhelp=myboolean;
	if(myboolean) {
		$("menu_help").style.display=""
	}
	else {
		$("menu_help").style.display="none";
	}
	triggerEvent("OX_menu_RESIZE");
}	
/************************* Error Panel*******************************************************************************/
function addErrorMessage(priority,errormessage)
{
	if(priority==4) {
		priority==2;
	}
	if(priority>0 && priority<5)
	{
		if(!errormessages[priority])
		{
			errormessages[priority]=new Array();
		}
		errormessages[priority].push(errormessage);
	}
	showErrors1();
}
function addInfoMessage(priority,infomessage) {
	if(priority>0 && priority<5)
	{
		if(!infomessages[priority])
		{
			infomessages[priority]=new Array();
		}
		infomessages[priority].push(infomessage);
	}
	showErrors1();
}
function menuremoveErrors(priority)
{
	var refillmenu=false;
	for(;priority<5;priority++)
	{
		if(errormessages[priority] && errormessages[priority].length != 0)
		{
			errormessages[priority]=new Array();
			refillmenu=true;
		}
		if(infomessages[priority] && infomessages[priority].length != 0)
		{
			infomessages[priority]=new Array();
			refillmenu=true;
		}
	}
	if(refillmenu)
	{
		showErrors1();
	}
}		
function showErrors1()
{
	var showerrors=false;
	var d=$("error_panel");
	d.innerHTML="";
	for(errorprio=4;errorprio>0;errorprio--)
	{
		if(errormessages[errorprio])
		{
			for (i=0;i<errormessages[errorprio].length;i++)
			{
				showerrors=true;
				var diverror=document.createElement("div");
				diverror.style.whiteSpace="normal";
				if(errorprio==4) {
//					diverror.className="warningtext";
					diverror.className="errortext font-color-error";
				}
				else {
					diverror.className="errortext font-color-error";
				}
				diverror.appendChild(document.createTextNode(errormessages[errorprio][i]));
				d.appendChild(diverror);
			}
		}
		if(infomessages[errorprio])
		{
			for (i=0;i<infomessages[errorprio].length;i++)
			{
				showerrors=true;
				var diverror=document.createElement("div");
				diverror.style.whiteSpace="normal";
				diverror.className="infotext font-color-info";
				diverror.appendChild(document.createTextNode(infomessages[errorprio][i]));
				d.appendChild(diverror);
			}
		}
	}
	if(showerrors)
	{
		$("menu_error").style.display="";
	}
	else
	{
		$("menu_error").style.display="none";
	}
	window.setTimeout(function(){triggerEvent("OX_menu_RESIZE");},0);
}


function changeClassAttributes(classname,element,value)
{
	 try {
	 	//Check Browser
	 	var pathtorules;
	 	if (document.all) { pathtorules = 'rules'; }
	 	else if (document.getElementById)  { pathtorules = 'cssRules'; }
		for(var i=0;i<document.styleSheets.length;i++) {
			var item=document.styleSheets.item(i);
			for(var i2=0;i2<item[pathtorules].length;i2++) {
				var rule=item[pathtorules].item(i2);
				if(rule.selectorText==classname) {
					rule.style[element]=value;
				}
			}
		}
	} catch (ex) {}
}


function afterLogin()
{
	new JSON().get("config/diemenu?session="+session,null,new function(reply)
	{
		if(reply)
		{
			if(reply.error)
			{
				alert(reply.error);
			}
			if (reply.oxlayout)
			{
				oxlayout=reply.oxlayout;
			}			
		}
		else
		{
			alert(_("No server connection"));
		}
	});
}

function changePopMenuModules(mymodule)
{
	for(zaehler in allcontentspicturepopup)
	{
		for (zaehler2 in allcontentspicturepopup[zaehler])
		{
			allcontentspicturepopup[zaehler][zaehler2].style.display="none";
		}
	}
	for (zaehler2 in allcontentspicturepopup[mymodule])
	{
		allcontentspicturepopup[newmodule][zaehler2].style.display="block";
	}
}
helpevents.register("help",doHelpMessage);
function doHelpMessage(stringtoshow)
{
	$("menu_help_text").firstChild.nodeValue=stringtoshow;
}
function showPictureOptions()
{
	var hallo=$("menu_picture_pfeil_unten");
	var mypopupmenu=$("menu_popup_picture");
	mypopupmenu.style.top=(hallo.offsetTop+hallo.offsetHeight+3)+"px";
	mypopupmenu.style.left=(hallo.offsetLeft-7)+"px;";
	mypopupmenu.style.zIndex="150";
	mypopupmenu.style.display="block";
}

function menuHeightChange(cancelHeightChange)
{
	
	if(menuiteration<menumaxiteration)
	{
		if(!cancelHeightChange) {
			menuiteration++;
		}
		if(menuiteration==menumaxiteration)
		{
			$("menu_options_arrow_down").style.display="none";
			$("menu_options_arrow_up").style.display="block";
		}
	}
	else
	{
		if(!cancelHeightChange) {
			menuiteration=menuminiteration;
		}
		$("menu_options_arrow_down").style.display="block";
		$("menu_options_arrow_up").style.display="none";
	}
	try
	{
		if(corewindow==window) { configSetKey("gui.menu.menuiteration",menuiteration); }
		changeClassAttributes(".backgroundheight","height",((menuiteration*menuiterationheight)+(menupadding))+"em");
	}
	catch(e){}
	triggerEvent("OX_menu_RESIZE");
}
function resizeMenuSplit()
{
	var contentheight=(menuiteration*menuiterationheight+(menupadding));
	var helpheight=menuhelpheight;
	if($("menu_help").style.display=="none")
	{
		helpheight=0;
	}
	var errorheight=erroriteration*menuerrorheight;
	if($("menu_error").style.display=="none")
	{
		errorheight=0;
	}
	var h = (headerheight+arrowheight+contentheight+helpheight+errorheight+2)+"em";
	try
	{
		resizeSplit("all_of_menu", h); 
	}
	catch (e)
	{}
}
function menuHeightChangePicture()
{
	if(menuisActive("menu_picture")) {
		fillsmallsymbols(menuiteration,activemodule);
	}
}
register("OX_Switched",function() {
	if(menuisActive("menu_picture")) {
		fillsmallsymbols(menuiteration,activemodule);
	}
});
var modulesinpicturemenu=new Object();
function fillsmallsymbols(iteration,mymodule)
{
	modulesinpicturemenu=new Object();
	var zeigeobjekte=(iteration-3)*2
	var zaehler=0;
	var zeigepfeil=false;
	for (zaehler1 in modules)
	{
		if(mymodule==modules[zaehler1]["name"])
		{
			if(document.getElementById("menu_small_"+modules[zaehler1]["name"]))
			{
				document.getElementById("menu_small_"+modules[zaehler1]["name"]).style.display="none";		
				if(modules[zaehler1]["name"]=="contacts" && configGetKey("modules.contacts.module")) {
					if(zaehler<zeigeobjekte) {
						
						document.getElementById("menu_small_distributionlist").style.display="";		
						zaehler++;					
					}
					else {
						modulesinpicturemenu["distributionlist"]=true;
						document.getElementById("menu_small_distributionlist").style.display="none";		
					}
				}
			}
		}
		else
		{
			if(zaehler<zeigeobjekte) {
				if($("menu_small_"+modules[zaehler1]["name"])) {
					if(mymodule == "portal" && modules[zaehler1]["name"] == "mail") {
						$("menu_small_"+modules[zaehler1]["name"]).style.display="none";	
					}	
					else {
						if(configGetKey("modules."+modules[zaehler1]["name"]+".module")) {
							$("menu_small_"+modules[zaehler1]["name"]).style.display="";		
							zaehler++;
							if(modules[zaehler1]["name"]=="contacts" && configGetKey("modules.contacts.module")) {
								if(zaehler<zeigeobjekte) {
									$("menu_small_distributionlist").style.display="";		
									zaehler++;					
								}
								else {
									modulesinpicturemenu["distributionlist"]=true;
									$("menu_small_distributionlist").style.display="none";		
								}
							}
						} else {
							$("menu_small_"+modules[zaehler1]["name"]).style.display="none";		
						}
					}
					
				}			
			}
			else
			{
				if($("menu_small_"+modules[zaehler1]["name"]))
				{
					if(modules[zaehler1]["name"]=="contacts") {
						modulesinpicturemenu["distributionlist"]=true;
						$("menu_small_distributionlist").style.display="none";		
					}
					modulesinpicturemenu[modules[zaehler1]["name"]]=true;
					$("menu_small_"+modules[zaehler1]["name"]).style.display="none";		
					zeigepfeil=true;
				}	
			}
		}
	}
	$("menu_picture_normal").style.display = (zeigeobjekte == 0) ? "none" : "";
	$("menu_picture_contextmenu_na").style.display= zeigepfeil ? "none" : "";
	$("menu_picture_contextmenu").style.display= zeigepfeil ? "" : "none";
}
function reloadDynamicContent()
{
	setTimeout(function() { showElement(status_content,activemodule); },0);
}
var startme=new Object();
function startit(param) {
	startme[param]=new Date().getTime();
}
function endit(status) {
	var end=new Date().getTime();
	startme[status]=new Date().getTime();
}
function showElement(mystatus,mymodule)
{	
  	var mycontents;   
  	var restmain=0;   
	var showscroller = false;
	var showscrollerlast = -1;
	var einzaehler = -1;
	var validcontents=new Object();
	var notvalidcontents=new Object();
	
	for(zaehler2 in dynamiccontents) {
		for(var zaehler3 in dynamiccontents[zaehler2]) {
			var isvalid=false;
			if(zaehler2==mymodule) {
				if(currentpath2.join("/").indexOf(dynamiccontents[mymodule][zaehler3]["view"]) == 0) {
					einzaehler++;
					if(dynamiccontents[mymodule][zaehler3]["display"]!="none") {
						showscrollerlast=einzaehler;showscroller=true;isvalid=true;
						validcontents[dynamiccontents[mymodule][zaehler3]["content"].id]=dynamiccontents[mymodule][zaehler3]["content"];			
					}
				}
			}
			if(!isvalid) {
				notvalidcontents[dynamiccontents[zaehler2][zaehler3]["content"].id]=dynamiccontents[zaehler2][zaehler3]["content"];
			}		
		}
	}

	for(var zaehler3 in notvalidcontents) {
		if(validcontents[zaehler3]) {
			delete notvalidcontents[zaehler3];
		}
	}
	
	if(!showscroller || mystatus == 0) {
		hideNode("menu_dynamic_leftscroll_active",true);
		if($("menu_dynamic_leftscroll_active").style.display!="none") { 	
			hideNode("menu_dynamic_leftscroll_active");
			$("menu_dynamic_leftscroll_active").style.display="none"; 
		}
	}
	if(!showscroller || mystatus != 0) {
		hideNode("menu_dynamic_leftscroll_not_active",true);
		if($("menu_dynamic_leftscroll_not_active").style.display!="none") {
			$("menu_dynamic_leftscroll_not_active").style.display="none"; 
		}
	}
	if(!showscroller || mystatus >=(showscrollerlast)) {
		hideNode("menu_dynamic_rightscroll_active",true);
		if($("menu_dynamic_rightscroll_active").style.display!="none") {  
			$("menu_dynamic_rightscroll_active").style.display="none"; 
		}
	}
	if(!showscroller || mystatus <(showscrollerlast)) {
		hideNode("menu_dynamic_rightscroll_not_active",true);
		if($("menu_dynamic_rightscroll_not_active").style.display!="none") { $("menu_dynamic_rightscroll_not_active").style.display="none"; }
	}
	if(showscroller && mystatus == 0) { 
		showNode("menu_dynamic_leftscroll_not_active",true);
		if($("menu_dynamic_leftscroll_not_active").style.display!="") { $("menu_dynamic_leftscroll_not_active").style.display=""; }
	}
	if(showscroller && mystatus != 0) {
		showNode("menu_dynamic_leftscroll_active",true);
		if($("menu_dynamic_leftscroll_active").style.display!="") { $("menu_dynamic_leftscroll_active").style.display=""; }
	}
	if(showscroller && mystatus>=(showscrollerlast)) {
		showNode("menu_dynamic_rightscroll_not_active",true);
		if($("menu_dynamic_rightscroll_not_active").style.display!="") { $("menu_dynamic_rightscroll_not_active").style.display=""; }
	}
	if(showscroller && mystatus<(showscrollerlast)) {
		showNode("menu_dynamic_rightscroll_active",true);
		if($("menu_dynamic_rightscroll_active").style.display!="") { $("menu_dynamic_rightscroll_active").style.display=""; }
	}
	if(!showscroller) { restmain=0; }
	else {
		restmain=$("menu_dynamic_content").scrollWidth;
		restmain=restmain-20;
	}
	for(var zaehler3 in notvalidcontents) {
		hideNode(notvalidcontents[zaehler3].id,true);
		if(notvalidcontents[zaehler3].style.display!="none") {
			notvalidcontents[zaehler3].style.display="none";
		}
	}
	var einzaehler=0;
	var mybool = true;	
	for (zaehlercontents in validcontents) {
		if(einzaehler>=mystatus) {
			if(mybool) {
				validcontents[zaehlercontents].style.visibility="hidden";
				validcontents[zaehlercontents].style.display="";
				if(validcontents[zaehlercontents].offsetWidth<restmain) {
					restmain=restmain-validcontents[zaehlercontents].offsetWidth;
					showNode(validcontents[zaehlercontents].id,true);
					validcontents[zaehlercontents].style.visibility="visible";
					if(validcontents[zaehlercontents].style.display!="") {
						validcontents[zaehlercontents].style.display="";
					}	
				} else {
					hideNode(validcontents[zaehlercontents].id,true);
					if(validcontents[zaehlercontents].style.display!="none") {
						validcontents[zaehlercontents].style.display="none";	
					}
					mybool=false;
				}
			}		
		}
		else {
			break;
		}
		einzaehler++;
	}	
}
function menu_portal_quick_settings_content_change_password()
{
	triggerEvent("OX_Change_Password_Click");
}


/**
 * Resizes the scrollable area in the menu.
 */


function scrollleft()
{
	myint=status_content-1;
	for (myint;myint>=0;myint--) 
	{
		if(dynamiccontents[activemodule][myint]["display"]!="none")
		{
			status_content=myint;
			break;
		}
	}
	reloadDynamicContent();	
}
function scrollright()
{
	myint=status_content+1;
	for (myint;myint<dynamiccontents[activemodule].length;myint++)
	{ 
		if(dynamiccontents[activemodule][myint]["display"]!="none")
		{
			status_content=myint;
			break;
		}
	}
	reloadDynamicContent();
}
function moduledynamiclength()
{
	return dynamiccontents[activemodule].length;
}
fileloaded();
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2007 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Jan Finsel <jan.finsel@open-xchange.com>
 * 
 */
register("Loaded",function() {
	    var count=0;
		if($("menu_picture")) {
		addContextNode(contextmenu_createImgDiv("E-Mail",/*i18n*/
		getFullImgSrc("img/mail/mail.gif"),getFullImgSrc("img/dummy.gif")),["menu_picture"],
 							function() { triggerEvent('OX_NEW_MAIL'); }, 
							null,
							function() { return configGetKey("modules.mail.module") && modulesinpicturemenu["mail"]; },			
							contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);		
		addContextNode(contextmenu_createImgDiv("Appointment",/*i18n*/
		getFullImgSrc("img/calendar/calendar.gif"),getFullImgSrc("img/dummy.gif")),["menu_picture"],
 							function() { triggerEvent('OX_Before_Create_Calendar'); }, 
							null,
							function() { return configGetKey("modules.calendar.module") && modulesinpicturemenu["calendar"]; },			
							contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);		
		addContextNode(contextmenu_createImgDiv("Contact",/*i18n*/
		getFullImgSrc("img/contacts/contacts.gif"),getFullImgSrc("img/dummy.gif")),["menu_picture"],
 							function() { triggerEvent('OX_Before_Create_Contact'); }, 
							null,
							function() { return configGetKey("modules.contacts.module") && modulesinpicturemenu["contacts"]; },			
							contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);		
		addContextNode(contextmenu_createImgDiv("Distributionlist",/*i18n*/
		getFullImgSrc("img/menu/distributionlist.gif"),getFullImgSrc("img/dummy.gif")),["menu_picture"],
 							function() { triggerEvent('OX_Before_Create_Distributionlist'); }, 
							null,
							function() { return configGetKey("modules.contacts.module") && modulesinpicturemenu["distributionlist"]; },			
							contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);		
		addContextNode(contextmenu_createImgDiv("Task",/*i18n*/
		getFullImgSrc("img/tasks/tasks.gif"),getFullImgSrc("img/dummy.gif")),["menu_picture"],
	 						function() { triggerEvent('OX_Before_Create_Task'); }, 
							null,
							function() { return configGetKey("modules.tasks.module") && modulesinpicturemenu["tasks"]; },			
							contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);	
		addContextNode(contextmenu_createImgDiv("InfoStore Item",/*i18n*/
		getFullImgSrc("img/infostore/infostore.gif"),getFullImgSrc("img/dummy.gif")),["menu_picture"],
 							function() { triggerEvent('OX_Before_Create_InfoStore'); }, 
							null,
							function() { return configGetKey("modules.infostore.module") && modulesinpicturemenu["infostore"]; },			
							contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);		
		registerContext($("menu_picture_contextmenu"),"menu_picture",null,null,null,"click",$("menu_picture_contextmenu"),null,"PopupMenuDivWhite background-color-popup-menu border-color-popup-menu font-size-small");
	}
});

var menudisableattachment=false;
var menucountunlockable=0;
var menucountlockable=0;
var menucounteditable=0;
var menucountselected=0;
var menucountconfirm=0;
var menucountsubselected=0;
var menucountsubselectedlinking=0;
var menunestedmessage=false;
var menucountsubselectedmembers;
var menuisdraftfolder=false;
var menuselectedobjects=new Array();
var menuselectedfolders=new Object();
var menuglobalzaehler=0;
corechangeSelected();
function menuisActive(id) {
	if($(id)) {
		if(!($(id).style.display == "none"))
			return true; 
	}
	return false;
}
function corechangeSelected()
{
	
	register("OX_Selected_Nested",function (nestedbool) {
		menunestedmessage=nestedbool;
	});
	register("OX_Selected_Draft",function (draftbool) {
		menuisdraftfolder=draftbool;
	});
	register("Selected",function (selected) {
		function acb(ofolder) {
			menuselectedfolders=ofolder;
			triggerEvent("OX_SELECTED_ITEMS_CHANGED",menucountselected);
			triggerEvent("OX_SELECTION_CHANGED",menucountselected);
		}
		if(!equals(menuselectedobjects,selected)) {		
			menucountselected=selected.length;
			menuselectedobjects=selected;
			var checkFolders=new Array();
			for(var i=0;i<menuselectedobjects.length;i++) {
				checkFolders.push(menuselectedobjects[i].folder || menuselectedobjects[i].folder_id);
			}
			if(checkFolders.length>0) {
				oMainFolderTree.fetchAndGetFolders(checkFolders,acb);
			} else {
				menuselectedfolders=new Array();
				triggerEvent("OX_SELECTED_ITEMS_CHANGED",menucountselected);
				triggerEvent("OX_SELECTION_CHANGED",menucountselected);
			}
		}
	});
	register("Confirmation_Changed",function(selected) {
		menucountconfirm=selected;
		triggerEvent("OX_SELECTION_CHANGED_CONFIRM",menucountconfirm);
	});
	register("SubSelected",function(selected,disable) {
		menucountsubselected=selected;
		if(disable) { menudisableattachment=true; } else { menudisableattachment=false; }
		triggerEvent("OX_SELECTED_SUBSELECTION_CHANGED",menucountsubselected);
		triggerEvent("OX_SELECTION_CHANGED",menucountsubselected);
	});
	register("SubSelectedTeamMember",function(selected) {
		menucountsubselectedmembers=selected;
		triggerEvent("OX_SELECTED_SUBSELECTION_CHANGED",menucountsubselected);
		triggerEvent("OX_SELECTION_CHANGED",menucountsubselected);
	});
	register("SubSelectedLinking",function(selected) {
		menucountsubselected=selected;
		menucountsubselectedlinking=selected;
		triggerEvent("OX_SELECTED_SUBSELECTIONLINKING_CHANGED",menucountsubselectedlinking);
	});
	register("OX_Selected_Member",function(selected) {
		triggerEvent("SubSelected",selected.length);
	});
	register("OX_Lockable",function(lock,unlock,editlock) {
		menucountlockable=lock;
		menucountunlockable=unlock;
		menucounteditable=editlock;
	})
}
function initContents(modules)
{
	initContextOptionsMenu();
	
	switch (modules) {
		case "core": 
			register("Loaded",function() {
					initDisplay("tasks/list","menu_search_global");
					initDisplay("tasks/split","menu_search_global");
					initDisplay("tasks/search","menu_search_global");
					initDisplay("infostore/list","menu_search_global");
					initDisplay("infostore/split","menu_search_global");
					initDisplay("mail/list","menu_search_mail");
					initDisplay("mail/hsplit","menu_search_mail");
					initDisplay("mail/vsplit","menu_search_mail");
					initDisplay("calendar/calendar","menu_search_global");
					initDisplay("calendar/list","menu_search_global");
					initDisplay("contacts/cards","menu_search_global");
					initDisplay("contacts/phonelist","menu_search_global");				
				registerContext($("menu_new_options_menu_img"),"menu_options_content_main",null,null,null,"click");
				menumailloaded();
				menucalendarloaded();	
				menucontactloaded();
				menutasksloaded();		
				menuinfostoreloaded();		
				menuconfigurationloaded();		
				menugloballoaded();
				createEnhancementMenu("menu_linking_enhancement","menu_linking","global");
			});
			
			register("OX_Configuration_Loaded_Complete",function() {
				setTimeout(function() {
					if(configContainsKey("gui.portal.tooltip"))	{
						var tmp=configGetKey("gui.portal.tooltip");
						triggerEvent("OX_Show_Help_Panel",tmp);
					}
					if(configContainsKey("gui.portal.infobox")) {
						var tmp=configGetKey("gui.portal.infobox");
						triggerEvent("OX_Show_Mail_Panel",tmp);
					}
					if(configContainsKey("gui.portal.minicalendar")) {
						var tmp=configGetKey("gui.portal.minicalendar");
						triggerEvent("OX_Show_MiniCalendar",tmp);
					}
				},0);
			});
			register("SubSelected",changeAttachments);
			register("OX_Switched",function() {changeAttachments(menucountsubselected)});
			register("OX_menu_Change_Height",function() {changeAttachments(menucountsubselected)});
			register("OX_SELECTION_CHANGED_CONFIRM",changeConfirm);
			register("OX_Switched",onmenuchangeHeightorSelectedGlobal);
			register("OX_Selected_Draft",onmenuchangeHeightorSelectedMail);
			register("OX_SELECTED_ITEMS_CHANGED",onmenuchangeHeightorSelectedGlobal);	
			register("SubSelectedLinking",onmenuchangeHeightorSelectedGlobal);
			register("OX_menu_Change_Height",onmenuchangeHeightorSelectedGlobal);
			register("OX_Linking_Cancel",menulinkingcancel);
			register("OX_Linking_Start",
			function() {
				doLinking(true)
			});
			register("OX_Linking_Close",
			function() {
				doLinking(false)
			});
			menuregisterPortalContents();
			menuregisterMailContents();
			menuregisterCalendarContents();
			menuregisterContactContents();
			menuregisterTaskContents();
			if(configGetKey("modules.infostore.module")) {
				menuregisterInfoStoreContents();
			}
			menuregisterConfigurationContents();
			break;
		
		case "mail_new": 
			register("Loaded", function () {
                mailnewonload();
				initDisplay("mail_new","menu_mail_new_attachment");			
				registerContext($("menu_new_options_menu_img"),"menu_options_content",null,null,null,"click");
			});
			register("OX_menu_Change_Height",mailnewattachmentchangeHeight);
			register("OX_Switched",mailnewattachmentchangeHeight);			
            register("SubSelected",mailnewattachmentchangeHeight);
			register("OX_SELECTED_SUBSELECTION_CHANGED",mailnewattachmentchangeHeight);					
			
			register("OX_Show_Help_Panel",function(bool) {
				menu_showhelp = bool;
			});
			break;
		
		case "mail_detail": 
			register("Loaded",function() {
				registerContext($("menu_new_options_menu_img"),"menu_options_content",null,null,null,"click");
				addOnClose($('menu_mail_move_foldertree'),function () { $('menu_mail_move_foldertree').style.display="none"; });
				menumaildetailloaded();
				menugloballoaded();
			});
			menuregisterMailContents();
			register("OX_Selected_Draft",onmenuchangeHeightorSelectedMail);
			register("SubSelected",changeAttachments);
			register("OX_Switched",function() { changeAttachments(menucountsubselected)});
			register("OX_menu_Change_Height",function() {changeAttachments(menucountsubselected)});
			register("OX_Show_Help_Panel",function(bool) {
				menu_showhelp = bool;
			});
			register("OX_Switched",onmenuchangeHeightorSelectedTags);
			register("OX_Switched",onmenuchangeModulePicture);
			register("OX_SELECTED_ITEMS_CHANGED",onmenuchangeHeightorSelectedTags);	
			register("SubSelectedLinking",onmenuchangeHeightorSelectedTags);
			register("OX_menu_Change_Height",onmenuchangeHeightorSelectedTags);
			
			break;
		
		case "calendar_new": 
			register("Loaded", function() {
				calendarnewonload();
				registerContext($("menu_new_options_menu_img"),"menu_options_content",null,null,null,"click");
			});
			
			register("OX_menu_Change_Height",calendarnewattachmentchangeHeight);
			register("OX_Switched",calendarnewattachmentchangeHeight);
			register("OX_SELECTED_SUBSELECTION_CHANGED",calendarnewattachmentchangeHeight);
			register("OX_menu_Change_Height",calendarnewparticipantchangeHeight);	 
			register("OX_SELECTED_SUBSELECTION_CHANGED",calendarnewparticipantchangeHeight);
			register("OX_Switched",calendarnewparticipantchangeHeight);
			register("OX_Show_Help_Panel",function(bool) {
				menu_showhelp=bool;
			});
			
			break;
		
		case "contact_new":  
			register("Loaded", function() {
				registerContext($("menu_new_options_menu_img"),"menu_options_content",null,null,null,"click");
			});
			menuregisterContactNewContents();
			register("OX_menu_Change_Height",calendarnewattachmentchangeHeight);
			register("SubSelected",calendarnewattachmentchangeHeight);
			register("OX_Switched",calendarnewattachmentchangeHeight);
			register("OX_menu_Change_Height",contactnewchangeHeight);
			register("SubSelected",contactnewchangeHeight);
			register("OX_Switched",contactnewchangeHeight);
			register("OX_Show_Help_Panel",function(bool) {
				menu_showhelp=bool;
			});
			break;
		
		case "infostore_new": 
			register("Loaded", function() {
				registerContext($("menu_new_options_menu_img"),"menu_options_content",null,null,null,"click");
			});
			register("OX_Show_Help_Panel",function(bool) {
				menu_showhelp=bool;
			});
			break;
		
		case "task_new": 
			register("Loaded", function() {
				registerContext($("menu_new_options_menu_img"),"menu_options_content",null,null,null,"click");
				tasknewonload();
			});
			register("OX_menu_Change_Height",tasknewparticipantchangeHeight);	 
			register("SubSelected",tasknewparticipantchangeHeight);
			register("OX_Switched",tasknewparticipantchangeHeight);
			register("OX_menu_Change_Height",calendarnewattachmentchangeHeight);
			register("OX_Switched",calendarnewattachmentchangeHeight);
			register("SubSelected",calendarnewattachmentchangeHeight);
			register("OX_Show_Help_Panel",function(bool) {
				menu_showhelp=bool;
			});
			break;	
	}
}

register("OX_Change_InfoStore_Detail_View",function(tab) {
 //triggerEvent("OX_Switched",tab);
});

/****************************************************************************************************/
/*** Debug Function ********************************************************************************/
/****************************************************************************************************/
function menudebugloaded()
{
	fillComboBoxDebug();
}
var combo1;
function fillComboBoxDebug()
{
	/* Language */
	//var myelement=document.getElementById("menu_global_debug_language");
	//combo1=new ComboBox("menu_global_debug_language_combobox","100%","OX_Change_Language");	
	combo1 = new ComboBox3(window, "menu_global_debug_language", "100px", 0, true, "OX_Change_Language", null);
	combo1.addElement("English","EN");
	combo1.addElement("German","DE");
	combo1.getDomNode();
	//myelement.appendChild(combo1.getDomNode());
}
menuregisterDebugContents();
function menuregisterDebugContents()
{
	register("OX_Change_Language",
	function(langu)
	{
		setLanguage(langu);
		reloadDynamicContent();
	});
	register("OX_Execute_Function",
	function() {
		var test=document.getElementById("menu_global_debugfield").value;
		eval(test);
	});
}


/****************************************************************************************************/
/*** Global Function ********************************************************************************/
/****************************************************************************************************/
function menu_display_views(module,modulecontent,checkenabled,enabled, id) {
	if(oldGecko()) {
		var tmp= $(id +"_active") || $(id +"_notactive") || $(id +"_disabled") || false;
		if(!tmp) { return; }
	 	if(tmp.parentNode.parentNode.parentNode.style.width) {
			tmp.parentNode.parentNode.parentNode.style.width="";
		}
	}
	if (menuglobalzaehler < menuiteration) {
		if(checkenabled) {
			if($(id+ "_active")) {
				if(enabled) {
					 if($(id + "_active").style.display != "") { $(id + "_active").style.display= ""; } 
				} else {
					 if($(id + "_active").style.display != "none") { $(id + "_active").style.display= "none"; } 
				}
			}
			if($(id+ "_notactive")) {
				if(enabled) {
					 if($(id + "_notactive").style.display != "none") { $(id + "_notactive").style.display= "none"; } 
				} else {
					 if($(id + "_notactive").style.display != "") { $(id + "_notactive").style.display= ""; } 
				}
			}
			if($(id+ "_disable")) {
				if($(id + "_disable").style.display != "none") { $(id + "_disable").style.display= "none"; } 
			}	
		} 
		else {
			if($(id+ "_disable")) {
				if($(id + "_disable").style.display != "") { $(id + "_disable").style.display= ""; } 
			}
			if($(id+ "_active")) {
				if($(id + "_active").style.display != "none") { $(id + "_active").style.display= "none"; } 
			}
			if($(id+ "_notactive")) {
				if($(id + "_notactive").style.display != "none") { $(id + "_notactive").style.display= "none"; } 
			}
		}
		menuglobalzaehler++;
	} 
	else {
		if($(id+ "_active")) {
			if($(id + "_active").style.display != "none") { $(id + "_active").style.display= "none"; } 
			if(checkenabled && enabled) {
				menuarrows[module][modulecontent].push(id +"_active");		
			}
		}
		if($(id+ "_notactive")) {
			if($(id + "_notactive").style.display != "none") { $(id + "_notactive").style.display= "none"; } 
			if(checkenabled && !enabled) {
				menuarrows[module][modulecontent].push(id +"_notactive");		
			}
		}
		if($(id+ "_disable")) {
			if($(id + "_disable").style.display != "none") { $(id + "_disable").style.display= "none"; } 
			if(!checkenabled) {
			//	menuarrows[module][modulecontent].push(id +"_disable");		
			}
		}
	}
	if(oldGecko()) {
		setTimeout(function() {
			if(tmp.parentNode.parentNode.offsetWidth!=tmp.parentNode.parentNode.parentNode.clientWidth) {
				tmp.parentNode.parentNode.parentNode.style.width=tmp.parentNode.parentNode.offsetWidth+"px";
			}
		},0);
	}
}
function menu_display_contents(module,modulecontent,enabled, id) 
{
	if(oldGecko()) {
		$(id).parentNode.parentNode.parentNode.style.width="";
	}
	try
	{
		if (menuglobalzaehler < menuiteration) 
		{
			if(document.getElementById(id)) {
				if(enabled) {
					if($(id).style.display != "") { $(id).style.display= ""; } 
					menuglobalzaehler++;
				} else {
					if($(id).style.display != "none") { $(id).style.display= "none"; } 
				}
			}
			if(document.getElementById(id + "_na")) {
				if(enabled) {
					if($(id + "_na").style.display != "none") { $(id + "_na").style.display= "none"; } 
				} else {
					if($(id + "_na").style.display != "") { $(id + "_na").style.display= ""; } 
					menuglobalzaehler++;	
				}
			}
		} else {
			if($(id + "_na")) {
				if($(id + "_na").style.display != "none") { $(id + "_na").style.display = "none"; } 
			}
			if($(id)) {
				if($(id).style.display != "none") { $(id).style.display= "none"; } 
			}
			if(enabled)	{
				if($(id)) {
					menuarrows[module][modulecontent].push(id);
				}
			}
			else {
				if($(id + "_na")) {
					//menuarrows[module][modulecontent].push(id + "_na");
				}
			}	
		}
		$(id).parentNode.parentNode.parentNode.style.width="";
	}
	catch (e) {}
	if(oldGecko()) {
		setTimeout(function() {
			$(id).parentNode.parentNode.parentNode.style.width=$(id).parentNode.parentNode.offsetWidth+"px";
		},0);
	}
}
var oFolderTreeMenu2;
var aFolderTreeArray = new Object();
function createMenuPopUpTree(elementid,moduletype,executeFunction)
{
	if($ALL(elementid+"_content") ) { removeChildNodes($ALL(elementid+"_content")) }
	if(moduletype == "folder") {
		aFolderTreeArray[elementid] = null;
		oFolderTreeMenu2 = null;
	
	}

	//if(!aFolderTreeArray[elementid])
	//{
		oFolderTreeMenu2 = new cFolderTree($(elementid+"_content"),executeFunction,false,oMainFolderTree.cache,
	 								optFolderState.getState(elementid+"_content"));	
		aFolderTreeArray[elementid] = oFolderTreeMenu2;
		if(moduletype != "folder")
		{
			oFolderTreeMenu2.enabled_modules = {};
			oFolderTreeMenu2.enabled_modules[moduletype] = true;								
		}
		if(moduletype == "folder")
			{
				var foldertoMove = null;		
				oMainFolderTree.cache.get_folder(configfolder,function (foldertoMove){
						function fn_enable_move(targetFolder)
						{
							return moveAllowed(foldertoMove.parent,foldertoMove, targetFolder,true);
						}
						oFolderTreeMenu2.enable_check_fn = fn_enable_move;
						oFolderTreeMenu2.load();		
				});			
			}		
			else				
				oFolderTreeMenu2.load();
	//}

	oFolderTreeMenu2.setSaveConfiguration(optFolderState);	
}
/*** Rights Function ********************************************************************************/
function menucheckRight(right)
{
	return menuhasRight(menuselectedobjects,right);	
}
function menuhasRightandLoad(selectedobjects,right,cbtrue,cbfalse) {
	function loaded(resp) {
		if(menuhasRight(selectedobjects,right,resp)) { if(cbtrue) cbtrue(); }
		else { if(cbfalse) cbfalse(); }
	}
	var checkFolders=new Array();
	for(var i in selectedobjects) {
		if(selectedobjects[i].folder) {
			checkFolders.push(selectedobjects[i].folder);
		} else {
			checkFolders.push(selectedobjects[i].folder_id);
		}
	}
	if(checkFolders.length>0) {
		oMainFolderTree.fetchAndGetFolders(checkFolders,loaded);
	} else {
		if(menuhasRight(selectedobjects,right)) {
			if(cbtrue) cbtrue();
		} else {
			if(cbfalse) cbfalse();
		}
	}
}
function menuhasRight(selectedobjects,right,resp) {
	var Self=this;
	var uid = configGetKey("identifier");
	var newselectedarray=new Array();
	var newselectedobjects=new Object();
	for(var zaehler in selectedobjects) {
		if(!newselectedobjects[(selectedobjects[zaehler].folder || selectedobjects[zaehler].folder_id)]) {
			newselectedarray.push(selectedobjects[zaehler]);
			newselectedobjects[(selectedobjects[zaehler].folder || selectedobjects[zaehler].folder_id)]=true;
		}
	}
	for(var zaehler in newselectedarray) {
 		var id= newselectedarray[zaehler].id;
 		var created_by = newselectedarray[zaehler].created_by || uid;
 		var folder;
 		if(newselectedarray[zaehler].folder) {
	 		folder= newselectedarray[zaehler].folder;
 		} else  {
	 		folder= newselectedarray[zaehler].folder_id;
 		} 

		var myrights;
		try {
			var oFolder = oMainFolderTree.cache.fast_access[folder];
			if(oFolder != undefined && menuselectedfolders[folder]) {
				oFolder = menuselectedfolders[folder];
			} else if(oFolder != undefined && resp && resp[folder]) {
				oFolder = resp[folder];
			} 
			if(!oFolder) {
				return false;
			}
			 
			if("READ"==right) {
				myrights = computePerm(oFolder.oxfolder.data.own_rights,7);
			} else if("WRITE"==right) {
				myrights = computePerm(oFolder.oxfolder.data.own_rights,14);
			} else if("DELETE"==right) {
				myrights = computePerm(oFolder.oxfolder.data.own_rights,21);
			} else if("ADMIN"==right) {
				myrights = computePerm(oFolder.oxfolder.data.own_rights,28);
			} else if("USER"==right) {
				myrights = computePerm(oFolder.oxfolder.data.own_rights,29);
			} else {
				return false;
			}
								
			if(!(myrights>=2)) {
			 	if(!(myrights>=1)) {
			 		return false;
			 	} else {
					//REMOVE SET BITS USER AND ADMIN
					if (right!="USER" || right!="ADMIN") {						
						switch (oFolder.oxfolder.data.type) {							
                            case 2:
                                 // public folder, check edit own permission
                                 if (created_by != uid) {
                                    return false;                                   
                                 }
                                 break;							
							case 3:
							     // shared folder, check edit own permission
							     if (created_by != uid) {
							     	return false;							     	
							     }
							     break;
							default:
							     break;
						}				
					}
			 	}
			}
		} catch (e) {
			// to be on the save side we return false if anything breaks
			return false;
		}
 	}
 	return true;	
}

/*** Enhancement Menu Functions *********************************************************************/
function createEnhancementMenu(enhancementid,idofelement,module)
{
	createArrowinMenu(enhancementid,idofelement,module);
	createPopUpinMenu(enhancementid,idofelement,module);
	createEvents(enhancementid,idofelement,module);
}
function createArrowinMenu(enhancementid,idofelement,module)
{
	var enhancementmenu=$(enhancementid);
	var mytable=document.createElement("table");
	var mytbody=document.createElement("tbody");
	if(idofelement=="menu_mail") {
		var mydiv=document.createElement("div");
		mydiv.id=enhancementid+"_div";
		mydiv.style.position="absolute";		
	}
	mytable.id=enhancementid+"_enhancementtable";
	addTMPId(mytable);	
	mytable.cellPadding="0";
	mytable.cellSpacing="0";
	mytable.border="0";
	mytable.style.textAlign="center";
	mytable.className="width100 arrowheight";
	var mytr=document.createElement("tr");
	mytr.id=enhancementid+"_arrow";
	addTMPId(mytr);	
	var mytd=document.createElement("td");
	mytd.style.textAlign="center";
	mytd.style.verticalAlign="middle";
	mytd.className="arrowheight";
	mytd.style.height=arrowheight + "em";
	mytd.style.lineHeight=arrowheight +"em";
	mytd.style.margin="0px";
	mytd.style.fontSize="1px";
	var actimg=document.createElement("img");
	actimg.style.height="4px";
	actimg.style.width="8px";
	actimg.setAttribute("src",getFullImgSrc("img/arrows/arrow-down.gif"));
	actimg.hspace="0";
	actimg.vspace="0";
	mytd.appendChild(actimg);
	mytr.appendChild(mytd);
	mytbody.appendChild(mytr);
	mytable.appendChild(mytbody);
	if(idofelement=="menu_mail") {
		mydiv.appendChild(mytable);		
		enhancementmenu.appendChild(mydiv);		
	} else {
		enhancementmenu.appendChild(mytable);		
	}
	var enh1=enhancementid;
	if(enh1=="menu_mail_enhancement") {
		enh1="menu_mail_enhancement_enhancementtable";
	}
	registerContext(mytable,"tmp_enhancementmenu",null,null,null,"click",$ALL(enh1),true,null,function() {
		createPopUpMenuEnhancement(enhancementid,idofelement,module);
	});
	/*addDOMEvent(mytable,"click",
		function (evt) { 
			showPopUpMenu(enhancementid,idofelement,module); 
		});*/
}

function createPopUpinMenu(enhancementid,idofelement,mymodule)
{
	var meinmenu= $(enhancementid);
	var popupdiv=document.createElement("div");
	popupdiv.id=enhancementid+"_popupmenu";
	popupdiv.style.zIndex="150";
	popupdiv.style.position="absolute";
	popupdiv.className="arrowcolor";
	popupdiv.style.display="none";
	meinmenu.appendChild(popupdiv);
	
}
function createEvents(enhancementid,idofelement,module)
{
	var all=false;
	if(module=="global")
	{
		all=true;
	}
	register("OX_menu_Change_Height",function ()
	{
		if(menuisActive(idofelement))
			showArrowPopUpMenu(enhancementid,idofelement,module);
	});
	register("OX_SELECTION_CHANGED",function ()
	{
		if(menuisActive(idofelement))
			showArrowPopUpMenu(enhancementid,idofelement,module);
	});
	register("SubSelectedLinking",function ()
	{
		if(menuisActive(idofelement))
			showArrowPopUpMenu(enhancementid,idofelement,module);
	});
	register("OX_Switched",function ()
	{
		if(menuisActive(idofelement))
			showArrowPopUpMenu(enhancementid,idofelement,module);
		
	});
	register("OX_NEW_LINKING_ENDED",function ()
	{
		if(menuisActive(idofelement))
			showArrowPopUpMenu(enhancementid,idofelement,module);
		
	});
	register("OX_NEW_LINKING_STARTED",function ()
	{
		if(menuisActive(idofelement))
			showArrowPopUpMenu(enhancementid,idofelement,module);
		
	});	
	register("SubSelected",function ()
	{
		if(menuisActive(idofelement))
			showArrowPopUpMenu(enhancementid,idofelement,module);
	});
	register("OX_Lockable",function () {
		if(menuisActive(idofelement))
			showArrowPopUpMenu(enhancementid,idofelement,module);
	});
}
function showArrowPopUpMenu(enhancementid,idofelement,module)
{
	var pfeil=$ALL(enhancementid+"_arrow");
	var isarrow=false;
	if(menuarrows[module])
	{
		if(menuarrows[module][idofelement])
		{
			if(menuarrows[module][idofelement].length>0)
			{
				isarrow=true;
			}
		}
	}
	if(isarrow)
	{
		//MENU_MAIL
		if($ALL(enhancementid+"_div")) {
			$ALL(enhancementid+"_div").style.width=$("menu_mail").offsetWidth+$("menu_mail2").offsetWidth+"px";
		} 
		pfeil.style.display="";
	}
	else
	{
		pfeil.style.display="none";
	}
}
function showPopUpMenu(enhancementid,idofelement,module)
{
	var showit=createPopUpMenuEnhancement(enhancementid,idofelement,module);
	var pfeil=$ALL(enhancementid+"_arrow");
	if(showit)
	{
		var meinmenu=$ALL(enhancementid);
		var popupdiv = $ALL(enhancementid+"_popupmenu");
		if($ALL(enhancementid+"_div")) {
			popupdiv.style.width=$("menu_mail").offsetWidth+$("menu_mail2").offsetWidth+"px";
		} else {
			popupdiv.style.width=meinmenu.offsetWidth+"px";
		}
		if(enhancementid == "menu_mail_enhancement") {
			popupdiv.style.top=$("menu_mail_offset").offsetHeight+"px";
		}
		//popupdiv.style.top=meinmenu.offsetTop+meinmenu.offsetHeight+"px";
		popup($ALL(enhancementid+"_popupmenu"));
	}
}
function createPopUpMenuEnhancement(enhancementid,idofelement,module)
{
	clearContext("tmp_enhancementmenu");
	if(menuarrows[module])
	{
		if(menuarrows[module][idofelement])
		{
			if(menuarrows[module][idofelement].length>0)
			{
				for(var zaehler=0; zaehler < menuarrows[module][idofelement].length;zaehler++) {
					var onetr=$ALL(menuarrows[module][idofelement][zaehler]);
					var textvalue;
					var imagevalue;
					var eventvalue=function() {};
					var noimage=false;
					
					var tmp=onetr.getElementsByTagName("div");
					if(tmp.length>1) {	textvalue=tmp[1].firstChild.nodeValue; noimage=false; } 
					else if (tmp.length>0) { textvalue=tmp[0].firstChild.nodeValue; }
					
					tmp=onetr.getElementsByTagName("img");
					if(tmp.length>0) { imagevalue=tmp[0].getAttribute("src"); }
					
					if(onetr.getAttribute("onclick")) { eventvalue=onetr.getAttribute("onclick"); }
					
					var lasttext=onetr.id.split("_")[onetr.id.split("_").length-1];
					
					if(imagevalue && textvalue) {
						addContextNode(contextmenu_createImgDiv(textvalue,imagevalue,imagevalue),["tmp_enhancementmenu"],
						(function(tmpevent) {
							return function() {
								if(tmpevent && tmpevent.constructor == Function) { tmpevent(); } 
								else { eval(tmpevent); } 
							}
						})(eventvalue),null,null,contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);
					} else if(imagevalue) {
						addContextNode(contextmenu_createOnlyImgDiv(imagevalue,imagevalue),["tmp_enhancementmenu"],
						(function(tmpevent) {
							return function() { 
								if(tmpevent && tmpevent.constructor == Function) { tmpevent(); } 
								else { eval(tmpevent); } 
							}
						})(eventvalue),null,null,contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);	
					} else {
						addContextNode(contextmenu_createOnlyDiv(textvalue),["tmp_enhancementmenu"],
						(function(tmpevent) {
							return function() {
								if(tmpevent && tmpevent.constructor == Function) { tmpevent(); } 
								else { eval(tmpevent); }  
							}
						})(eventvalue),(function(tmpindex) {
							return function() { if(tmpindex!="active") {return true; } }
						})(lasttext),null,contextmenu_setActive,null,null,null,contextmenu_setEnabled);
					}
				}
			}
		}
	}
}

/****************************************************************************************************/
/*** Portal Function ********************************************************************************/
/****************************************************************************************************/
var menu_showmail=false;
var menu_showhelp=false;
var menu_showmini=false;
menuarrows["portal"]=new Array();
function menuregisterPortalContents()
{
	register("OX_Switched",onmenuchangeContentsPortal);
//	register("OX_SELECTED_ITEMS_CHANGED",onmenuchangeContentsPortal);
	register("OX_Show_Mail_Panel",function(bool) {
		if($("menu_portal_mail_checkbox")) {
			$("menu_portal_mail_checkbox").checked = $("menu_portal_mail_checkbox").defaultChecked = bool;
		}
		//configSetKey("gui.portal.infobox",bool);
		menu_showmail=bool;
	});
	register("OX_Show_Help_Panel",function(bool) {
		if($("menu_portal_help_checkbox1")) {
			$("menu_portal_help_checkbox1").checked = $("menu_portal_help_checkbox1").defaultChecked = bool;
		}
		//configSetKey("gui.portal.tooltip",bool);
		menu_showhelp=bool;
	});
	register("OX_Show_MiniCalendar",function(bool) {
		if($("menu_portal_minicalendar_checkbox")) {
			$("menu_portal_minicalendar_checkbox").checked = $("menu_portal_minicalendar_checkbox").defaultChecked = bool;
		}
		//configSetKey("gui.portal.infobox",bool);
		menu_showmini=bool;
	});
	
}
function onmenuchangeContentsPortal()
{
	if(menuisActive("menu_picture"))
		onmenuchangeModulePicture();
}
function onmenuchangeModulePicture()
{ 	
	var oldnode;
	var newnode;
	var prioold;
	for(zaehler in modules) {
		if($("menu_big_"+modules[zaehler]["name"])) {
			if($("menu_big_"+modules[zaehler]["name"]).offsetWidth!=0) {
				if(oldnode) { oldnode.style.width="0px"; }
				oldnode=$("menu_big_"+modules[zaehler]["name"]);
				prioold=modules[zaehler]["priority"];
			}
		}
	}
	for (zaehler in modules)
	{
		if(modules[zaehler]["name"]=="configuration") continue; //Break not a valid module
		if(modules[zaehler]["name"]==activemodule || modules[zaehler]["name"]=="mail" && activemodule == "mail_detail" )
		{
			if((configGetKey("modules."+[modules[zaehler]["name"]]+".module") && activemodule!="portal") || ((activemodule == "portal" && configGetKey("modules.mail.module")))) {
				if($("menu_big_"+modules[zaehler]["name"]))
				{
					newnode=$("menu_big_"+modules[zaehler]["name"]);
					((modules[zaehler]["priority"])<prioold) ? prioold = -1: prioold = 1; 
					/*	$("menu_big_"+modules[zaehler]["name"]).style.display="block"; */		
				}
			} else {
				/*$("menu_big_"+modules[zaehler]["name"]).style.display="none";*/	
			}
		}
		/*else
		{
			if($("menu_big_"+modules[zaehler]["name"]))
			{
				$("menu_big_"+modules[zaehler]["name"]).style.display="none";				
			}
		}*/
	}
	startrotate(oldnode,newnode,prioold);
}

//////////////////////////////////////////////////////////////////////////////////////////////////////
/****************************************************************************************************/
/*** Mail Function **********************************************************************************/
/****************************************************************************************************/
//////////////////////////////////////////////////////////////////////////////////////////////////////
menuarrows["mail"]=new Array();
var menumailfoldertreecopy=false;
function menumaildetailloaded() {	
	createEnhancementMenu("menu_mail_enhancement","menu_mail","mail");
	if ($("menu_dyn_cancel") && $("menu_dyn_close")) {
        $("menu_dyn_cancel").style.display="none";
        $("menu_dyn_close").style.display="";
    }
}
function menumailloaded()
{
	menumaildetailloaded();
	addOnClose($('menu_mail_move_foldertree'),function () { $('menu_mail_move_foldertree').style.display="none"; });
	createEnhancementMenu("menu_mail_view_enhancement","menu_mail_view","mail");
	addContextNode(contextmenu_createImgDiv("Reply",getFullImgSrc("img/mail/reply.gif"),getFullImgSrc("img/mail/reply_d.gif")),["mail"],function() { triggerEvent('OX_Mail_Reply'); }, function() { return menucountselected == 1; },null,contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);	
	addContextNode(contextmenu_createImgDiv("Reply all",getFullImgSrc("img/mail/reply_all.gif"),getFullImgSrc("img/mail/reply_all_d.gif")),["mail"],function() { triggerEvent('OX_Mail_ReplyAll'); }, function() { return menucountselected == 1; },null,contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);	
	addContextNode(contextmenu_createImgDiv("Forward",getFullImgSrc("img/mail/forward.gif"),getFullImgSrc("img/mail/forward_d.gif")),["mail"],function() { triggerEvent('OX_Mail_Forward'); }, function() { return menucountselected > 0; },null,contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);	
	addContextNode(contextmenu_createImgDiv("Delete",getFullImgSrc("img/menu/delete.gif"),getFullImgSrc("img/menu/delete_d.gif")),["mail"],function() { triggerEvent('OX_Mail_Delete'); }, function() { return menucountselected > 0; },null,contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);	
	addContextLine(null,["mail"]);
	addContextNode(contextmenu_createImgDiv("Edit draft",getFullImgSrc("img/menu/edit_draft.gif"),getFullImgSrc("img/dummy.gif")),function() { triggerEvent('OX_Mail_Edit_Draft'); }, function() { return menucountselected == 1 && menuisdraftfolder; },function() { return menuisdraftfolder; },null,contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);	
	addContextNode(contextmenu_createImgDiv("Show source",getFullImgSrc("img/menu/view_emailsource.gif"),getFullImgSrc("img/menu/view_emailsource_d.gif")),["mail"],function() { triggerEvent('OX_Mail_Show_Source'); }, function() { return menucountselected == 1; },null,contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);	
	addContextNode(contextmenu_createImgDiv("Spam",getFullImgSrc("img/mail/spam.gif"),getFullImgSrc("img/mail/spam_d.gif")),["mail"],function() { triggerEvent('OX_Mail_Spam'); },null, function() { 
			return menucountselected > 0 && configGetKey("mail.spambutton") && checkSpamFolder("SPAM") 
		},contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);	
	addContextNode(contextmenu_createImgDiv("No Spam",getFullImgSrc("img/menu/ham.gif"),getFullImgSrc("img/menu/ham.gif")),["mail"],function() { triggerEvent('OX_Mail_Ham'); },null,function() { return menucountselected > 0 && configGetKey("mail.spambutton") && checkSpamFolder("HAM") },contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);	

}
function menuregisterMailContents()
{
	register("OX_Mail_Move",menumailmoveclicked);
	register("OX_Mail_Copy",menumailcopyclicked);
	register("OX_SELECTED_ITEMS_CHANGED",onmenuchangeHeightorSelectedMail);
	register("OX_menu_Change_Height",onmenuchangeHeightorSelectedMail);
	register("OX_Switched",onmenuchangeHeightorSelectedMail);
}	
function menumailmoveclicked()
{
	menumailfoldertreecopy=false;
	createMenuPopUpTree("menu_mail_move_foldertree","mail",doMailsFolder);	
	document.getElementById("menu_mail_move_foldertree").style.display="";
} 
function menumailcopyclicked()
{
	menumailfoldertreecopy=true;
	createMenuPopUpTree("menu_mail_move_foldertree","mail",doMailsFolder);	
   	document.getElementById("menu_mail_move_foldertree").style.display="";
} 
function doMailsFolder(ofolder)
{
	var selectedfolderid= ofolder.oxfolder.data.id;
	if(menumailfoldertreecopy) {
		triggerEvent("OX_Object_Copy",selectedfolderid);
	} else {
		triggerEvent("OX_Object_Move",selectedfolderid);
	}
	document.getElementById("menu_mail_move_foldertree").style.display="none";
}

function onmenuchangeHeightorSelectedMail()
{
	if(menuisActive("menu_mail_structure")) 	
		onmenuchangeMailStructure(menuiteration,currentpath2[2],menucountselected);
	if(menuisActive("menu_mail_view")) 	
		onmenuchangeMailView(menuiteration,currentpath2[1],menucountselected);
	if(menuisActive("menu_mail"))	
		onmenuchangeHeightMailMainContent(menuiteration,menucountselected);
}
function onmenuchangeMailStructure(iteration,item,selecteditems)
{
	if(item == "threaded")
	{
		$("menu_mail_structure_threaded_active").style.display="";
		$("menu_mail_structure_threaded_notactive").style.display="none";
		$("menu_mail_structure_unthreaded_active").style.display="none";
		$("menu_mail_structure_unthreaded_notactive").style.display="";
	}
	else if(item == "unthreaded")
	{
		$("menu_mail_structure_threaded_active").style.display="none";
		$("menu_mail_structure_threaded_notactive").style.display="";
		$("menu_mail_structure_unthreaded_active").style.display="";
		$("menu_mail_structure_unthreaded_notactive").style.display="none";
	}
}
function onmenuchangeMailView(iteration,value,selecteditems)
{
	menuglobalzaehler=0;
    menuarrows["mail"]["menu_mail_view"]=new Array();	
	menu_display_views("mail","menu_mail_view",true,value == "hsplit", "menu_mail_views_split");
	menu_display_views("mail","menu_mail_view",true,value == "vsplit", "menu_mail_views_split2");
	menu_display_views("mail","menu_mail_view",true,value == "list", "menu_mail_views_list");
	menu_display_views("mail","menu_mail_view",selecteditems==1,value == "detail", "menu_mail_views_detail");
}
function checkSpamFolder(type) {
	for(var i in menuselectedobjects) {
		var folder = menuselectedobjects[i].folder;
		if(!folder) folder= menuselectedobjects[i].folder_id;
		switch(type) {
			case "SPAM":
				if(folder == configGetKey("mail.folder.spam")) {
					return false;
				}			
				break;
			case "HAM":
				if(folder != configGetKey("mail.folder.spam")) {
					return false;
				}
				break;
			default:
				return false;
		}
	}
	return true;
}

function isNestedMailMessage() {	
	if (activemodule == "mail_detail" && menuselectedobjects && menuselectedobjects.length) {		
		return menuselectedobjects[0].type != undefined && menuselectedobjects[0].type == "isNestedMessage";		
	}
	return false;
}

function onmenuchangeHeightMailMainContent(iteration,selecteditems) {
	menuglobalzaehler=0;
	menuarrows["mail"]["menu_mail"]=new Array();
	menu_display_contents("mail","menu_mail",selecteditems == 1 && menuisdraftfolder, "menu_mail_draft_edit");
	menu_display_contents("mail","menu_mail",selecteditems == 1, "menu_mail_reply");
	menu_display_contents("mail","menu_mail",selecteditems == 1, "menu_mail_reply_all");
	menu_display_contents("mail","menu_mail",selecteditems > 0, "menu_mail_forward");
    menu_display_contents("mail","menu_mail",selecteditems > 0 && activemodule != "mail_detail" , "menu_mail_copy");
	menu_display_contents("mail","menu_mail",selecteditems == 1 && !isNestedMailMessage(),  "menu_mail_showsource");
	menuglobalzaehler=0;
	menu_display_contents("mail","menu_mail",selecteditems > 0 && !isNestedMailMessage(), "menu_mail_delete");
	menu_display_contents("mail","menu_mail",isPrintableView(currentpath2.join('/')) && !isNestedMailMessage(),"menu_mail_print");
    menu_display_contents("mail","menu_mail",selecteditems > 0 && configGetKey("mail.spambutton") && checkSpamFolder("SPAM") && !isNestedMailMessage(),  "menu_mail_spam");
    menu_display_contents("mail","menu_mail",selecteditems > 0 && configGetKey("mail.spambutton") && checkSpamFolder("HAM") && !isNestedMailMessage(),  "menu_mail_ham");	
    menu_display_contents("mail","menu_mail",selecteditems > 0 && !isNestedMailMessage(),  "menu_mail_mark");	
	menu_display_contents("mail","menu_mail",selecteditems > 0 && activemodule != "mail_detail" , "menu_mail_move");	
}

/****************************************************************************************************/
/*** Calendar Function ******************************************************************************/
/****************************************************************************************************/
function multipleconfirmcalendar(objects) {
	var myobjects=new Array();
	for (var i in objects) {
		myobjects.push(new AcceptDenyObject("calendar",objects[i]));
	}
	if(myobjects.length) {
		var myad=new AcceptDeny(false);
		myad.changeConfirm(myobjects);
	}
}
register("OX_Calendar_Change_Confirm",multipleconfirmcalendar);
menuarrows["calendar"]=new Array();
function menucalendarloaded()
{	
	addOnClose($('menu_calendar_move_foldertree'),function () { $('menu_calendar_move_foldertree').style.display="none"; });
	function getdata (entrycomplete) {
		
		function responseData(resp) {
			returnobject.objects=resp;
			entrycomplete.params=returnobject;		
			if(entrycomplete.conditionvisible(entrycomplete)) {
				addDOMEvent(entrycomplete.div,'mouseover',entrycomplete.fn_mouseover);
				addDOMEvent(entrycomplete.div,'mouseout',entrycomplete.fn_mouseout);				
				addDOMEvent(entrycomplete.div,'click',entrycomplete.fn_callback);
				if(!entrycomplete.conditionactive(entrycomplete)) {
					entrycomplete.disabled(entrycomplete.div);
				} else {
					entrycomplete.enabled(entrycomplete.div);
				}
			}
		}
		var returnobject=new Object();
		var myobjects = clone(calendarSelection.getSelected());
		for (var i=0;i<myobjects.length;i++) {
			myobjects[i]["module"] = "calendar";
			myobjects[i]["folder_id"] = myobjects[i].folder;
			delete myobjects[i].folder;
		}
		objectcache.get(myobjects,["created_by","users"],responseData,responseData);				
	}
	function count(myobjects) {
		var returnobject=new Object();
		returnobject.countall=0;
		for(var i in myobjects) {
			returnobject.countall++;
		}
		return returnobject.countall;
	}
	addContextNode(contextmenu_createImgDiv("Edit",getFullImgSrc("img/menu/edit.gif"),getFullImgSrc("img/menu/edit_d.gif")),["calendar"],function() { triggerEvent('OX_Calendar_Edit'); },
	function(entryobject) { 
		var myobjects=entryobject.params.objects;
		if(count(myobjects) == 1 && menuhasRight(myobjects,"WRITE")) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,getdata,contextmenu_setEnabled);	
	addContextNode(contextmenu_createImgDiv("Delete",getFullImgSrc("img/menu/delete.gif"),getFullImgSrc("img/menu/delete_d.gif")),["calendar"],function() { triggerEvent('OX_Calendar_Delete'); },
	function(entryobject) { 
		var myobjects=entryobject.params.objects;
		if(count(myobjects) > 0 && menuhasRight(myobjects,"DELETE")) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,getdata,contextmenu_setEnabled);
	addContextLine(null,["calendar"]);	
	var tmp="Change confirmation"; /*i18n*/
	addContextNode(contextmenu_createImgDiv("Change confirmation",getFullImgSrc("img/menu/global_confirmation_change.gif"),getFullImgSrc("img/menu/global_conf_change_d.gif")), 
	["calendar"],function(e,entryobject) { triggerEvent("OX_Calendar_Change_Confirm",entryobject.params.objects); },
	function(entryobject) { 
		var myobjects=entryobject.params.objects;
		for(var i in myobjects) {
			var userof=false;
			for(var zaehler=0;zaehler<myobjects[i].users.length;zaehler++) {
				if(myobjects[i].users[zaehler].id == configGetKey("identifier")) {
					userof=true;
				} 
			}
			if(!userof) { return false }
		}
		if(count(myobjects) > 0) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,getdata,contextmenu_setEnabled);	
	
	createEnhancementMenu("menu_calendar_enhancement","menu_calendar","calendar");
	createEnhancementMenu("menu_calendar_timerange_enhancement","menu_calendar_timerange","calendar");
	createEnhancementMenu("menu_calendar_views_enhancement","menu_calendar_views","calendar");
	createEnhancementMenu("menu_calendar_teammember_enhancement","menu_calendar_teammember","calendar");
}

function menuregisterCalendarContents()
{
	register("OX_Calendar_Move",menucalendarmoveclicked);
	register("OX_Switched",onmenuchangeHeightorSelectedCalendar);
	register("OX_menu_Change_Height",onmenuchangeHeightorSelectedCalendar);
	register("OX_SELECTED_ITEMS_CHANGED",onmenuchangeHeightorSelectedCalendar);
	register("OX_SELECTED_SUBSELECTION_CHANGED",onmenuchangeSubSelectedCalendar);
}
function menucalendarmoveclicked()
{
	createMenuPopUpTree("menu_calendar_move_foldertree","calendar",moveCalendarFolder);
	document.getElementById("menu_calendar_move_foldertree").style.display="";
} 
function moveCalendarFolder(ofolder)
{
	var selectedfolderid= ofolder.oxfolder.data.id;
	triggerEvent("OX_Object_Move",selectedfolderid);
	document.getElementById("menu_calendar_move_foldertree").style.display="none";
}
function onmenuchangeHeightorSelectedCalendar()
{
	if(menuisActive("menu_calendar")) 
		onmenuchangeHeightCalendarMainContent(menuiteration,menucountselected);
	if(menuisActive("menu_calendar_views"))		
		onmenuchangeHeightCalendarViewContent(menuiteration,currentpath2[1],menucountselected);
	if(menuisActive("menu_calendar_timerange"))		
		onmenuchangeHeightCalendarTimerangeContent(menuiteration,currentpath2[2],menucountselected);
	if(menuisActive("menu_calendar_teammember"))
		onmenuchangeHeightCalendarTeamContent(menuiteration,menucountsubselected);
}
function onmenuchangeSubSelectedCalendar(selected)
{
	if(activemodule=="calendar")
	{
		onmenuchangeHeightCalendarTeamContent(menuiteration,selected);
	}
}

function onmenuchangeHeightCalendarTimerangeContent(iteration,value,selecteditems)
{
	menuglobalzaehler=0;
	menuarrows["calendar"]["menu_calendar_timerange"]=new Array();
	menu_display_views("calendar","menu_calendar_timerange",true,value == "day", "menu_calendar_timerange_day");
	menu_display_views("calendar","menu_calendar_timerange",true,value == "workweek", "menu_calendar_timerange_workweek");
	menu_display_views("calendar","menu_calendar_timerange",currentpath2[1]!="team",value == "month", "menu_calendar_timerange_month");
	menu_display_views("calendar","menu_calendar_timerange",currentpath2[1]!="team",value == "week", "menu_calendar_timerange_week");
	menu_display_views("calendar","menu_calendar_timerange",currentpath2[1]!="team" && currentpath2[1] != "list" ,value == "custom", "menu_calendar_timerange_custom");
}
function onmenuchangeHeightCalendarViewContent(iteration,value,selecteditems) {
	menuglobalzaehler=0;
	menuarrows["calendar"]["menu_calendar_views"]=new Array();	
    menu_display_views("calendar","menu_calendar_views",true,value == "calendar", "menu_calendar_views_calendar");
    menu_display_views("calendar","menu_calendar_views",configGetKey("modules.calendar.calendar_freebusy") && configGetKey("modules.calendar.calendar_teamview"),value == "team", "menu_calendar_views_team");	
	menu_display_views("calendar","menu_calendar_views",true,value == "list", "menu_calendar_views_list");
	menu_display_views("calendar","menu_calendar_views",selecteditems==1,value == "detail", "menu_calendar_views_detail");
}
function onmenuchangeHeightCalendarMainContent(iteration,selecteditems)
{
	menuglobalzaehler=0;
	menuarrows["calendar"]["menu_calendar"]=new Array();
	menu_display_contents("calendar","menu_calendar",selecteditems == 1 && menucheckRight("WRITE"), "menu_calendar_edit");
	menu_display_contents("calendar","menu_calendar",isPrintableView(currentpath2.join('/')), "menu_calendar_print");
	menu_display_contents("calendar","menu_calendar",selecteditems > 0 && menucheckRight("DELETE"), "menu_calendar_delete");
	menu_display_contents("calendar","menu_calendar",selecteditems > 0 && menucheckRight("DELETE"), "menu_calendar_move");		
}
function onmenuchangeHeightCalendarTeamContent(menuiteration,selected) {
	menuglobalzaehler=0;
	menuarrows["calendar"]["menu_calendar_teammember"]=new Array();
	menu_display_contents("calendar","menu_calendar_teammember",true, "menu_calendar_teammember_add");
	menu_display_contents("calendar","menu_calendar_teammember",selected>0, "menu_calendar_teammember_remove");
	menu_display_contents("calendar","menu_calendar_teammember",configContainsKey("gui.calendar.teams") && configGetKey("gui.calendar.teams").length, "menu_calendar_teammember_change");
	menu_display_contents("calendar","menu_calendar_teammember",true, "menu_calendar_teammember_add_team");
}
register("OX_Calendar_Team_Add_Team",calendar_teams_add_team_clicked);
function calendar_teams_add_team_clicked() {
	triggerEvent('OX_Switch_View','configuration/calendar/teams');
}

///////////////////////////////////////////////////////////////////////////////////////////////////////
/****************************************************************************************************/
/*** Task Function ******************************************************************************/
/****************************************************************************************************/
function multipleconfirmtask(objects) {
	var myobjects=new Array();
	for (var i in objects) {
		myobjects.push(new AcceptDenyObject("tasks",objects[i]));
	}
	if(myobjects.length) {
		var myad=new AcceptDeny(false);
		myad.changeConfirm(myobjects);
	}
}
register("OX_Task_Change_Confirm",multipleconfirmtask);

var menucomboboxsearch1;
menuarrows["tasks"]=new Array();
function menutasksloaded()
{
	addOnClose($('menu_tasks_move_foldertree'),function () { $('menu_tasks_move_foldertree').style.display="none"; });
	createEnhancementMenu("menu_tasks_enhancement","menu_tasks","tasks");
	function defaultloading(entrycomplete) {	
		function handleresponse(resp) {
			entrycomplete.params=resp;		
			if(myobject.conditionvisible(entrycomplete)) {
				addDOMEvent(entrycomplete.div,'mouseover',entrycomplete.fn_mouseover);
				addDOMEvent(entrycomplete.div,'mouseout',entrycomplete.fn_mouseout);				
				addDOMEvent(entrycomplete.div,'click',entrycomplete.fn_callback);
				if(!myobject.conditionactive(entrycomplete)) {
					myobject.disabled(entrycomplete.div);
				} else {
					myobject.enabled(entrycomplete.div);
				}
			}
		}
		var checkFolders=new Array();
		for(var i=0;i<menuselectedobjects.length;i++) {
			checkFolders.push(menuselectedobjects[i].folder);
		}
		if(checkFolders.length>0) {
			oMainFolderTree.fetchAndGetFolders(checkFolders,handleresponse);
		} 
	}
	function getdata (entrycomplete) {
		function responseData(resp) {
			returnobject.objects=resp;
			entrycomplete.params=returnobject;		
			if(entrycomplete.conditionvisible(entrycomplete)) {
				addDOMEvent(entrycomplete.div,'mouseover',entrycomplete.fn_mouseover);
				addDOMEvent(entrycomplete.div,'mouseout',entrycomplete.fn_mouseout);				
				addDOMEvent(entrycomplete.div,'click',entrycomplete.fn_callback);
				if(!entrycomplete.conditionactive(entrycomplete)) {
					entrycomplete.disabled(entrycomplete.div);
				} else {
					entrycomplete.enabled(entrycomplete.div);
				}
			}
		}
		var returnobject=new Object();
		var myobjects = tasksSelection2.getSelected();
		objectcache.get(myobjects,["users","created_by"],responseData,responseData);				
	}
	function count(myobjects) {
		var returnobject=new Object();
		returnobject.countall=0;
		for(var i in myobjects) {
			returnobject.countall++;
		}
		return returnobject.countall;
	}
	addContextNode(contextmenu_createImgDiv("Edit",getFullImgSrc("img/menu/edit.gif"),getFullImgSrc("img/menu/edit_d.gif")),["tasks"],function() { triggerEvent('OX_Task_Edit'); },
	function(entryobject) { 
		var myobjects=entryobject.params.objects;
		if(count(myobjects) == 1 && menuhasRight(myobjects,"WRITE")) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,getdata,contextmenu_setEnabled);	
	addContextNode(contextmenu_createImgDiv("Delete",getFullImgSrc("img/menu/delete.gif"),getFullImgSrc("img/menu/delete_d.gif")),["tasks"],function() { triggerEvent('OX_Task_Delete'); },
	function(entryobject) { 
		var myobjects=entryobject.params.objects;

		if(count(myobjects) > 0 && menuhasRight(myobjects,"DELETE")) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,getdata,contextmenu_setEnabled);	
	addContextNode(contextmenu_createImgDiv("Use as template",getFullImgSrc("img/menu/task_copy.gif"),getFullImgSrc("img/menu/task_copy_d.gif")),["tasks"],function() { triggerEvent('OX_Task_Duplicate'); },
	function(entryobject) { 
		var myobjects=entryobject.params.objects;

		if(count(myobjects) == 1 && menuhasRight(myobjects,"WRITE")) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,getdata,contextmenu_setEnabled);
	addContextNode(contextmenu_createImgDiv("Mark as Done",getFullImgSrc("img/dummy.gif"),getFullImgSrc("img/dummy.gif")),["tasks"],function() { triggerEvent('OX_Task_MarkAsDone'); },
    function(entryobject) { 
        var myobjects=entryobject.params.objects;
        if(count(myobjects) >= 1 && menuhasRight(myobjects,"WRITE")) { return true; }
        return false;
    },null,contextmenu_setDisabled,null,null,getdata,contextmenu_setEnabled);	
	addContextLine(null,["tasks"]);
	addContextNode(contextmenu_createImgDiv("Change confirmation",getFullImgSrc("img/menu/global_confirmation_change.gif"),getFullImgSrc("img/menu/global_conf_change_d.gif")),["tasks"],function(e,entryobject) { triggerEvent('OX_Task_Change_Confirm',entryobject.params.objects); },
		function(entryobject) { 
		var myobjects=entryobject.params.objects;
		for(var i in myobjects) {
			var userof=false;
			for(var zaehler=0;zaehler<myobjects[i].users.length;zaehler++) {
				if(myobjects[i].users[zaehler].id == configGetKey("identifier")) {
					userof=true;
				} 
			}
			if(!userof) { return false }
		}
		if(count(myobjects) > 0) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,getdata,contextmenu_setEnabled);	
	
}

function menuregisterTaskContents()
{
	register("OX_Task_Move",menutaskmoveclicked);
	register("OX_Switched",onmenuchangeHeightorSelectedTask);
	register("OX_SELECTED_ITEMS_CHANGED",onmenuchangeHeightorSelectedTask);
	register("OX_menu_Change_Height",onmenuchangeHeightorSelectedTask);
	register("OX_Task_Search",onmenusearchTask);
}
function onmenusearchTask() 
{
	var mysearchfield=menucomboboxsearch1.getValue();
	var mysearchparam=document.getElementById("menu_search_tasks_searchfield").value;
	triggerEvent("OX_Task_Standard_Search",activefolder,mysearchfield,mysearchparam);
}
register("OX_Task_Standard_Search",todoSearchTask);

function todoSearchTask(folder,mysearchfield,mysearchparam)
{
	alert("TODO: Hier Suche in Folder "+ folder +" im Feld "+mysearchfield+" nach "+mysearchparam);
}
function menutaskmoveclicked()
{
	createMenuPopUpTree("menu_tasks_move_foldertree","tasks",moveTasksFolder);
	document.getElementById("menu_tasks_move_foldertree").style.display="";
} 
function moveTasksFolder(ofolder)
{
	var selectedfolderid= ofolder.oxfolder.data.id;
	triggerEvent("OX_Object_Move",selectedfolderid);
	document.getElementById("menu_tasks_move_foldertree").style.display="none";
}

function onmenuchangeHeightorSelectedTask()
{
	if(menuisActive("menu_tasks_views"))		
		onmenuchangeTaskView(menuiteration,currentpath2[1],menucountselected);
	if(menuisActive("menu_tasks"))		
		onmenuchangeHeightTaskContent(menuiteration,menucountselected);	
}
function onmenuchangeTaskView(iteration,value,selecteditems) {
	menuglobalzaehler=0;
	menuarrows["tasks"]["menu_tasks_views"]=new Array();
    menu_display_views("tasks","menu_tasks_views",true,value == "split", "menu_tasks_views_split");
	menu_display_views("tasks","menu_tasks_views",true,value == "list", "menu_tasks_views_list");
    menu_display_views("tasks","menu_tasks_views",selecteditems==1,value == "detail", "menu_tasks_views_detail");
}
function onmenuchangeHeightTaskContent(iteration,selecteditems)
{
	menuglobalzaehler=0;
	menuarrows["tasks"]["menu_tasks"]=new Array();
	menu_display_contents("tasks","menu_tasks",selecteditems == 1 && menucheckRight("WRITE"), "menu_tasks_edit");
	menu_display_contents("tasks","menu_tasks",selecteditems > 0 && menucheckRight("DELETE"), "menu_tasks_delete");
	menu_display_contents("tasks","menu_tasks",isPrintableView(currentpath2.join('/')), "menu_tasks_print");
	menu_display_contents("tasks","menu_tasks",selecteditems > 0 && menucheckRight("DELETE"), "menu_tasks_move");
	menu_display_contents("tasks","menu_tasks",selecteditems > 0 && menucheckRight("WRITE"), "menu_tasks_mark_as_done");
	menu_display_contents("tasks","menu_tasks",selecteditems == 1 && menucheckRight("WRITE"), "menu_tasks_duplicate");
}
	//////////////////////////////////////////////////////////////////////////////////////////////////////
/****************************************************************************************************/
/*** Contact Function ******************************************************************************/
/****************************************************************************************************/
menuarrows["contacts"]=new Array();
var menucontactfoldertreecopy=false;
function menucontactloaded()
{
	addOnClose($('menu_contacts_move_foldertree'),function () { $('menu_contacts_move_foldertree').style.display="none"; });
	function getdata (entrycomplete) {
		function responseData(resp) {
			returnobject.objects=resp;
			entrycomplete.params=returnobject;		
			if(entrycomplete.conditionvisible(entrycomplete)) {
				addDOMEvent(entrycomplete.div,'mouseover',entrycomplete.fn_mouseover);
				addDOMEvent(entrycomplete.div,'mouseout',entrycomplete.fn_mouseout);				
				addDOMEvent(entrycomplete.div,'click',entrycomplete.fn_callback);
				if(!entrycomplete.conditionactive(entrycomplete)) {
					entrycomplete.disabled(entrycomplete.div);
				} else {
					entrycomplete.enabled(entrycomplete.div);
				}
			}
		}
		var returnobject=new Object();
		var myobjects = contactsSelection2.getSelected();
		objectcache.get(myobjects,["created_by"],responseData,responseData);				
	}
	function count(myobjects) {
		var returnobject=new Object();
		returnobject.countall=0;
		for(var i in myobjects) {
			returnobject.countall++;
		}
		return returnobject.countall;
	}
	addContextNode(contextmenu_createImgDiv("Edit",getFullImgSrc("img/menu/edit.gif"),getFullImgSrc("img/menu/edit_d.gif")),["contacts"],function() { triggerEvent('OX_Contact_Edit'); },
	function(entryobject) { 
		var myobjects=entryobject.params.objects;
		if(count(myobjects) == 1 && menuhasRight(myobjects,"WRITE")) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,getdata,contextmenu_setEnabled);	
	addContextNode(contextmenu_createImgDiv("Delete",getFullImgSrc("img/menu/delete.gif"),getFullImgSrc("img/menu/delete_d.gif")),["contacts"],function() { triggerEvent('OX_Contact_Delete'); },
	function(entryobject) { 
		var myobjects=entryobject.params.objects;
		if(count(myobjects) > 0 && menuhasRight(myobjects,"DELETE")) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,getdata,contextmenu_setEnabled);	
	addContextNode(contextmenu_createImgDiv("Use as template",getFullImgSrc("img/menu/duplicate_contacts.gif"),getFullImgSrc("img/menu/duplicate_contacts_d.gif")),["contacts"],function() { triggerEvent('OX_Contact_Duplicate'); },
	function(entryobject) { 
		var myobjects=entryobject.params.objects;
		if(count(myobjects) == 1 && menuhasRight(myobjects,"WRITE")) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,getdata,contextmenu_setEnabled);	
}
function menuregisterContactContents()
{
	register("OX_Contact_Move",menucontactmoveclicked);
	register("OX_Contact_Copy",menucontactcopyclicked);
	register("OX_Switched",onmenuchangeHeightorSelectedContact);
	register("OX_SELECTED_ITEMS_CHANGED",onmenuchangeHeightorSelectedContact);
	register("OX_menu_Change_Height",onmenuchangeHeightorSelectedContact);
}
function menucontactmoveclicked()
{
	createMenuPopUpTree("menu_contacts_move_foldertree","contacts",doContactsFolder);
	document.getElementById("menu_contacts_move_foldertree").style.display="";
	menucontactfoldertreecopy=false;
} 
function menucontactcopyclicked()
{
	createMenuPopUpTree("menu_contacts_move_foldertree","contacts",doContactsFolder);
	document.getElementById("menu_contacts_move_foldertree").style.display="";
	menucontactfoldertreecopy=true;
} 
function doContactsFolder(ofolder)
{
	var selectedfolderid= ofolder.oxfolder.data.id;
	if(menucontactfoldertreecopy)
	{
		triggerEvent("OX_Object_Copy",selectedfolderid);
	} else {
		triggerEvent("OX_Object_Move",selectedfolderid);
	}
	document.getElementById("menu_contacts_move_foldertree").style.display="none";
}
function onmenuchangeHeightorSelectedContact()
{
	if(menuisActive("menu_contacts_views"))		
		onmenuchangeContactView(menuiteration,currentpath2[1],menucountselected);
	if(menuisActive("menu_contact"))				
		onmenuchangeHeightContactContent(menuiteration,menucountselected);
	if(menuisActive("menu_contact2"))				
		onmenuchangeHeightContact2Content(menuiteration,menucountselected);
}
function onmenuchangeContactView(iteration,value,selecteditems) {
	menuglobalzaehler=0;
	menuarrows["contacts"]["menu_contacts_views"]=new Array();
	menu_display_views("contacts","menu_contacts_views",true,value == "cards", "menu_contacts_views_cards");
	menu_display_views("contacts","menu_contacts_views",true,value == "phonelist", "menu_contacts_views_phonelist");
	menu_display_views("contacts","menu_contacts_views",selecteditems==1,value == "detail", "menu_contacts_views_detail");
}

function onmenuchangeHeightContact2Content(iteration,selecteditems)
{
	menuglobalzaehler=0;
	menuarrows["contacts"]["menu_contacts2"]=new Array();
	menu_display_contents("contacts","menu_contacts2",selecteditems == 1 && menucheckRight("WRITE"), "menu_contact_duplicate");
	menu_display_contents("contacts","menu_contacts2",selecteditems > 0 && menucheckRight("DELETE"), "menu_contact_move");
	menu_display_contents("contacts","menu_contacts2",selecteditems > 0 && menucheckRight("READ") , "menu_contact_copy");
}	
function onmenuchangeHeightContactContent(iteration,selecteditems)
{
	menuglobalzaehler=0;
	menuarrows["contacts"]["menu_contacts"]=new Array();
	menu_display_contents("contacts","menu_contacts",selecteditems == 1 && menucheckRight("WRITE"), "menu_contact_edit");
	menu_display_contents("contacts","menu_contacts",selecteditems > 0 && menucheckRight("DELETE"), "menu_contact_delete");
	menu_display_contents("contacts","menu_contacts",isPrintableView(currentpath2.join('/')), "menu_contact_print");
}
///////////////////////////////////////////////////////////////////////////////////////////////////////
/****************************************************************************************************/
/*** InfoStore Function ******************************************************************************/
/****************************************************************************************************/
menuarrows["infostore"]=new Array();

function menuinfostoreloaded() {
	addOnClose($('menu_infostore_move_foldertree'),function () { $('menu_infostore_move_foldertree').style.display="none"; });
	function getdata (entrycomplete) {
		function responseData(resp) {
			returnobject.objects=resp;
			entrycomplete.params=returnobject;		
			if(entrycomplete.conditionvisible(entrycomplete)) {
				addDOMEvent(entrycomplete.div,'mouseover',entrycomplete.fn_mouseover);
				addDOMEvent(entrycomplete.div,'mouseout',entrycomplete.fn_mouseout);				
				addDOMEvent(entrycomplete.div,'click',entrycomplete.fn_callback);
				if(!entrycomplete.conditionactive(entrycomplete)) {
					entrycomplete.disabled(entrycomplete.div);
				} else {
					entrycomplete.enabled(entrycomplete.div);
				}
			}
		}
		var returnobject=new Object();
		var myobjects = infostoreSelection.getSelected();
		objectcache.get(myobjects,["created_by","modified_by","locked_until","file_size","filename"],responseData,responseData);				
	}
	function hasAttachment(myobjects) {
		for(var i in myobjects) {
			if(!(myobjects[i]["file_size"]>0)) {
				return false;
			}
		}
		return true;
	}
	function checkRights(myobjects,type) {
		var returnobject=new Object();
		returnobject.countunlocked=0;
		returnobject.countlocked=0;
		returnobject.counteditlocked=0;
		returnobject.countall=0;
		for(var i in myobjects) {
			if(myobjects[i]["locked_until"]) {
				if(myobjects[i]["modified_by"] == configGetKey("identifier")) {
					returnobject.countunlocked ++;
					returnobject.counteditlocked ++;
				}
				else if(myobjects[i]["created_by"] == configGetKey("identifier")) {
					returnobject.countunlocked ++;
				}
			}
			else {
				returnobject.countlocked++;
			}
			returnobject.countall++;
		}
		switch(type) {
			case "EDIT" : 
				return (returnobject.countall == (returnobject.countlocked + returnobject.counteditlocked));
				break;	
			case "DELETE":
				return (returnobject.countall == (returnobject.countlocked));
				break;
			case "LOCK":  
				return (returnobject.countall == (returnobject.countlocked));
			case "UNLOCK":  
				return (returnobject.countall == returnobject.countunlocked);
			case "COUNT" :
				return returnobject.countall;
		}
	}
	addContextNode(contextmenu_createImgDiv("Edit",getFullImgSrc("img/menu/edit.gif"),getFullImgSrc("img/menu/edit_d.gif")),["infostore"],function() { triggerEvent('OX_InfoStore_Edit'); },
	function(entryobject) { 
		var myobjects=entryobject.params.objects;
		if(checkRights(myobjects,"COUNT") == 1 && menuhasRight(myobjects,"WRITE") && checkRights(myobjects,"EDIT")) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,getdata,contextmenu_setEnabled);
	addContextNode(contextmenu_createImgDiv("Delete",getFullImgSrc("img/menu/delete.gif"),getFullImgSrc("img/menu/delete_d.gif")),["infostore"],function() { triggerEvent('OX_InfoStore_Delete'); },
	function(entryobject) { 
		var myobjects=entryobject.params.objects;
		if(checkRights(myobjects,"COUNT") > 0 && menuhasRight(myobjects,"DELETE") && checkRights(myobjects,"DELETE")) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,getdata,contextmenu_setEnabled);
	addContextLine(null,["infostore"]);
	addContextNode(contextmenu_createImgDiv("Lock",getFullImgSrc("img/menu/sperren.gif"),getFullImgSrc("img/menu/sperren_d.gif")),["infostore"],function() { triggerEvent('OX_InfoStore_Lock'); },
	function(entryobject) { 
		var myobjects=entryobject.params.objects;
		if(checkRights(myobjects,"COUNT") > 0 && menuhasRight(myobjects,"WRITE") && checkRights(myobjects,"LOCK")) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,getdata,contextmenu_setEnabled);
	addContextNode(contextmenu_createImgDiv("Unlock",getFullImgSrc("img/menu/entsperren.gif"),getFullImgSrc("img/menu/entsperren_d.gif")),["infostore"],function() { triggerEvent('OX_InfoStore_Unlock'); },
	function(entryobject) { 
		var myobjects=entryobject.params.objects;
		if(checkRights(myobjects,"COUNT") > 0 && menuhasRight(myobjects,"WRITE") && checkRights(myobjects,"UNLOCK")) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,getdata,contextmenu_setEnabled);
	addContextLine(null,["infostore"]);
	addContextNode(contextmenu_createImgDiv("Send as attachment",getFullImgSrc("img/menu/alsanlagesenden.gif"),getFullImgSrc("img/menu/alsanlagesenden_d.gif")),["infostore"],function() { triggerEvent('OX_InfoStore_Send_Attachment'); },
	function(entryobject) { 
		var myobjects=entryobject.params.objects;
//		if(checkRights(myobjects,"COUNT") > 0 && menuhasRight(myobjects,"READ") && hasAttachment(myobjects)) { return true; }
		if(checkRights(myobjects,"COUNT") > 0 && menuhasRight(myobjects,"READ")) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,getdata,contextmenu_setEnabled);
	var tmp="Send as link"; /*i18n*/
	tmp="Send as attachment"; /*i18n*/
	addContextNode(contextmenu_createImgDiv("Send as link",	getFullImgSrc("img/menu/save_as_link.gif"),getFullImgSrc("img/menu/save_as_link_d.gif")),["infostore"],function() { triggerEvent('OX_InfoStore_Send_Link'); },
	function(entryobject) { 
		var myobjects=entryobject.params.objects;
		if(checkRights(myobjects,"COUNT") > 0 && menuhasRight(myobjects,"READ")) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,getdata,contextmenu_setEnabled);
	addContextNode(contextmenu_createImgDiv("Open",getFullImgSrc("img/mail/open_ISattachment.gif"),getFullImgSrc("img/mail/open_ISattachment_d.gif")),["infostore"],function(e,entryobject) { triggerEvent('OX_InfoStore_Open_Attachment',entryobject.params.objects); },
	function(entryobject) { 
		var myobjects=entryobject.params.objects;
		if(checkRights(myobjects,"COUNT") == 1 && menuhasRight(myobjects,"READ") && hasAttachment(myobjects)) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,getdata,contextmenu_setEnabled);
}
function menuregisterInfoStoreContents()
{
	register("OX_InfoStore_Move",menuinfostoremoveclicked);
	register("OX_Switched",onmenuchangeHeightorSelectedInfoStore);
	register("OX_SELECTED_ITEMS_CHANGED",onmenuchangeHeightorSelectedInfoStore);
	register("OX_menu_Change_Height",onmenuchangeHeightorSelectedInfoStore);
	register("SubSelected",onmenuchangeSubSelectedInfoStore);
	register("OX_Search_InfoStore",onmenusearchInfoStore);
	register("OX_Lockable",onmenuchangeLocks);
}
function onmenusearchInfoStore() 
{
	var mysearchparam=document.getElementById("menu_search_infostore_searchfield").value;
	var mysearch=new Object();
	mysearch["pattern"]=mysearchparam;
	triggerEvent("OX_InfoStore_Search",mysearch);
}
function menuinfostoremoveclicked()
{
	register("OX_InfoStore_Move",menuinfostoremoveclicked);
	createMenuPopUpTree("menu_infostore_move_foldertree","infostore",moveInfoStoreFolder);
	document.getElementById("menu_infostore_move_foldertree").style.display="";
} 
function moveInfoStoreFolder(ofolder)
{
	var selectedfolderid= ofolder.oxfolder.data.id;
	triggerEvent("OX_Object_Move",selectedfolderid);
	document.getElementById("menu_infostore_move_foldertree").style.display="none";
}
function onmenuchangeLocks() {
	onmenuchangeHeightorSelectedInfoStore();
}
function onmenuchangeHeightorSelectedInfoStore()
{
	if(menuisActive("menu_infostore"))
		onmenuchangeHeightInfoStoreMainContent(menuiteration,menucountselected);
	if(menuisActive("menu_infostore2"))
		onmenuchangeHeightInfoStoreMainContent2(menuiteration,menucountselected);
	if(menuisActive("menu_infostore_views"))
		onmenuchangeInfoStoreView(menuiteration,currentpath2[1],menucountselected);
	if(menuisActive("menu_infostore_send"))		
		onmenuchangeHeightInfoStoreSendContent(menuiteration,menucountselected);
	if(menuisActive("menu_global_tags"))
		onmenuchangeHeightGlobalTagsContent(menuiteration,menucountselected);
	if(menuisActive("menu_infostore_version"))
		onmenuchangeHeightInfoStoreVersionContent(menuiteration,menucountsubselected);
	
}
function onmenuchangeSubSelectedInfoStore(selected)
{
	if(menuisActive("menu_infostore_version"))
		onmenuchangeHeightInfoStoreVersionContent(menuiteration,selected);
	
}
function onmenuchangeInfoStoreView(iteration,value,selecteditems) {
	menuglobalzaehler=0;
	menuarrows["infostore"]["menu_infostore_views"]=new Array();
    menu_display_views("infostore","menu_infostore_views",true,value == "split", "menu_infostore_views_split");
	menu_display_views("infostore","menu_infostore_views",true,value == "list", "menu_infostore_views_list");	
//	triggerEvent("OX_Switched",value);
}
function onmenuchangeHeightInfoStoreVersionContent(iteration,selecteditems)
{
	menuglobalzaehler=0;
	menuarrows["infostore"]["menu_infostore_version"]=new Array();
	menu_display_contents("infostore","menu_infostore_version",selecteditems == 1 && menucheckRight("WRITE") && (menucountselected == menucountlockable+menucounteditable), "menu_infostore_version_current"); //TODO:CHECK RIGHT WRITE
	menu_display_contents("infostore","menu_infostore_version",selecteditems > 0 && menucheckRight("WRITE") && (menucountselected == menucountlockable+menucounteditable), "menu_infostore_version_delete"); //TODO:CHECK RIGHT WRITE
}
function onmenuchangeHeightInfoStoreMainContent2(iteration,selecteditems)
{
	menuglobalzaehler=0;
	menuarrows["infostore"]["menu_infostore2"]=new Array();
	menu_display_contents("infostore","menu_infostore2",selecteditems > 0 && menucheckRight("DELETE") && (selecteditems == menucountlockable+menucounteditable), "menu_infostore_move"); 
	menu_display_contents("infostore","menu_infostore2",selecteditems > 0 && menucheckRight("WRITE") && (menucountlockable==selecteditems), "menu_infostore_log"); 
	menu_display_contents("infostore","menu_infostore2",selecteditems > 0 && menucheckRight("WRITE") && (menucountunlockable==selecteditems), "menu_infostore_unlog"); 
}
function onmenuchangeHeightInfoStoreMainContent(iteration,selecteditems)
{
	menuglobalzaehler=0;
	menuarrows["infostore"]["menu_infostore"]=new Array();
	menu_display_contents("infostore","menu_infostore",selecteditems == 1 && menucheckRight("WRITE") && (selecteditems == menucountlockable+menucounteditable), "menu_infostore_edit"); 
	menu_display_contents("infostore","menu_infostore",selecteditems > 0 && menucheckRight("DELETE") && (selecteditems == menucountlockable+menucounteditable), "menu_infostore_delete"); 
	menu_display_contents("infostore","menu_infostore",isPrintableView(currentpath2.join('/')), "menu_infostore_print"); 
}
function onmenuchangeHeightInfoStoreSendContent(iteration,selecteditems) {
	menuglobalzaehler=0;
	menuarrows["infostore"]["menu_infostore"]=new Array();
	menu_display_contents("infostore","menu_infostore_send", configGetKey("modules.mail.module") && selecteditems > 0, "menu_infostore_send_attachment"); 
	menu_display_contents("infostore","menu_infostore_send", configGetKey("modules.mail.module") && selecteditems > 0, "menu_infostore_send_link"); 
}
///////////////////////////////////////////////////////////////////////////////////////////////////////
/****************************************************************************************************/
/*** Global Function ******************************************************************************/
/****************************************************************************************************/

var linkingactive 				= false;
var linkingmodulefirst			= "";
var linkingfolderfirst			= "";
var linkingidfirst				= "";
menuarrows["global"]=new Array();

function menugloballoaded()
{
	createEnhancementMenu("menu_global_tags_enhancement","menu_global_tags","global");
	createEnhancementMenu("menu_global_attachments_enhancement","menu_global_attachments","global");
	function checkRights(myobjects,type) {
		switch(type) {
			case "READ" : 
				return (menucountsubselected == 1);
				break;	
		}
	}
	addContextNode(contextmenu_createImgDiv("Open",getFullImgSrc("img/menu/attachment_open.gif"),getFullImgSrc("img/menu/attachment_open_d.gif")),["attachment"],function() { menu_doAttachmentOpen(); },
	function(entryobject) { 
		if(checkRights(entryobject.params,"READ") && menucheckRight("READ") && !menudisableattachment) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);
	addContextNode(contextmenu_createImgDiv("Save as",getFullImgSrc("img/menu/attachment_save.gif"),getFullImgSrc("img/menu/attachment_save_d.gif")),["attachment"],function() { triggerEvent('OX_Attachment_Save'); },
	function(entryobject) { 
		if(checkRights(entryobject.params,"READ") && menucheckRight("READ") && !menudisableattachment) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);
	addContextNode(contextmenu_createImgDiv("Save in InfoStore",getFullImgSrc("img/infostore/infostore_save.gif"),getFullImgSrc("img/infostore/infostore_save_d.gif")),["attachment"],function() { triggerEvent('OX_Attachment_SaveInfoStore'); },
	function(entryobject) { 
		if(configGetKey("modules.infostore.module") && checkRights(entryobject.params,"READ") && menucheckRight("READ") && !menudisableattachment) { return true; }
		return false;
	},null,contextmenu_setDisabled,null,null,null,contextmenu_setEnabled,null,
	function() { 
	       if (!configGetKey("modules.infostore.module")) { 
	           triggerEvent("Feature_Not_Available", "modules/infostore/mail/save_to_infostore"); 
	       } 
        });
	
}
function onmenuchangeHeightorSelectedGlobal()
{
	onmenuchangeHeightorSelectedTags();
	menuLinkungHeightChangeContent(menuiteration,menucountselected);
}
function onmenuchangeHeightorSelectedTags() {
	onmenuchangeHeightGlobalTagsContent(menuiteration,menucountselected);	
}
function menuLinkungHeightChangeContent(iteration,selecteditems) {
	if(linkingactive) {
		$("linkingheaderactive").style.display="";
		$("linkingheaderna").style.display="none";
	} else {
		$("linkingheaderactive").style.display="none";
		$("linkingheaderna").style.display="";
	}
	menuglobalzaehler=0;
	menuarrows["global"]["menu_linking"]=new Array();
	menu_display_contents("global","menu_linking",selecteditems == 1 && menucheckRight("READ") && !linkingactive, "menu_linking_start"); 
	menu_display_contents("global","menu_linking",selecteditems == 1 && menucheckRight("READ") && linkingactive, "menu_linking_close"); 
	menu_display_contents("global","menu_linking",menucountsubselectedlinking == 1, "menu_linking_follow");
	menu_display_contents("global","menu_linking",linkingactive, "menu_linking_cancel"); 
	menu_display_contents("global","menu_linking",menucountsubselectedlinking > 0 , "menu_linking_delete");
}
function menulinkingstartlinking(linkedmodule,linkedfolder,linkedid)
{
	linkingactive 				= true;
	linkingmodulefirst			= linkedmodule;
	linkingfolderfirst			= linkedfolder;
	linkingidfirst				= linkedid;	
	menuLinkungHeightChangeContent(menuiteration,menucountselected);
	triggerEvent("OX_NEW_LINKING_STARTED");		
}
function menulinkingcloselinking(linkedmodule,linkedfolder,linkedid)
{
	if((linkingmodulefirst == linkedmodule) && (linkingfolderfirst == linkedfolder) && (linkingidfirst == linkedid))
	{
		triggerEvent("OX_New_Error",4,_("Can not link an object to itself"));
	}
	else
	{
		saveLink(linkingmodulefirst,linkingfolderfirst,linkingidfirst,linkedmodule,linkedfolder,linkedid);
		triggerEvent("OX_SAVE_NEW_LINK",linkingmodulefirst,linkingfolderfirst,linkingidfirst,linkedmodule,linkedfolder,linkedid);
		linkingactive 				= false;
		linkingmodulefirst			= "";
		linkingfolderfirst			= "";
		linkingidfirst				= "";	
		menuLinkungHeightChangeContent(menuiteration,menucountselected);
		triggerEvent("OX_NEW_LINKING_ENDED");
	}
}
function saveLink(lmf,lff,lif,lms,lfs,lis) {
	var calendarint = 1;
	var taskint = 4;
	var contactint = 7;
	var projectint = 13;
	var forumint = 17;
	var pinboardint = 18;
	var emailint = 19;
	var infostoreint = 137;
	
	var lmfint;
	var lmsint;
	switch (lmf) {
		case "calendar" : lmfint = calendarint; break;
		case "tasks" : lmfint = taskint; break;
		case "contacts" : lmfint = contactint; break;
		case "mail" : lmfint = emailint; break;
		case "infostore" : lmfint = infostoreint; break;
		default: triggerEvent("OX_New_Error",4,"Unknown Module Type"+ lmf);
	}
	switch (lms) {
		case "calendar" : lmsint = calendarint; break;
		case "tasks" : lmsint = taskint; break;
		case "contacts" : lmsint = contactint; break;
		case "mail" : lmsint = emailint; break;
		case "infostore" : lmsint = infostoreint; break;
		default: triggerEvent("OX_New_Error",4,"Unknown Module Type"+ lms);
	}
	var linkingJSON = new JSON();
	var linkingRequest = AjaxRoot + "/link?action=new&session=" + session;
	var requestObject = new Object();
	requestObject["id1"]=lif;
	requestObject["module1"]=lmfint;
	requestObject["folder1"]=lff;
	requestObject["id2"]=lis;
	requestObject["module2"]=lmsint;
	requestObject["folder2"]=lfs;
	linkingJSON.put(linkingRequest, requestObject, null,
	function callback(response) {
		if(response.error) {
			//triggerEvent("OX_New_Error",4,_("This link already exists"));
		}
		else
		{
			triggerEvent("OX_New_Info",4,_("Link established"));
		}
	});
}

function menulinkingcancel() {
	linkingactive 				= false;
	linkingmodulefirst			= "";
	linkingfolderfirst			= "";
	linkingidfirst				= "";	
	menuLinkungHeightChangeContent(menuiteration,menucountselected);
}
function doLinking(start) {
	if(menuselectedobjects.length!=1) {
		if(menuselectedobjects.length<1) {
			triggerEvent("OX_New_Error",4,_("No objects selected"));
		} else {	
			triggerEvent("OX_New_Error",4,_("To much selected objects")+" "+selected.length);
		}
	} else {
		if(start) {
			menulinkingstartlinking(activemodule,menuselectedobjects[0].folder || menuselectedobjects[0].folder_id ,menuselectedobjects[0].id);
		} else {
			menulinkingcloselinking(activemodule,menuselectedobjects[0].folder || menuselectedobjects[0].folder_id,menuselectedobjects[0].id);
		}
	}
}

function changeConfirm() {
	if(menuisActive("menu_global_confirmation")) {
		onmenuchangeHeightGlobalConfirmContent(menuiteration,menucountconfirm);
	}
}
function changeAttachments(select)
{
	if(menuisActive("menu_global_attachments")) {
		onmenuchangeHeightGlobalAttachmentsContent(menuiteration,menucountsubselected);
	}
}
function onmenuchangeHeightGlobalConfirmContent(menuiteration,selecteditems) {
	menuglobalzaehler=0;
	menuarrows["global"]["menu_global_confirmation"] = new Array();
	menu_display_contents("global","menu_global_confirmation",selecteditems == 1, "menu_global_confirmation_change"); 
}
function onmenuchangeHeightGlobalAttachmentsContent(iteration,selecteditems)
{
	menuglobalzaehler=0;
	menuarrows["global"]["menu_global_attachments"] = new Array();
	menu_display_contents("global","menu_global_attachments",selecteditems == 1 && menucheckRight("READ") && !menudisableattachment , "menu_global_attachments_open"); 
	menu_display_contents("global","menu_global_attachments",selecteditems == 1 && menucheckRight("READ") && !menudisableattachment, "menu_global_attachments_save"); 
	menu_display_contents("global","menu_global_attachments",configGetKey("modules.infostore.module") && selecteditems == 1 && menucheckRight("READ") && !menudisableattachment, "menu_global_attachments_saveinfostore"); 
}	
function menu_doAttachmentOpen() {
	(menunestedmessage) ? triggerEvent("OX_Attachment_Open_Nested") : triggerEvent("OX_Attachment_Open");
}
function onmenuchangeHeightGlobalTagsContent(iteration,selecteditems){
	
	menuglobalzaehler=0;
	menuarrows["global"]["menu_global_tags"]=new Array();
	menu_display_contents("global","menu_global_tags",selecteditems > 0 && menucheckRight("WRITE") && ((activemodule!="infostore" && activemodule != "mail" && activemodule != "mail_detail" && true) || (activemodule=="infostore" && (selecteditems == menucountlockable+menucounteditable) || ((activemodule == "mail" || activemodule == "mail_detail" && !isNestedMailMessage()) && menucheckRight("USER")))), "menu_global_tags_tag0"); 
	menu_display_contents("global","menu_global_tags",selecteditems > 0 && menucheckRight("WRITE") && ((activemodule!="infostore" && activemodule != "mail" && activemodule != "mail_detail" && true) || (activemodule=="infostore" && (selecteditems == menucountlockable+menucounteditable) || ((activemodule == "mail" || activemodule == "mail_detail" && !isNestedMailMessage()) && menucheckRight("USER")))), "menu_global_tags_tag1"); 
	menu_display_contents("global","menu_global_tags",selecteditems > 0 && menucheckRight("WRITE") && ((activemodule!="infostore" && activemodule != "mail" && activemodule != "mail_detail" && true) || (activemodule=="infostore" && (selecteditems == menucountlockable+menucounteditable) || ((activemodule == "mail" || activemodule == "mail_detail" && !isNestedMailMessage()) && menucheckRight("USER")))), "menu_global_tags_tag2"); 
	menu_display_contents("global","menu_global_tags",selecteditems > 0 && menucheckRight("WRITE") && ((activemodule!="infostore" && activemodule != "mail" && activemodule != "mail_detail" && true) || (activemodule=="infostore" && (selecteditems == menucountlockable+menucounteditable) || ((activemodule == "mail" || activemodule == "mail_detail" && !isNestedMailMessage()) && menucheckRight("USER")))), "menu_global_tags_tag3"); 
	menu_display_contents("global","menu_global_tags",selecteditems > 0 && menucheckRight("WRITE") && ((activemodule!="infostore" && activemodule != "mail" && activemodule != "mail_detail" && true) || (activemodule=="infostore" && (selecteditems == menucountlockable+menucounteditable) || ((activemodule == "mail" || activemodule == "mail_detail" && !isNestedMailMessage()) && menucheckRight("USER")))), "menu_global_tags_tag4"); 
	menu_display_contents("global","menu_global_tags",selecteditems > 0 && menucheckRight("WRITE") && ((activemodule!="infostore" && activemodule != "mail" && activemodule != "mail_detail" && true) || (activemodule=="infostore" && (selecteditems == menucountlockable+menucounteditable) || ((activemodule == "mail" || activemodule == "mail_detail" && !isNestedMailMessage()) && menucheckRight("USER")))), "menu_global_tags_tag5"); 
	menu_display_contents("global","menu_global_tags",selecteditems > 0 && menucheckRight("WRITE") && ((activemodule!="infostore" && activemodule != "mail" && activemodule != "mail_detail" && true) || (activemodule=="infostore" && (selecteditems == menucountlockable+menucounteditable) || ((activemodule == "mail" || activemodule == "mail_detail" && !isNestedMailMessage()) && menucheckRight("USER")))), "menu_global_tags_tag6"); 
	menu_display_contents("global","menu_global_tags",selecteditems > 0 && menucheckRight("WRITE") && ((activemodule!="infostore" && activemodule != "mail" && activemodule != "mail_detail" && true) || (activemodule=="infostore" && (selecteditems == menucountlockable+menucounteditable) || ((activemodule == "mail" || activemodule == "mail_detail" && !isNestedMailMessage()) && menucheckRight("USER")))), "menu_global_tags_tag7"); 
	menu_display_contents("global","menu_global_tags",selecteditems > 0 && menucheckRight("WRITE") && ((activemodule!="infostore" && activemodule != "mail" && activemodule != "mail_detail" && true) || (activemodule=="infostore" && (selecteditems == menucountlockable+menucounteditable) || ((activemodule == "mail" || activemodule == "mail_detail" && !isNestedMailMessage()) && menucheckRight("USER")))), "menu_global_tags_tag8"); 
	menu_display_contents("global","menu_global_tags",selecteditems > 0 && menucheckRight("WRITE") && ((activemodule!="infostore" && activemodule != "mail" && activemodule != "mail_detail" && true) || (activemodule=="infostore" && (selecteditems == menucountlockable+menucounteditable) || ((activemodule == "mail" || activemodule == "mail_detail" && !isNestedMailMessage()) && menucheckRight("USER")))), "menu_global_tags_tag9"); 
	menu_display_contents("global","menu_global_tags",selecteditems > 0 && menucheckRight("WRITE") && ((activemodule!="infostore" && activemodule != "mail" && activemodule != "mail_detail" && true) || (activemodule=="infostore" && (selecteditems == menucountlockable+menucounteditable) || ((activemodule == "mail" || activemodule == "mail_detail" && !isNestedMailMessage()) && menucheckRight("USER")))), "menu_global_tags_tag10"); 
}
/****************************************************************************************************/
/*** Configuration Function ******************************************************************************/
/****************************************************************************************************/
menuarrows["configuration"]=new Array();
function menuconfigurationloaded()
{
	addOnClose($('menu_configuration_folder_folderpopup'),function () { $('menu_configuration_folder_folderpopup').style.display="none"; });
	createEnhancementMenu("menu_configuration_adminright_change_enhancement","menu_configuration_adminright","configuration");
	createEnhancementMenu("menu_configuration_folderright_folder_enhancement","menu_configuration_folderright_folder","configuration");
	createEnhancementMenu("menu_configuration_objectright_delete_enhancement","menu_configuration_objectright_delete","configuration");
	createEnhancementMenu("menu_configuration_objectright_read_enhancement","menu_configuration_objectright_read","configuration");
	createEnhancementMenu("menu_configuration_objectright_write_enhancement","menu_configuration_objectright_write","configuration");
}
function menuregisterConfigurationContents()
{
	register("OX_Configuration_Change_Right",onmenuchangeRight);
	register("OX_Configuration_Move_Folder",menuconfigurationfolderclicked);
	register("OX_Switched",changeRightHeightorSelectedConfigurationFolder);
	register("SubSelected",changeRightHeightorSelectedConfigurationFolder);
	register("OX_Config_Folder_Changed",changeRightHeightorSelectedConfigurationFolder);
	register("OX_Show_Rights",setRights);
	register("OX_menu_Change_Height",changeRightHeightorSelectedConfigurationFolder);
	register("OX_Switched",configurationmailsignatureschangeHeight);
	register("SubSelected",configurationmailsignatureschangeHeight);
	register("OX_Switched",configurationportalchangeHeight);
	register("SubSelected",configurationportalchangeHeight);
	register("OX_Switched",configurationsettingscategorieschangeHeight);
	register("SubSelected",configurationsettingscategorieschangeHeight);
	register("OX_Switched",configurationmailsignatureschangeHeight);
	register("OX_Switched",configurationmailteamschangeHeight);
	register("SubSelected",configurationmailteamschangeHeight);
	register("SubSelectedTeamMember",configurationmailteamschangeHeight);
	register("OX_menu_Change_Height",configurationmailsignatureschangeHeight);
}	
function menuconfigurationfolderclicked(){
	createMenuPopUpTree("menu_configuration_folder_folderpopup","folder",menu_moveFolder);
	document.getElementById("menu_configuration_folder_folderpopup").style.display="";
} 
function menu_moveFolder(ofolder) {
	var selectedfolderid= ofolder.oxfolder.data.id;
	triggerEvent("OX_Folder_Move",ofolder);
	document.getElementById("menu_configuration_folder_folderpopup").style.display="none";
	
}
function configurationsettingscategorieschangeHeight() {
	if(menuisActive("menu_configuration_settings_categories"))
			onmenuchangeHeightConfigurationSettingsCategories(menuiteration,menucountsubselected);
}
function onmenuchangeHeightConfigurationSettingsCategories(menuiteration,selecteditems) {
	menuglobalzaehler=0;
	menuarrows["configuration"]["menu_configuration_settings_categories"] = new Array();
	menu_display_contents("configuration","menu_configuration_settings_categories",true, "menu_configuration_settings_categories_add"); 
	menu_display_contents("configuration","menu_configuration_settings_categories",selecteditems==1, "menu_configuration_settings_categories_edit"); 
	menu_display_contents("configuration","menu_configuration_settings_categories",selecteditems>0, "menu_configuration_settings_categories_remove"); 
	
}
function configurationportalchangeHeight() {
	if(menuisActive("menu_config_portal_external")) 
		onmenuchangeHeightConfigurationPortalSettings(menuiteration,menucountsubselected);
}
function configurationmailsignatureschangeHeight()
{
	if(menuisActive("menu_config_mail_signatures")) 
		onmenuchangeHeightConfigurationMailSignatures(menuiteration,menucountsubselected);
}
function configurationmailteamschangeHeight() {
	if(menuisActive("menu_configuration_teams_teams")) 
		onmenuchangeHeightConfigurationTeamsTeamsSettings(menuiteration,menucountsubselected);
}
function onmenuchangeHeightConfigurationMailSignatures(iteration,selecteditems)
{
	menuglobalzaehler=0;
	menuarrows["configuration"]["menu_config_mail_signatures"] = new Array();
	menu_display_contents("configuration","menu_config_mail_signatures",true, "menu_config_mail_signatures_add_signature"); 
	menu_display_contents("configuration","menu_config_mail_signatures",selecteditems == 1, "menu_config_mail_signatures_edit_signature"); 
	menu_display_contents("configuration","menu_config_mail_signatures",selecteditems > 0, "menu_config_mail_signatures_remove_signature"); 
}	
function onmenuchangeHeightConfigurationPortalSettings(menuiteration,selecteditems) {
	menuglobalzaehler=0;
	menuarrows["configuration"]["menu_config_portal_external"] = new Array();
	menu_display_contents("configuration","menu_config_portal_external",true, "menu_config_portal_external_add"); 
	menu_display_contents("configuration","menu_config_portal_external",(selecteditems == 1), "menu_config_portal_external_edit"); 				
	menu_display_contents("configuration","menu_config_portal_external",selecteditems > 0, "menu_config_portal_external_remove"); 
	menuglobalzaehler=0;	
	menu_display_contents("configuration","menu_config_portal_external",selecteditems > 0, "menu_config_portal_external_enable"); 	
	menu_display_contents("configuration","menu_config_portal_external",selecteditems > 0, "menu_config_portal_external_disable"); 		
}
function onmenuchangeHeightConfigurationTeamsTeamsSettings(menuiteration,selecteditems) {
	menuglobalzaehler=0;
	menuarrows["configuration"]["menu_configuration_teams_teams"] = new Array();
	menu_display_contents("configuration","menu_configuration_teams_teams",true, "menu_configuration_teams_teams_add_team"); 
	menu_display_contents("configuration","menu_configuration_teams_teams",selecteditems == 1, "menu_configuration_teams_teams_edit_team"); 
	menu_display_contents("configuration","menu_configuration_teams_teams",selecteditems > 0, "menu_configuration_teams_teams_remove_team"); 
}
/****************************************************************************************************/
/*** Folder Functions Function ******************************************************************************/
/****************************************************************************************************/

rightsstate = {
	OXRIGHTSREAD: 0,
	OXRIGHTSWRITE: 0,
	OXRIGHTSDELETE: 0,
	OXRIGHTSFOLDER: 0
};

function setRights(var1,var2,var3,var4,var5)
{
	onmenuchangeRight("FOLDER",var1);
	onmenuchangeRight("READ",var2);
	onmenuchangeRight("WRITE",var3);
	onmenuchangeRight("DELETE",var4);
	onmenuchangeRight("ADMIN",var5);
}
function changeRightHeightorSelectedConfigurationFolder()
{
	if(menuisActive("menu_configuration_objectright_read")) 
		onmenuchangeHeightRightsContentReadObject(menuiteration,rightsstate["OXRIGHTSREAD"],menucountsubselected);
	if(menuisActive("menu_configuration_objectright_write")) 
		onmenuchangeHeightRightsContentWriteObject(menuiteration,rightsstate["OXRIGHTSWRITE"],menucountsubselected);
	if(menuisActive("menu_configuration_objectright_delete")) 
		onmenuchangeHeightRightsContentDeleteObject(menuiteration,rightsstate["OXRIGHTSDELETE"],menucountsubselected);
	if(menuisActive("menu_configuration_folderright_folder")) 
		onmenuchangeHeightRightsContentFolderObject(menuiteration,rightsstate["OXRIGHTSFOLDER"],menucountsubselected);
	if(menuisActive("menu_configuration_adminright")) 
		onmenuchangeHeightRightsChangeRights(menuiteration,rightsstate["OXRIGHTSADMIN"],menucountsubselected);
	if(menuisActive("menu_configuration_user")) {
		onmenuchangeHeightRightsTeamUser(menuiteration,null,menucountsubselected);
	}
	if(menuisActive("menu_configuration_folder")) {
		onmenuchangeHeightRightsFolder(menuiteration,null,menucountsubselected);
	}
	
}

function onmenuchangeRight(righttype,value)
{
	if(righttype=="READ") {
		rightsstate["OXRIGHTSREAD"]=value;
		onmenuchangeHeightRightsContentReadObject(menuiteration,rightsstate["OXRIGHTSREAD"],menucountsubselected);
	} else if(righttype=="WRITE") {
		rightsstate["OXRIGHTSWRITE"]=value;
		onmenuchangeHeightRightsContentWriteObject(menuiteration,rightsstate["OXRIGHTSWRITE"],menucountsubselected);
	} else if(righttype=="DELETE") {
		rightsstate["OXRIGHTSDELETE"]=value;
		onmenuchangeHeightRightsContentDeleteObject(menuiteration,rightsstate["OXRIGHTSDELETE"],menucountsubselected);
	} else if(righttype=="FOLDER") {
		rightsstate["OXRIGHTSFOLDER"]=value;
		onmenuchangeHeightRightsContentFolderObject(menuiteration,rightsstate["OXRIGHTSFOLDER"],menucountsubselected);
	} else if(righttype=="ADMIN") {
		rightsstate["OXRIGHTSADMIN"]=value;
		onmenuchangeHeightRightsChangeRights(menuiteration,rightsstate["OXRIGHTSADMIN"],menucountsubselected);
	}
}
function onmenuchangeHeightRightsTeamUser(iteration,value,selecteditems) {
	menuarrows["configuration"]["menu_configuration_user"]=new Array();
	menuglobalzaehler=0;
	oMainFolderTree.cache.get_folder(configfolder,function (oFolder){
		var haveadminrights = (configparentfolder != -1 && getFolderRightCreateSubFolders(oFolder)) 
				|| (configparentfolder == -1 && getFolderRightFolderAdmin(oFolder));
				
				
		//check here shared functionality and enable add user button, if the shared folder functionality is enabled. 
		var haveadminrights2 = (configGetKey("modules.folder.read_create_shared_folders") 
					|| (!configGetKey("modules.folder.read_create_shared_folders") && configGetKey("modules.folder.public_folders") && oFolder.oxfolder.data.type==5));
		
		menu_display_contents("configuration","menu_configuration_user",haveadminrights && haveadminrights2, "menu_configuration_user_add"); 
		menu_display_contents("configuration","menu_configuration_user",haveadminrights && haveadminrights2 && selecteditems>0 , "menu_configuration_user_remove"); 
	});
}
function onmenuchangeHeightRightsFolder(iteration,value,selecteditems) {
	menuarrows["configuration"]["menu_configuration_folder"]=new Array();
	menuglobalzaehler=0;
	oMainFolderTree.cache.get_folder(configfolder,function (oFolder){
		var haveadminrights = (configparentfolder == -1 && getFolderRightFolderAdmin(oFolder));
		menu_display_contents("configuration","menu_configuration_folder",haveadminrights, "menu_configuration_folder_move"); 
		menu_display_contents("configuration","menu_configuration_folder",haveadminrights, "menu_configuration_folder_delete"); 
	});
}

function onmenuchangeHeightRightsChangeRights(iteration,value,selecteditems)
{
	menuarrows["configuration"]["menu_configuration_adminright"]=new Array();
	menuglobalzaehler=0;
	oMainFolderTree.cache.get_folder(configfolder,function (oFolder){
		var haveadminrights = (configparentfolder != -1 && getFolderRightCreateSubFolders(oFolder)) || (configparentfolder == -1
																			&& getFolderRightFolderAdmin(oFolder));
	var noprivatefolder = (configfolder != 1 && oFolder.oxfolder.data.type != 1) 
	menu_display_views("configuration","menu_configuration_adminright",selecteditems == 1 && haveadminrights && noprivatefolder,value == 0,"menu_configuration_adminright_none"); 
	menu_display_views("configuration","menu_configuration_adminright",selecteditems == 1 && haveadminrights && noprivatefolder,value == 1,"menu_configuration_adminright_admin"); 
	});
}
function onmenuchangeHeightRightsContentFolderObject(iteration,value,selecteditems)
{
	menuarrows["configuration"]["menu_configuration_folderright_folder"]=new Array();
	menuglobalzaehler=0;
	oMainFolderTree.cache.get_folder(configfolder,function (oFolder){
		var haveadminrights = (configparentfolder != -1 && getFolderRightCreateSubFolders(oFolder)) 
													|| (configparentfolder == -1 && getFolderRightFolderAdmin(oFolder));
		menu_display_views("configuration","menu_configuration_folderright_folder",selecteditems == 1 && haveadminrights,value == 0,"menu_configuration_folderright_folder_none"); 
		menu_display_views("configuration","menu_configuration_folderright_folder",selecteditems == 1 && haveadminrights,value == 1,"menu_configuration_folderright_folder_visible"); 
		menu_display_views("configuration","menu_configuration_folderright_folder",selecteditems == 1 && haveadminrights,value == 2,"menu_configuration_folderright_folder_visible_objects"); 
		menu_display_views("configuration","menu_configuration_folderright_folder",selecteditems == 1 && haveadminrights,value == 4,"menu_configuration_folderright_folder_create_subfolder"); 
		menu_display_views("configuration","menu_configuration_folderright_folder",selecteditems == 1 && haveadminrights,value == 64,"menu_configuration_folderright_folder_admin"); 	
	});
}
function onmenuchangeHeightRightsContentDeleteObject(iteration,value,selecteditems)
{
	menuglobalzaehler=0;
	menuarrows["configuration"]["menu_configuration_objectright_delete"]=new Array();
	oMainFolderTree.cache.get_folder(configfolder,function (oFolder){
		var haveadminrights = (configparentfolder != -1 && getFolderRightCreateSubFolders(oFolder)) 
													|| (configparentfolder == -1 && getFolderRightFolderAdmin(oFolder));
		menu_display_views("configuration","menu_configuration_objectright_delete",selecteditems == 1 && haveadminrights,value == 0,"menu_configuration_objectright_delete_none"); 
		menu_display_views("configuration","menu_configuration_objectright_delete",selecteditems == 1 && haveadminrights,value == 1,"menu_configuration_objectright_delete_own"); 
		menu_display_views("configuration","menu_configuration_objectright_delete",selecteditems == 1 && haveadminrights,value == 2,"menu_configuration_objectright_delete_all"); 
		menu_display_views("configuration","menu_configuration_objectright_delete",selecteditems == 1 && haveadminrights,value == 64,"menu_configuration_objectright_delete_admin"); 
	});

}
function onmenuchangeHeightRightsContentWriteObject(iteration,value,selecteditems)
{
	menuglobalzaehler=0;
	menuarrows["configuration"]["menu_configuration_objectright_write"]=new Array();
	oMainFolderTree.cache.get_folder(configfolder,function (oFolder){
		var haveadminrights = (configparentfolder != -1 && getFolderRightCreateSubFolders(oFolder)) || (configparentfolder == -1 && getFolderRightFolderAdmin(oFolder));
		menu_display_views("configuration","menu_configuration_objectright_write",selecteditems == 1 && haveadminrights,value == 0,"menu_configuration_objectright_write_none"); 
		menu_display_views("configuration","menu_configuration_objectright_write",selecteditems == 1 && haveadminrights,value == 1,"menu_configuration_objectright_write_own"); 
		menu_display_views("configuration","menu_configuration_objectright_write",selecteditems == 1 && haveadminrights,value == 2,"menu_configuration_objectright_write_all"); 
		menu_display_views("configuration","menu_configuration_objectright_write",selecteditems == 1 && haveadminrights,value == 64,"menu_configuration_objectright_write_admin"); 
	});
}
function onmenuchangeHeightRightsContentReadObject(iteration,value,selecteditems)
{
	menuglobalzaehler=0;
	menuarrows["configuration"]["menu_configuration_objectright_read"]=new Array();
	oMainFolderTree.cache.get_folder(configfolder,function (oFolder){
		var haveadminrights = (configparentfolder != -1 && getFolderRightCreateSubFolders(oFolder)) || (configparentfolder == -1 && getFolderRightFolderAdmin(oFolder));
		menu_display_views("configuration","menu_configuration_objectright_read",selecteditems == 1 && haveadminrights,value == 0,"menu_configuration_objectright_read_none"); 
		menu_display_views("configuration","menu_configuration_objectright_read",selecteditems == 1 && haveadminrights,value == 1,"menu_configuration_objectright_read_own"); 
		menu_display_views("configuration","menu_configuration_objectright_read",selecteditems == 1 && haveadminrights,value == 2,"menu_configuration_objectright_read_all"); 
		menu_display_views("configuration","menu_configuration_objectright_read",selecteditems == 1 && haveadminrights,value == 64,"menu_configuration_objectright_read_admin"); 
	});
}
/***************************************************************** End **************************************/
/************************************************************************************************************/
/*** New Object Functions ***********************************************************************************/
/************************************************************************************************************/
/************************************************************************************************************/
/*** Mail Functions  ****************************************************************************************/
/************************************************************************************************************/
menuarrows["mail_new"]=new Array();
function mailnewonload() {
	createEnhancementMenu("menu_new_mail_view_enhancement","menu_new_mail_view","mail_new");
}
function mailnewattachmentchangeHeight()
{
	if($("menu_mail_new_attachment")) {
		onmenuchangeHeightMailNewAttachmentContent(menuiteration,menucountsubselected);
	}
	if ($("menu_new_mail_view")) {
		onmenuchangeHeightMailNewView(menuiteration,menucountsubselected);
	}
}
function onmenuchangeHeightMailNewAttachmentContent(iteration,selecteditems)
{
	menuglobalzaehler=0;
    menuarrows["mail_new"]["menu_mail_new_attachment"]=new Array();
    menu_display_contents("mail_new","menu_mail_new_attachment",true, "menu_mail_new_add_attachment");
    menu_display_contents("mail_new","menu_mail_new_attachment",selecteditems > 0, "menu_mail_new_remove_attachment");
} 
function onmenuchangeHeightMailNewView(iteration,selecteditems) {
	menuglobalzaehler=0;
    menuarrows["mail_new"]["menu_new_mail_view"]=new Array();
    menu_display_contents("mail_new","menu_new_mail_view",true, "menu_new_mail_view_fromfield");
	menu_display_contents("mail_new","menu_new_mail_view",true, "menu_new_mail_view_ccfield");
	menu_display_contents("mail_new","menu_new_mail_view",true, "menu_new_mail_view_bccfield");
	menu_display_contents("mail_new","menu_new_mail_view",true, "menu_new_mail_view_optionsfield");
}
/****************************************************************************************************/
/*** Calendar Functions  ******************************************************************************/
/****************************************************************************************************/
menuarrows["calendar_new"]=new Array();
function calendarnewonload() {
	createEnhancementMenu("menu_calendar_new_participant_enhancement","menu_calendar_new_participant","calendar_new");
}

function calendarnewattachmentchangeHeight()
{
	if(menuisActive("menu_calendar_new_attachment")) 
		onmenuchangeHeightCalendarNewAttachmentContent(menuiteration,menucountsubselected);
}
function calendarnewparticipantchangeHeight()
{
	if(menuisActive("menu_calendar_new_participant")) 
		onmenuchangeHeightCalendarNewParticipantContent(menuiteration,menucountsubselected);
}
function onmenuchangeHeightCalendarNewAttachmentContent(iteration,selecteditems)
{
	menuglobalzaehler=0;
	menuarrows["calendar_new"]["menu_calendar_new_attachment"]=new Array();
	menu_display_contents("calendar_new","menu_calendar_new_attachment",(configContainsKey("modules.infostore.module") && configGetKey("modules.infostore.module") == true), "menu_calendar_new_attachment_add_attachment");
	menu_display_contents("calendar_new","menu_calendar_new_attachment",(configContainsKey("modules.infostore.module") && configGetKey("modules.infostore.module") == true && selecteditems > 0), "menu_calendar_new_attachment_delete");
} 
function onmenuchangeHeightCalendarNewParticipantContent(iteration,selecteditems)
{
	menuglobalzaehler=0;
	menuarrows["calendar_new"]["menu_calendar_new_participant"]=new Array();
	menu_display_contents("calendar_new","menu_calendar_new_participant", (configContainsKey("participants.showDialog") && configGetKey("participants.showDialog") == true), "menu_calendar_new_participant_add_participant");
	menu_display_contents("calendar_new","menu_calendar_new_participant", (configContainsKey("participants.showDialog") && configGetKey("participants.showDialog") == true), "menu_calendar_new_participant_add_external");
	menu_display_contents("calendar_new","menu_calendar_new_participant", (configContainsKey("participants.showDialog") && configGetKey("participants.showDialog") == true && selecteditems > 0), "menu_calendar_new_participant_delete");	
}
/****************************************************************************************************/
/*** Task Functions  ******************************************************************************/
/****************************************************************************************************/
menuarrows["task_new"]=new Array();
function tasknewonload() {
	createEnhancementMenu("menu_task_new_participant_enhancement","menu_task_new_participant","task_new");
}
function tasknewparticipantchangeHeight()
{
	if(menuisActive("menu_task_new_participant")) {
		onmenuchangeHeightTaskNewParticipantContent(menuiteration,menucountsubselected);
	}
}
function onmenuchangeHeightTaskNewParticipantContent(iteration,selecteditems)
{
	menuglobalzaehler=0;
	menuarrows["task_new"]["menu_task_new_participant"]=new Array();
	menu_display_contents("task_new","menu_task_new_participant", (configContainsKey("participants.showDialog") && configGetKey("participants.showDialog") == true), "menu_task_new_participant_add_participant");
	menu_display_contents("task_new","menu_task_new_participant", (configContainsKey("participants.showDialog") && configGetKey("participants.showDialog") == true), "menu_task_new_participant_add_external");
	menu_display_contents("task_new","menu_task_new_participant", (configContainsKey("participants.showDialog") && configGetKey("participants.showDialog") == true && selecteditems > 0), "menu_task_new_participant_delete");

	
} 
/****************************************************************************************************/
/*** Contact Functions  ******************************************************************************/
/****************************************************************************************************/
menuarrows["contact_new"]=new Array();
function menuregisterContactNewContents()
{
	register("OX_SELECTED_SUBSELECTION_CHANGED",contactnewchangeHeight);
	addContextNode(contextmenu_createImgDiv("Add",getFullImgSrc("img/menu/add_picture.gif"),getFullImgSrc("img/menu/add_picture_d.gif")),["contact_new_picture"],function() { triggerEvent('OX_Picture_Add') },
	null,null,contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);
	addContextNode(contextmenu_createImgDiv("Remove",getFullImgSrc("img/menu/remove_picture.gif"),getFullImgSrc("img/menu/remove_picture_d.gif")),["contact_new_picture"],function() { triggerEvent('OX_Picture_Remove') },
	null,null,contextmenu_setDisabled,null,null,null,contextmenu_setEnabled);

	
}	
function contactnewchangeHeight()
{
	if(menuisActive("menu_new_contact_distributionlist")) {
		onmenuchangeHeightContactDistributionListContent(menuiteration,menucountsubselected);
	}
	if(menuisActive("menu_contact_new_picture")) {
		onmenuchangeHeightContactPictureContent(menuiteration,menucountsubselected);
	}
}
function onmenuchangeHeightContactDistributionListContent(iteration,selecteditems){
	menuglobalzaehler=0;
	menuarrows["contact_new"]["menu_contact_new_distributionlist"]=new Array();
	menu_display_contents("contact_new","menu_contact_new_distributionlist",true, "menu_new_contact_distributionlist_ic");
	menu_display_contents("contact_new","menu_contact_new_distributionlist",true, "menu_new_contact_distributionlist_em");
	menu_display_contents("contact_new","menu_contact_new_distributionlist",selecteditems>0, "menu_new_contact_distributionlist_delete");
} 
function onmenuchangeHeightContactPictureContent(iteration,selecteditems) {
	menuglobalzaehler=0;
	menuarrows["contact_new"]["menu_contact_new_picture"]=new Array();
	menu_display_contents("contact_new","menu_contact_new_picture",true, "menu_contact_new_picture_add");
	menu_display_contents("contact_new","menu_contact_new_picture",true, "menu_contact_new_picture_remove");
} 
function call_triggerEventFunction() 
{
	oMainFolderTree.cache.get_folder((configparentfolder!=-1)?configparentfolder:configfolder,function (oFolder){
		triggerEvent('OX_Configuration_Add_User',oFolder);
	});
}
fileloaded();
function getStackTrace() {
    var result = '';

    if(true) { // Mozilla, not ECMA
        // fake an exception so we can get Mozilla's error stack
        var testExcp;
        try
        {
            foo.bar;
        }
        catch(testExcp)
        {
            var stack = parseErrorStack(testExcp);
            result += '> ' + stack[3] + '\n';
        }
    }
    return result;
}
function parseErrorStack(excp)
{
    var stack = [];
    var name;

    if (!excp || !excp.stack)
    {
        return stack;
    }

    var stacklist = excp.stack.split('\n');

    for (var i = 0; i < stacklist.length - 1; i++)
    {
        var framedata = stacklist[i];

        name = framedata.match(/^(\w*)/)[1];
        if (!name) {
            name = 'anonymous';
        }

        stack[stack.length] = name;
    }
    // remove top level anonymous functions to match IE

    while (stack.length && stack[stack.length - 1] == 'anonymous')
    {
        stack.length = stack.length - 1;
    }
    return stack;
}

function setButtonMenuElement1(node) {
	if(node) {
		var divsinelement=node.getElementsByTagName("div");
		for(var i=0;i<divsinelement.length;i++) {
			divsinelement[i].style.position="relative";
			divsinelement[i].style.top="2px";
			divsinelement[i].style.left="2px";
		}
	}
}
function resetButtonMenuElement1(node) {
	if(node) {
		var divsinelement=node.getElementsByTagName("div");
		for(var i=0;i<divsinelement.length;i++) {
			divsinelement[i].style.top="0px";
			divsinelement[i].style.left="0px";
		}
	}
}
function setButtonMenuElement2(node) {
	if(node) {
		var divsinelement=node.getElementsByTagName("img");
		for(var i=0;i<divsinelement.length;i++) {
			divsinelement[i].style.position="relative";
			divsinelement[i].style.top="2px";
			divsinelement[i].style.left="2px";
		}
	}
}
function resetButtonMenuElement2(node) {
	if(node) {
		var divsinelement=node.getElementsByTagName("img");
		for(var i=0;i<divsinelement.length;i++) {
			divsinelement[i].style.top="0px";
			divsinelement[i].style.left="0px";
		}
	}
}
function setButtonMenuElement3(node) {
	if(node) {
		node.style.top="2px";
		node.style.left="9px";
	}
}
function resetButtonMenuElement3(node) {
	if(node) {
		node.style.top="0px";
		node.style.left="7px";
	}
}
function makeButtonElement1(node) {
	if(node) {
		var allelements=node.getElementsByTagName("tr");
		for(var t=0;t<allelements.length;t++) {
			if(allelements[t].id && allelements[t].id.indexOf("_na")==-1 && allelements[t].id.indexOf("_disable")==-1 && allelements[t].id.indexOf("_active")==-1 && allelements[t].id.indexOf("menu_mail_flags_") != 0) {	
				setDOMMenuElement1(allelements[t]);
			}
		}
	}
}
function makeButtonElement2(node) {
	if(node) {
		var allelements=node.getElementsByTagName("tr");
		for(var t=0;t<allelements.length;t++) {
			if(allelements[t].id && allelements[t].id.indexOf("_na")==-1 && allelements[t].id.indexOf("_disable")==-1 && allelements[t].id.indexOf("_active")==-1 && allelements[t].id.indexOf("_arrow") == -1) {	
				setDOMMenuElement2(allelements[t]);
			}
		}
	}
}
function setDOMMenuElement1(mynode) {
	if(mynode) {
		addDOMEvent(mynode,"mousedown",function(e) {
			if (e.button != undefined && e.button == RightButton) return;
			setButtonMenuElement1(mynode);
		});
		addDOMEvent(mynode,"mouseup",function(e) {
			resetButtonMenuElement1(mynode);
		});
		addDOMEvent(mynode,"mouseout",function(e) {
			resetButtonMenuElement1(mynode);
		});
	}
}
function setDOMMenuElement2(mynode) {
	if(mynode) {
		addDOMEvent(mynode,"mousedown",function(e) {
			if (e.button != undefined && e.button != RightButton) return;
			setButtonMenuElement2(mynode);
		});
		addDOMEvent(mynode,"mouseup",function(e) {
			resetButtonMenuElement2(mynode);
		});
		addDOMEvent(mynode,"mouseout",function(e) {
			resetButtonMenuElement2(mynode);
		});
	}
}
function setDOMMenuElement3(mynode) {
	if(mynode) {
		addDOMEvent(mynode,"mouseup",function(e) {
			resetButtonMenuElement3(mynode);
		});
		addDOMEvent(mynode,"mouseout",function(e) {
			resetButtonMenuElement3(mynode);
		});
		addDOMEvent(mynode,"mousedown",function(e) {
			if (e.button != undefined && e.button != RightButton) return;
			setButtonMenuElement3(mynode);
		});
	}
}
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2007 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Andreas Mayer <andreas.mayer@open-xchange.com>
 * 
 */

function fnQueue() {
	this.queue = new Object();
}
fnQueue.prototype = {
	push:function(idParam,functionParam) {
		if(this.queue[idParam] != undefined) {
			this.queue[idParam].push(functionParam);
			return true;
		} else {
			this.queue[idParam] = new Array();
			this.queue[idParam].push(functionParam);
			return false;
		}
	},
	exe: function(idParam,objParam) {
		if(this.queue[idParam]) {
			for(i=0; i<this.queue[idParam].length; i++) {
				var nextFN = this.queue[idParam][i];
				nextFN(objParam);
			}
			delete this.queue[idParam];
		}		
	}
}

function foldertreeState(oState) {
	this.state = oState;
	this.links = new Object();
}
foldertreeState.prototype = {
	setState: function(oState) {
		var Self = this;
		this.links = new Object();
		function cl_state(stateParam) {
			var newObj = new Object();
			for (var nIndx in stateParam) {
				var newIndx = nIndx.split('$')[1];
				if((nIndx.split('$')[2] == "link"))
					Self.links[newIndx] = true;
				newObj[newIndx] = cl_state (stateParam[nIndx]);
			}
			return newObj;
		}
		this.state = cl_state (oState);
	},
	getState: function(sPrefix) {
		var Self = this;
		function mk_state(stateParam) {
			var newObj = new Object();
			for(var nIndx in stateParam) {
				if(Self.links[nIndx])
					var newIndx = sPrefix + '$' + nIndx + '$link';
				else
					var newIndx = sPrefix + '$' + nIndx;
				newObj[newIndx] = mk_state (stateParam[nIndx]);
			}
			return newObj;
		}
		return mk_state (this.state);
	}
}
var optFolderState = new foldertreeState({1:{},2:{}});

//Folder Object Definition
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
 * class cFolder combines all attributes of a Folder,  
 * attributes must be set in constructer during folder creation
 */
function cFolder(sIdParam, sModuleParam, sNameParam, sContainedInfoParam,
        bContainsSubfoldersParam, nTimeStampParam, aPermissionsParam, 
        sTypeParam, oOwnRightsParam, bDefaultFolderParam, nUnreadParam,
        nOwnerIdParam, nSubscribedParam, nCapabilities) {
	this.sId = sIdParam;
	this.sModule = sModuleParam;
	this.sName = sNameParam;
	this.sContainedInfo = sContainedInfoParam;
	this.bContainsSubfolders = bContainsSubfoldersParam;
	this.nTimeStamp = nTimeStampParam;
	this.aPermissions=aPermissionsParam; 
	this.oOwnRights=oOwnRightsParam; 
	this.enabled = true;
	this.sType = sTypeParam;
	this.bDefaultFolder = bDefaultFolderParam;
	this.nUnread = nUnreadParam;
	this.owner_id = nOwnerIdParam;
	this.subscribed = nSubscribedParam;
	this.capabilities = nCapabilities;
	this.bLink = false;
}

/**
 * @return {Array} - an array with [0] the closed and [1] the opened icon
 */
function getIconImagePath(sParentId, sId, sModule, bLink, bREnabledIMAP, bDefaultFolder) {
	var sImageSRC = new Array(2);
	if (bLink && sModule == 'infostore') {
		var tmp = (bREnabledIMAP) ? "img/folder/myinfostore.gif" : "img/folder/myinfostore_dis.gif";
		sImageSRC[0] = getFullImgSrc(tmp);
        sImageSRC[1] = sImageSRC[0];
        			
	} else if (sModule == 'system' && sId == 1 || sParentId==10 && bDefaultFolder && sModule == 'infostore') {
		tmp = (bREnabledIMAP) ? "img/folder/user.gif" : "img/folder/user_dis.gif";
		sImageSRC[0] = getFullImgSrc(tmp);
        sImageSRC[1] = sImageSRC[0];
        				
	} else if (sModule == 'system' && sId == 'default') {
		var tmp = (bREnabledIMAP) ? "img/folder/mail.gif" : "img/folder/mail_dis.gif";
        sImageSRC[0] = getFullImgSrc(tmp);
        sImageSRC[1] = sImageSRC[0];
        
	} else if (sModule == 'system' && sId == 2) {
		var tmp = (bREnabledIMAP) ? "img/folder/public.gif" : "img/folder/public_dis.gif";
        sImageSRC[0] = getFullImgSrc(tmp);
        sImageSRC[1] = sImageSRC[0];
        		
	} else if (sModule == 'system' && sId == 3) {
		var tmp = (bREnabledIMAP) ? "img/folder/shared.gif" : "img/folder/shared_dis.gif";
        sImageSRC[0] = getFullImgSrc(tmp);
        sImageSRC[1] = sImageSRC[0];
				
	} else if (sModule != 'system') {		
		if (sModule == 'mail') {
			var oMailDefaultFolder = configGetKey("mail.folder");
			for (var eachMailFolder in oMailDefaultFolder) {
				if (sId == oMailDefaultFolder[eachMailFolder]) {
					switch(eachMailFolder) {
						case "inbox":
						      var tmp = (bREnabledIMAP) ? "img/folder/inbox.gif" : "img/folder/inbox_dis.gif";
						      sImageSRC[0] = getFullImgSrc(tmp);
						      sImageSRC[1] = sImageSRC[0];
					 		  break;
						case "sent":	
						      var tmp = (bREnabledIMAP) ? "img/folder/outbox.gif" : "img/folder/outbox_dis.gif";
                              sImageSRC[0] = getFullImgSrc(tmp);
                              sImageSRC[1] = sImageSRC[0];
                              break;
						case "trash":	
						      var tmp = (bREnabledIMAP) ? "img/folder/garbage.gif" : "img/folder/garbage_dis.gif";
                              sImageSRC[0] = getFullImgSrc(tmp);
                              sImageSRC[1] = sImageSRC[0];
                              break;
						case "drafts":
						      var tmp = (bREnabledIMAP) ? "img/folder/draft.gif" : "img/folder/draft_dis.gif";
                              sImageSRC[0] = getFullImgSrc(tmp);
                              sImageSRC[1] = sImageSRC[0];
                              break;									 		
						case "spam":
						      var tmp = (bREnabledIMAP) ? "img/folder/spam.gif" : "img/folder/spam_dis.gif";
                              sImageSRC[0] = getFullImgSrc(tmp);
                              sImageSRC[1] = sImageSRC[0];
                              break;
						case "ham":	
						      var tmp = (bREnabledIMAP) ? "img/folder/ham.gif" : "img/folder/ham_dis.gif";
                              sImageSRC[0] = getFullImgSrc(tmp);
                              sImageSRC[1] = sImageSRC[0];
                              break;								 																 									 											 											 		
						default:	
						      var tmp = (bREnabledIMAP) ? "img/folder/folder_closed.gif" : "img/folder/folder_closed_dis.gif";
		                      sImageSRC[0] = getFullImgSrc(tmp);
		                      tmp = (bREnabledIMAP) ? "img/folder/folder_opened.gif" : "img/folder/folder_opened_dis.gif";
		                      sImageSRC[1] = getFullImgSrc(tmp);
					}
					break;
				} else {
					var tmp = (bREnabledIMAP) ? "img/folder/folder_closed.gif" : "img/folder/folder_closed_dis.gif";
			        sImageSRC[0] = getFullImgSrc(tmp);
			        tmp = (bREnabledIMAP) ? "img/folder/folder_opened.gif" : "img/folder/folder_opened_dis.gif";
			        sImageSRC[1] = getFullImgSrc(tmp);
				}
			}
		} else {
			var tmp = (bREnabledIMAP) ? "img/folder/"+sModule+".gif" : "img/folder/"+sModule+"_dis.gif";
            sImageSRC[0] = getFullImgSrc(tmp);
            sImageSRC[1] = sImageSRC[0];
        }
	} else {
		var tmp = (bREnabledIMAP) ? "img/folder/folder_closed.gif" : "img/folder/folder_closed_dis.gif";
        sImageSRC[0] = getFullImgSrc(tmp);
        tmp = (bREnabledIMAP) ? "img/folder/folder_opened.gif" : "img/folder/folder_opened_dis.gif";
        sImageSRC[1] = getFullImgSrc(tmp);
	}	
	
	return sImageSRC;			
}

/**
 * Fetch folder information using multiple request
 * @param folderIDs - An array of folder ids
 * @param columns - String of columns which should be fetched (optional). If null a default set will be used.
 * @param callback - Callback wich will be executed after the server delivers the reponse
 */
function doAjaxFetchFolder(folderIDs, columns, callback) {
    var sCols =  columns || oMainFolderTree.cache.columns;        
    var sAjaxRequest = AjaxRoot + "/multiple?session=" + session;
    var finalArr = new Array();
    for (var i=0; i<folderIDs.length; i++) {
    	finalArr.push({ module: "folder", action: "get", id: folderIDs[i], folder: folderIDs[i], columns: sCols });
    }    
    newFolderJson.put(sAjaxRequest, finalArr, null, callback);
}

function doAjaxUpdateRequest(sId, oFolder, nTimeStamp, callback, err_handlr) {
	var sAjaxRequest = AjaxRoot + "/folders?action=update"
        + "&id=" + getUrlEncodedString(sId)
        + "&session=" + session
	if (nTimeStamp != undefined) {
		sAjaxRequest += "&timestamp=" + nTimeStamp;
	}
	newFolderJson.put(sAjaxRequest, oFolder, null,callback,err_handlr);			
}

function doAjaxNewFolderRequest(parentNodeId ,oFolder, callback, callbackERR) {
    var sAjaxRequest = AjaxRoot + "/folders?action=new"
       + "&folder_id=" + getUrlEncodedString(parentNodeId) 
       + "&session=" + session;
    newFolderJson.put(sAjaxRequest, oFolder, null, callback, callbackERR);        
}

function doAjaxRemoveFolderRequest(aFolders, nTimeStamp, callback) {
    var sAjaxRequest = AjaxRoot + "/folders?action=delete&session=" + session;
    if (nTimeStamp != undefined) {
        sAjaxRequest += "&timestamp=" + nTimeStamp;
    }
    newFolderJson.put(sAjaxRequest, aFolders, null, callback);  
}

function computePerm(nBits, nOffset) {
	if (nOffset == 28) {
		return (nBits & Math.pow(2,nOffset))?1:0;		
	} else if (nOffset == 29) {
		return (nBits & Math.pow(2,nOffset))?1:0;		
	} else {
		for (nBit = 0;nBit <= 6;nBit++) {	
			if( nBit == 0 && nBits & Math.pow(2,(nBit + nOffset))) {
				return 1;
			} else if (nBit == 1 && nBits & Math.pow(2,(nBit + nOffset))) {
				return 2;
			} else if (nBit == 2 && nBits & Math.pow(2,(nBit + nOffset))) {
				return 4;
			} else if (nBit == 6 && nBits & Math.pow(2,(nBit + nOffset))) {
				return 64;
			}
		}
	}
	return 0;
}

function FolderPath(module) {   
    this.module = module;
    this.folderId = null;
    this.nodeId = null;
    this.className = "linkstyle-headline";
    this.classNameD = "font-style-low";
    this.events = new Array();
    this.cb = function(module, folderId) {
            return function() {
                triggerEvent("OX_Direct_Linking", module, 
                    { module: module, folder: folderId });
            }                
        }
    this._cancelDraw=false;
    this.additionalNode;
}

FolderPath.prototype = {
    setFolderId: function(folderId) {
        this.folderId = folderId;
    },  
    drawDOMNode: function(nodeId, folderId) {
        if (nodeId) this.nodeId = nodeId;
        if (folderId) this.folderId = folderId;
        if (!$(this.nodeId) || !folderId || this._cancelDraw) return;
        this._cancelDraw = true;
        var Self = this;
        this.clear();   
        function fn_folder(sPath) {
            for (var i = sPath.length-1; i >= 0; i--) {                        
                oMainFolderTree.cache.get_folder(sPath[i].id, (function(sPath) {
                	return function(e) {
	                	var ids = configGetKey("folder");
				        var replacedname = "";
				        switch (sPath.id) {
				            case ids.tasks:
				                replacedname = "Tasks"; /*i18n*/
				                break;
				            case ids.calendar:
				                replacedname = "Calendar"; /*i18n*/
				                break;
				            case ids.contacts:
				                replacedname = "Contacts"; /*i18n*/
				                break;
				        }
	                    var read = computePerm(e.oxfolder.data.own_rights,7) > 0;
	                    var myNode = newnode("span", 0, { className: read  ? Self.className : Self.classNameD },
	                        [ document.createTextNode(replacedname ? _(replacedname) : sPath.title) ]);
	                    if (read) {
	                        var fcb = (Self.cb)(e.oxfolder.data.module, sPath.id);
	                        addDOMEvent(myNode, "click", fcb);
	                        Self.events.push(["click", fcb]);
	                    } 
	                    $(Self.nodeId).appendChild(myNode);
	                    if (i>0) {
	                    	$(Self.nodeId).appendChild(document.createTextNode(" / "));
	                    } else if (Self.additionalNode != undefined) {
	                    	$(Self.nodeId).appendChild(Self.additionalNode);
	                    }
                	}
                })(sPath[i]));
            }
            Self._cancelDraw=false;
        }               
        oMainFolderTree.get_folder_path_array(this.folderId, fn_folder);        
    },
    drawSearch: function(module, searchTerm) {
    	if (!this.nodeId || this.module != module) return;
    	this.clear();
    	var myNode = newnode("span", 0, 0, [ document.createTextNode(format(_("Results for \"%s\""), searchTerm)) ]);
    	$(this.nodeId).appendChild(myNode);
    },
    clear: function() {
        if (!$(this.nodeId)) return;
        for (var i=0; i< this.events.length; i++) {
            removeDOMEvent($(this.nodeId), this.events[i][0], this.events[i][1]);
        }
        this.events = new Array();
        removeChildNodes($(this.nodeId));
    }
}

fileloaded();
/* each folder tree should use this cache */
function ox_folder_cache(){	
	/* register all events here */	
	this.events = new Events();
	/* contains all folders which are stored hierarchically in storagecaches */
	this.folders = {"root":{children:null,parent:null,oxfolder:null}};
	var Self = this;
	this.columns = "1,301,300,307,304,306,302,305,308,311,2,314,313,315";
	this.fast_access = new Object();
	this.timestamp = 0;
}

ox_folder_cache.prototype = {

	/* load the root folders or all folder defined in oState */
	load_folders : function (oState, join_param, bReplace) {
		var Self = this;
		var join_fn = join_param.add( function (arg){ join_param.arg = arg; } );
		if (Self.folders["root"].children != null && !bReplace) {
			for (var nfldrN in Self.folders["root"].children[nfldrN]) {											
				if(oState && oState[Self.folders["root"].children[nfldrN].oxfolder.data.id]) {
					Self.load_subfolders(
					       Self.folders["root"].children[nfldrN].oxfolder.data.id,
					       Self.folders["root"].children[nfldrN],
					       oState[nfldrN], join_param, bReplace 
				    );
				}														
			}		
			join_fn(Self.folders);			
		} else {
			var locJson = new JSON();		
			locJson.get("/ajax/folders?action=root&columns="+this.columns+"&session="+session,null,
									function(arg) {		
											Self.folders["root"].children = {};
											Self.timestamp = arg.timestamp;
											for (var nfldr=0;nfldr<arg.data.length;nfldr++)
											{											
												Self.folders["root"].children[arg.data[nfldr][0]] = 
																{oxfolder:new ox_folder(arg.timestamp,arg.data[nfldr]),parent:Self.folders["root"],
																																		children:null};
												Self.fast_access[Self.folders["root"].children[arg.data[nfldr][0]].oxfolder.data.id] = Self.folders["root"].children[arg.data[nfldr][0]];
												if(oState && oState[arg.data[nfldr][0]])
												{
													Self.load_subfolders(arg.data[nfldr][0],Self.folders["root"].children[arg.data[nfldr][0]],
																		oState[arg.data[nfldr][0]],join_param,bReplace);
												}														
											}		
											join_fn(Self.folders);							
										});		
		}
		
	},
	
	do_load_subfolders : function (nId,refer,oState,callback,bReplace){
		this.load_subfolders(nId, refer, oState, 
		      new Join(
		              function() {
		                  callback(this.arg);
		              }
		      ), bReplace);
	},
	
	/*loads subfolders from server*/
	load_subfolders : function (nId, refer, oState, join_param, bReplace) {
		var Self = this;
		var join_fn = join_param.add(
		      function (arg) {
		          join_param.arg = arg;
		      }
		);
	
		if (refer.children != null && !bReplace) {
			for (var nfldrN in refer.children) {
				if(oState && oState[nfldrN]) {
					Self.load_subfolders(nfldrN, refer.children[nfldrN], oState[nfldrN], join_param, bReplace);
				}													
			}					
			join_fn(refer);		
		} else {
			var locJson = new JSON();
			function fn_cb(arg) {      
	            // if children loaded before server responsed
	            if(refer.children != null && !bReplace) {
	                join_fn(refer); 
	                return;
	            }
	            refer.children = {};
	            /*
	            function fn_sort(a,b) { 
	                var Satz = /^(default)(\S)(INBOX)$/;
	                if (b[0].match(Satz)) {
	                    return 1;
	                } else if (a[0].match(Satz)) {
	                    return -1;
	                } else if(a[2].toUpperCase() < b[2].toUpperCase()) {
	                    return -1;
	                } else {
	                    return 1;
	                }                   
	            }
	            if (refer.oxfolder.data.id == "default" || refer.oxfolder.data.module == "mail") {
	                arg.data.sort(fn_sort);
	            }
	            */
	            for (var nfldr=0; nfldr<arg.data.length; nfldr++) {
	                refer.children[arg.data[nfldr][0]] = 
                        { "oxfolder": new ox_folder(arg.timestamp,arg.data[nfldr]),
                          "parent"  : refer,
                          "children": null
                        };
	                var oldrefer = null;
	                if (refer.oxfolder.data.id == configGetKey("folder.infostore") && Self.fast_access[arg.data[nfldr][0]] != undefined) {
	                    refer.children[arg.data[nfldr][0]] = 
	                        { "oxfolder": new ox_folder(arg.timestamp,arg.data[nfldr]),
                              "parent"  : refer, 
                              "children": Self.fast_access[arg.data[nfldr][0]].children
                            };
	                }
	                    
	                Self.fast_access[refer.children[arg.data[nfldr][0]].oxfolder.data.id] = refer.children[arg.data[nfldr][0]]; 
	                if (oState && oState[arg.data[nfldr][0]]) {
	                    Self.load_subfolders(arg.data[nfldr][0], refer.children[arg.data[nfldr][0]],
                                             oState[arg.data[nfldr][0]], join_param, bReplace);
	                }                                                   
	            }
	            join_fn(refer);                         
	        }
	        
			locJson.get("/ajax/folders?action=list&all=1"
			     + "&columns=" + this.columns 
			     + "&parent=" + getUrlEncodedString(nId) + "&session=" + session, null, fn_cb);
		}
	},
	
	updateFolders : function(folderIDs, fn_cb) {
        var Self = this;
        var locJson = new JSON();
        var _reqParams = new Array();
        for (var i in folderIDs) {
            _reqParams.push( { module: "folder", action: "get", id: folderIDs[i], columns: this.columns } );
        }
        locJson.put(AjaxRoot + "/multiple?session=" + session, _reqParams, null, 
            function(resp) {
                for (var ia=0; ia<resp.length; ia++) {
                    var _tmp = resp[ia];
                    if (_tmp.error && _tmp.error_id) continue;
                    var _oFolder = Self.find_folder(_tmp.data.id);
                    _oFolder.oxfolder = { timestamp: 0, ENABLED: true, data: _tmp.data };
                    Self.fast_access[_tmp.data.id].oxfolder = _oFolder.oxfolder;
                }   
            }
        );
    },
	
	/* loads the path and all folder siblings */
	load_path : function (nId, join_param, bNOReplace){
		var Self = this;
		var sAjaxRequest = AjaxRoot + '/folders?action=path'
		                 + '&session=' + session 
		                 + '&id=' + getUrlEncodedString(nId) 
		                 + '&columns=1,300';
		var pathJson = new JSON();
		
		function cb_fn_path(arg) {
			var oState = {};
			var statetmp  = oState;
			var bFoundFolder = false;
			var oRefer;
			/*
			 * fixed bug, in this case is the requested folder a first level folder and it must be present
			 * return the path
			 */
			if (arg.data.length <= 1) {
				var join_fn = join_param.add(
			        function (arg, arg2) {
			            if (arg) join_param.arg = arg;
			            if( arg2) join_param.arg2 = arg2;
			        }
				);		
				join_fn(Self.fast_access[nId], arg.data);
				return;
			}
			join_param.arg2 = arg.data;
			for (var indx=(arg.data.length-1); indx>0; indx--) {
				//found the last loaded node and load the rest
				if (!bFoundFolder && Self.fast_access[arg.data[indx][0]] != undefined) {
					oRefer = Self.fast_access[arg.data[indx][0]];
				} else {
					if (!bFoundFolder) {
						nParentId = arg.data[indx][0];				
					}
					bFoundFolder = true;
					statetmp[arg.data[indx][0]] = {};
					statetmp = statetmp[arg.data[indx][0]];
				}
			}
			var join_fn = join_param.add(
			    function (arg, arg2) {
			    	if (arg) join_param.arg = arg;
			        if (arg2) join_param.arg2 = arg2;
			    }
			);				
			//if node not found, load complete path
			if (oRefer == undefined) {
				join_param.count--;
				Self.load_folders(oState, join_param, true);
				
			} else if(oRefer.children == null || oRefer.children != null && oRefer.children[nId] == undefined) {
				join_param.count--;			
				Self.load_subfolders(oRefer.oxfolder.data.id, oRefer, oState, join_param, (true && !bNOReplace));
				
			} else {
				join_fn(oRefer.children[nId], arg.data);
			}
		}
		pathJson.get(sAjaxRequest,null,cb_fn_path);
	},
	
	/* updates all folders; fetches new subfolder if availible, removes deleted folder from cache */
	update : function (nIdOpt, bWith_mail, fn_callback_opt) {
		var Self = this;
		var locJson = new JSON();
		var sAjaxRequest = AjaxRoot + "/folders?action=updates"
	            + "&columns=1,300&mail=" + (bWith_mail ? 1 : 0) 
	            + "&timestamp=" + this.timestamp + "&session=" + session;
	            		
		function cb_fn_updates(arg){
			var aReferences = new Array();
			if (arg.data) {
				Self.timestamp = arg.timestamp;
						
				//separate folder, which will be updated in cache
				var aRootNodes = new Object();
				function fn_traverse(refer) {
					for (var indx in refer.children) {
						var nFound = 0;
						for (var indxResp=0; indxResp<arg.data.length; indxResp++) {
							//first condition is for deleted folder identification
							if (arg.data[indxResp][1] != undefined && arg.data[indxResp][0] == indx) {						
								nFound = 1;
								break;
							} else if (arg.data[indxResp][1] == undefined && arg.data[indxResp][0] == indx) {					
								nFound = 2;
								break;
							}
						}
						//don't traverse the subtree, if the parent of the sub tree must be updated;
						//all subtree elements will be updated automatically
						//->added != 10 condition to update links correcty
						if (nFound != 0 && Self.fast_access[indx].parent.oxfolder && Self.fast_access[indx].parent.oxfolder.data.id != 10) {					
							if (aRootNodes[Self.fast_access[indx].parent.oxfolder.data.id] == undefined) {
								aRootNodes[Self.fast_access[indx].parent.oxfolder.data.id] = new Array();
								aRootNodes[Self.fast_access[indx].parent.oxfolder.data.id].push({id:indx,type:nFound});
							} else {
								aRootNodes[Self.fast_access[indx].parent.oxfolder.data.id].push({id:indx,type:nFound});
							}
							
						} else if (nFound != 0) {					
							if (aRootNodes[Self.fast_access[indx].oxfolder.data.id] == undefined) {
								aRootNodes[Self.fast_access[indx].oxfolder.data.id] = new Array();
								aRootNodes[Self.fast_access[indx].oxfolder.data.id].push({id:indx,type:nFound});
							} else {
								aRootNodes[Self.fast_access[indx].oxfolder.data.id].push({id:indx,type:nFound});
							}
						} else {
							fn_traverse(refer.children[indx]);
						}
					}
				}
				fn_traverse(Self.folders["root"]);
				if (!fn_callback_opt) {
					fn_callback_opt = function () {};
				}
				var outer_join = new Join(fn_callback_opt);
				for (var nNum in aRootNodes) {
					var oRefer = Self.fast_access[nNum];
					var oReferState = Self.compute_state(oRefer);
					var fn_out_join = outer_join.add();
		//			delete oRefer.parent.children;
		//			oRefer.parent.children = {};			
					function close_it(obj,aFolders,fn_out_join_close) {
						return function () {
							Self.events.trigger("Changed",obj);
							if (fn_out_join_close) {
								fn_out_join_close();
							}
					    }
					}
					Self.load_subfolders(oRefer.oxfolder.data.id, oRefer, oReferState, 
					       new Join(close_it(oRefer, aRootNodes[nNum], fn_out_join)), true);
				}
			}
		}
		locJson.get(sAjaxRequest, null, cb_fn_updates);
	},
	
	compute_state : function (refer){
		function fn_traverse(refer) {
			var retval = {};
			for (var i in refer.children) {
				var bSubChilds = false
				for (var j in refer.children[i].children) {
					bSubChilds=true;				
				}
				if (bSubChilds) {
				    retval[refer.children[i].oxfolder.data.id] = fn_traverse(refer.children[i]);
				}			
			}
			return retval;
		}
		var oState = fn_traverse(refer);
		return oState;
	},
	
	find_folder : function (nId){
		var Self = this;
		var oFolder = null;		
		function fn_traverse(refer) {
			for (var indx in refer.children) {
				if (indx != nId) {
					fn_traverse(refer.children[indx]);				
				} else {
					oFolder = refer.children[indx];
					return;
				}
			}
		}
		fn_traverse(this.folders["root"]);
		return oFolder;
	},
	
	/*loads a folder if not loaded*/
	get_folder : function (nId,callback){
		var Self = this;
		var oFolder = null;
		// trying fast acess cache
		oFolder = this.fast_access[nId];
		if (oFolder == undefined) {
			// not found, so trying to find it in regular cache
			oFolder = this.find_folder(nId);
		}
		//request the folder if that could not be found
		if (oFolder == null) {		
			this.load_path(nId, 
			     new Join(
				     function () {
				     	callback(this.arg);
				     }
				 ), true);
		} else if (callback) {	
			callback(oFolder);
		}
	}
}

function ox_folder(timestamp, dataArray) {
	this.data = new Object();	
	if (dataArray && dataArray.length > 11) {
		this.data.id = dataArray[0];
		this.data.module = dataArray[1];
		this.data.title = dataArray[2];
		this.data.summary = dataArray[3];
		this.data.subfolders = dataArray[4];
		this.data.permissions=dataArray[5]; 
		this.data.type = dataArray[6];
		this.data.own_rights=dataArray[7]; 
		this.data.standard_folder = dataArray[8];	
		this.data.unread= dataArray[9];
		this.data.created_by = dataArray[10];
		this.data.subscribed = dataArray[11];		
		this.data.capabilities = dataArray[12];
		this.data.subscr_subflds = dataArray[13];
	}
	
	this.timestamp = timestamp;
	//flags
	this.ENABLED = true;
}
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2007 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Andreas Mayer <andreas.mayer@open-xchange.com>
 * @author Stefan Preuss <stefan.preuss@open-xchange.com>
 * 
 */
 
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Folder Tree Object Definition
//////////////////////////////////////////////////////////////////////////////////////////////////////////////
/*
 * 
 * */
var oMainFolderTree;
function cFolderTree(DOMNode, actionTree, enableDND,cache_param,state)
{
	this.cache = cache_param;
	this.tree = null;
	this.state = state;
	this.enabled_modules = {system:true,calendar:true,contacts:true,tasks:true,mail:true,infostore:true};
	this.dom_node = DOMNode;
	this.selected_node = null;
	this.events = new Events();
	this.dnd_enabled = enableDND;
	this.inplace_enabled = false;
	this.select_for_action = false;	
	this.context_menu_enabled = false;	
	this.show_new_messages = false;
	this.show_unsubscribed = false;
	this.check_read_right = true;
	this.toggle_cursor = false;
	this.register_update = false;
	this.saveConfObj = null;
	this.tree = new Tree(this.dom_node,this.state,DOMNode.id+"_tree");
	var Self = this;
	this.enable_check_fn = function (){ return true; };
	this.action_node = function (arg) {
		var oFolder = Self.cache.find_folder(Self.getFolderID(arg.id));
        if (Self.enabled_modules[oFolder.oxfolder.data.module] && Self.enable_check_fn(oFolder)) {
        	actionTree(oFolder,arg);
			//check read right befor			
			if (Self.select_for_action && 
			    (!Self.check_read_right || Self.check_read_right && computePerm(oFolder.oxfolder.data.own_rights,7)>0)) {
				Self.set_selected(arg);
			}
		} else {
			if(!arg.open) arg.setOpen(true);
		}
	};
											
	this.tree.events.register ("OX_Tree_Expand",function (node,fn_opt){Self.expand_node(node,fn_opt);});
	this.tree.events.register ("OX_Tree_Node_Click",Self.action_node);
	this.tree.events.register('OX_Tree_State_Changed',function (){Self.saveConfiguration();});
	// Disable foldertree text in foldertree
	DOMNode.onselectstart = function () { var tmp=menuselectstart ; menuselectstart = false;  return tmp; }  
	DOMNode.onmousedown = function () { var tmp=menumousedown ; menumousedown = false; return tmp; }
	
	this.nodes = {};
	this.dashed = null;
}
cFolderTree.prototype = {
get_folder_path : function (nId,fn_callback){		
	var oF;
	//if folder already in cache
	if(oF = this.cache.find_folder(nId))
	{
		var sPath = "";
		while(oF.parent)
		{
			sPath = oF.oxfolder.data.title + "/" +  sPath;
			oF = oF.parent;
		}	
		fn_callback(sPath);		
	}
	else
		this.cache.load_path(nId,new Join(function (){
										var sPath = "";
										for(var nNumParent=0;nNumParent<this.arg2.length;nNumParent++)
										{
											sPath = this.arg2[nNumParent][1] + "/" +  sPath;
										}
										fn_callback(sPath);}),true);	
},
get_folder_path_array : function (nId,fn_callback){       
    var oF;
    //if folder already in cache
    if(oF = this.cache.find_folder(nId)) {
        var tFolder = new Array();
        while(oF.parent) {
        	tFolder.push({ id: oF.oxfolder.data.id, title: oF.oxfolder.data.title });
            oF = oF.parent;
        }   
        fn_callback(tFolder);     
    }
    else
        this.cache.load_path(nId,new Join(function (){
                                        var tFolder = new Array();
                                        for(var nNumParent=0;nNumParent<this.arg2.length;nNumParent++) {
                                        	tFolder.push({ id: this.arg2[nNumParent][0], title: this.arg2[nNumParent][1] });
                                        }
                                        fn_callback(tFolder);}),true);    
},
set_dashed: function (node){
		if(this.dashed != null)
		{
			this.dashed.text.className = removeClass(this.dashed.text.className,
																"treeHighlightDashed");    
            this.dashed.text.className += " border-color-content-default";
			this.dashed = null;
		}
		if(node)
		{
			this.dashed = node;
			this.dashed.text.className = removeClass(this.dashed.text.className,
                                                                "border-color-content-default");
			this.dashed.text.className += " treeHighlightDashed";
		}
},
destroy : function (){
	/*
	try{
	var aTemp = new Array();
	for(var nChild in this.tree.children)
	{
		aTemp.push(this.tree.children[nChild]);
	}
	for(var nindx=0;nindx<aTemp.length;nindx++)
	{
		this.tree.removeChild(aTemp[nindx]);
	}
	}catch(e){}*/
},
saveConfiguration : function ()
{	
	if(this.saveConfObj != null)	
	{	
		this.saveConfObj.setState(this.tree.getState());
	}
},
setSaveConfiguration : function (oReferObj)
{
	this.saveConfObj = oReferObj;
},	
fetchAndGetFolders : function (afolders,cb_fn){
	var aResult = new Object();
	function exec_cb()
	{
		cb_fn(aResult);
	}
	var join_all = new Join(exec_cb);		
	var fn_join_outer = join_all.add();
	for(var indx=0;indx<afolders.length;indx++)
	{		
		var fn_join_ = join_all.add();
		this.cache.get_folder(afolders[indx],function (arg){
															aResult[afolders[indx]] = arg;
															fn_join_();
														});														
	}
	fn_join_outer();	
},
getVisibleImapFolders : function (){
	var Self = this;
	var rootImap = this.nodes[Self.getDOMID("default")];
	if(rootImap == undefined)
		return new Array();
	var aFolderArr = new Array();
	function recTravSubTree(oChildr)
	{
		for (var indx in oChildr)
		{
			if(oChildr[indx] != undefined && Self.nodes[oChildr[indx].id] != undefined)
			{
				aFolderArr.push(Self.getFolderID(oChildr[indx].id));
				if(oChildr[indx].open)
					recTravSubTree(oChildr[indx].children);
			}
		}
	}
	recTravSubTree(rootImap.children);
	return aFolderArr;
},
load : function (){
	var Self = this;	
	if(this.cache.folders.root.children == null)
	{
		this.cache.load_folders(this.state,new Join(function (){
												Self.cache.events.trigger("initialized");
												Self.load_view(this.arg.root,Self.tree);
											}));	
	}
	else
	{
		this.cache.events.trigger("initialized");
		this.load_view(this.cache.folders.root,Self.tree,false,true);
	}
	if(this.register_update)
	{
		this.cache.events.register("Changed",function (arg){Self.update_sub_tree(arg);});
		this.cache.events.register("Deleted",function (arg){Self.delete_node(arg)});
	}
},
getFolderID : function (sId){
	var aSplited = sId.split('$');
	var sFolderID = '';
	for(var i=1;i<aSplited.length;i++)
	{
		if(i > 1)
		{
			sFolderID += '$'; 
		}
		sFolderID += aSplited[i];
	}
	return sFolderID.replace(/\$link$/,"");	
},
getDOMID : function (sId){
	return this.dom_node.id+'$'+sId;	
},
display_folder : function (nId,callback_opt){	
	var Self = this;
	var join_inge = new Join(function (){
										var folders = this.arg1;
										var aPath = this.arg2;				
										for(var indx=(aPath.length-1);indx>0;indx--)										
										{							
											var _node = Self.nodes[Self.getDOMID(aPath[indx][0])];
											if(_node && !_node.open)
											{
												_node.setOpen(true);
											}											
										}
										var sDOMFolderID = Self.getDOMID(nId);
										if(callback_opt)
											callback_opt(Self.nodes[sDOMFolderID]);		

									});
	this.cache.load_path(nId,join_inge,true);
},
set_selected : function (oNode){
	if(this.selected_node != null)
	{
		this.selected_node.text.className = removeStyleClass(this.selected_node.text.className,'background-color-PMG-selection-elements');
		this.selected_node.text.className = removeStyleClass(this.selected_node.text.className,'dndOver');
	}
	if(!oNode || !oNode.text)
	{
		this.selected_node = null;
		return;
	}
	oNode.text.className += " background-color-PMG-selection-elements";
	this.selected_node = oNode;
},
scroll_to : function (oNode)
{
	//scroll to the folder
	var nOffsetTop = oNode.div.offsetTop;
	if(this.dom_node.id == "configfoldertempl")
		$('config_folder_tree').scrollTop = nOffsetTop - $('config_folder_tree').offsetHeight/3;
	else
		this.dom_node.scrollTop = nOffsetTop - this.dom_node.offsetHeight/3;
},
load_view : function (oFolders,attPoint,bReplace,bRootOnly){
	var Self = this;
	//store all child node ids in an array
	var aOldChildrenIds = new Object();
	if(bReplace)
	{
		for(var nfolder in attPoint.children)
		{
			aOldChildrenIds[attPoint.children[nfolder].id] = true;
		}
	}
	for(var nFldr in oFolders.children)
	{			
		var OXFolder = oFolders.children[nFldr].oxfolder;
		if(oFolders.children[nFldr].oxfolder.data.module== "mail" && 
			!this.show_unsubscribed && !oFolders.children[nFldr].oxfolder.data.subscribed && !oFolders.children[nFldr].oxfolder.data.subscr_subflds)
		{
			continue;
		}
		var sNodeDOMID = this.getDOMID(OXFolder.data.id);
		if(OXFolder.data.module == 'infostore' &&  oFolders.oxfolder && (oFolders.oxfolder.data.id == 1 ||
										attPoint.link)) //children from link are links too.
		{
			sNodeDOMID += '$link';
			bLink = true;
		}
		else
		{
			bLink = false;				
		}		
		var bEnabled = true;
		if(OXFolder.data.module=='mail') {
			bEnabled = (getPermAsString(false, OXFolder.data.own_rights, 7).bit>=2)
		} else if(!(OXFolder.data.module == "infostore" && (OXFolder.data.id==9 || OXFolder.data.id==10 || OXFolder.data.id==15))) {
			if ((getPermAsString(true, OXFolder.data.own_rights, 0).bit<1
					|| getPermAsString(false, OXFolder.data.own_rights, 7).bit<1) &&  
															OXFolder.data.module != "mail" &&	
																		OXFolder.data.module != "system")
				bEnabled = false;		
			else
				bEnabled = true;						
		}				

		if (oFolders.children[nFldr].parent.oxfolder) {
			var sImageSRC = getIconImagePath(oFolders.children[nFldr].parent.oxfolder.data.id,OXFolder.data.id,
																				OXFolder.data.module,
																				bLink,bEnabled,
																				OXFolder.data.standard_folder);	
		} else {																						
			var sImageSRC = getIconImagePath(oFolders.children[nFldr].parent,OXFolder.data.id,
																				OXFolder.data.module,															
																				bLink,bEnabled,
																	OXFolder.data.standard_folder);
		}

		var sShowUnread = (parseInt(OXFolder.data.unread) > 0 && this.show_new_messages)?' (' + String(OXFolder.data.unread) + ')':'';

        var ids = configGetKey("folder");
        var replacedname = "";
        switch (OXFolder.data.id) {
            case ids.tasks:
                replacedname = "Tasks"; /*i18n*/
                break;
            case ids.calendar:
                replacedname = "Calendar"; /*i18n*/
                break;
            case ids.contacts:
                replacedname = "Contacts"; /*i18n*/
                break;
        }
		var oNode = new TreeNode(attPoint, (OXFolder.data.subfolders && 
		(oFolders.children[nFldr].oxfolder.data.subscr_subflds || oFolders.children[nFldr].oxfolder.data.subscr_subflds == null || 
																														this.show_unsubscribed)),
				 sImageSRC[0], sImageSRC[1],
                 replacedname ? addTranslated(replacedname)
                              : OXFolder.data.title,
                 sNodeDOMID, {}, sShowUnread);
		oNode.link = bLink;
		if (OXFolder.data.id == 1) {
			if (configGetKey("modules.folder.read_create_shared_folders")) {
				oNode.div.quickinfo="Private folders are under your control and by default no other user has access to your private folders. You may share your private folders to let others look at your private folders. In case you grant other users modification permission, such changes will be done under your name.";/*i18n*/
			} else {
				oNode.div.quickinfo="Private folders are under your control and no other user has access to your private folders."; /*i18n*/
			}
		} else if (OXFolder.data.id == 2) {
			if (configGetKey("modules.folder.public_folders")) {
				oNode.div.quickinfo="Public folders are available for all users or selected user groups as determined by the administrator. The access rights can be denied for individual users or user groups. If a user creates an object in a public folder, he acts on behalf of himself, and the object will be created under his name.";/*i18n*/
			} else {
				oNode.div.quickinfo="Public folders are readable for all users or selected user groups as determined by the administrator."; /*i18n*/
			}
		} else if (OXFolder.data.id == 3) {
			oNode.div.quickinfo="Shared folders are folders where another user has given you access. In case you are allowed to modify items in a shared folder, you make these changes on behalf of the folder's owner.";/*i18n*/
		} else if (OXFolder.data.id == 9) {
			oNode.div.quickinfo="Document sharing, storage & management - The centralisation of documents is the key to teamwork. Combined with version control an organisation can work from a united vision that is always up-to-date."; /*i18n*/
		} else if (OXFolder.data.id == 10) {
			oNode.div.quickinfo="The UserStore includes all shared personal InfoStore folders from other users. This means, if an user shared his InfoStore folder or the subfolder structure, the user can see the objects.";/*i18n*/
		} else if( OXFolder.data.id == 14) {
			oNode.div.quickinfo="Other InfoStore folders contain shared subfolders from parent folders that are not accessible to you.";/*i18n*/
		} else if (oNode.link) {
			oNode.div.quickinfo="The My InfoStore folder is a copy of the personal UserStore folder at the InfoStore section.";/*i18n*/
		}
		
		if (parseInt(OXFolder.data.unread) > 0 && this.show_new_messages) {
			oNode.text.className += ' font-style-lable';
		}
		
		if (!bEnabled) {
			oNode.text.className += ' font-style-low font-color-disabled';
		}
		
		if (!this.enabled_modules[OXFolder.data.module] || !(this.enable_check_fn(oFolders.children[nFldr]))) {
			oNode.text.className += " font-color-disabled";
			if(!OXFolder.data.subfolders) {
				oNode.icon.style.cursor = "default";
				oNode.text.style.cursor = "default";	
			}
		}
		
	 	if (this.dnd_enabled) {
			this.register_dnd(oNode);
	 	}
			
		if (this.inplace_enabled) {		
		 	oNode.inpl_edit = new inplaceEdit(oNode.text, 
			 	    OXFolder.data.title, 
					(function(id_cl) {
						return function (sStr,oObj){
                                renameFolder(id_cl,{title:sStr});
                            };
					})(OXFolder.data.id)
				);
		 	//oNode.input_width = "100%";
		}
		
		var oOldNode = this.nodes[sNodeDOMID];
		if (oOldNode != undefined && bReplace && oOldNode.div.parentNode != null && 
		    oOldNode.parent.id == attPoint.id) {
			aOldChildrenIds[sNodeDOMID] = false;
			this.nodes[sNodeDOMID] = oNode;			
			attPoint.replaceChild(oNode,oOldNode);
		} else {
		 	this.nodes[sNodeDOMID] = oNode;
			attPoint.addChild(oNode);
		}
		
		if (this.context_menu_enabled) {
			registerContext(oNode.table_Node, oNode.tree_id, null, oNode, 
			         function(oldHighlightedNode) {
			         	Self.set_dashed();
			         }, null, null, true);
		}
			
		if (this.check_read_right &&
           computePerm(OXFolder.data.own_rights,7) == 0 && !(OXFolder.data.subfolders)) {
			oNode.text.style.cursor = "default";
			oNode.icon.style.cursor = "default";								
		}
			
		//display user name on private folder
		if(OXFolder.data.id == 1) {
			var oObjNode = oNode.textnode;
			internalCache.getUsers([configGetKey("identifier")], 
                function(cbObj) {
				    oObjNode.data = cbObj[configGetKey("identifier")].display_name;
                }					
			);
		}
		
		//mark selected folder
		if(this.selected_node != null && this.selected_node.id == sNodeDOMID) {
			this.set_selected(oNode);
		}
		
		if(oFolders.children[nFldr].children != null && !bRootOnly) {
			this.load_view(oFolders.children[nFldr],oNode);
		}
	}
	
	var aNodesToRemove = new Array();
	for(var nOldN in aOldChildrenIds)
	{
		if(aOldChildrenIds[nOldN])
			aNodesToRemove.push(nOldN);
	}
	this.remove_nodes(attPoint,aNodesToRemove);		
	this.events.trigger("created");
},
remove_nodes : function (attPoint,aNodesToRemove){
	for(indx=0;indx<aNodesToRemove.length;indx++)
	{
		attPoint.removeChild(this.nodes[aNodesToRemove[indx]]);
		delete this.nodes[aNodesToRemove[indx]];
	}
},
get_folder_by_node_id : function (node_id)
{
	return this.cache.find_folder(this.getFolderID(node_id));
},
expand_node : function (oNode,fn_opt){
	var Self = this;
	var parentCache = this.get_folder_by_node_id(oNode.id);
	var childrennodes = false;
	for(var i in oNode.children) {
		childrennodes = true;
		break;
	}
	if (parentCache && parentCache.children == null) {
		this.cache.do_load_subfolders(this.getFolderID(oNode.id),parentCache ,{},function (arg)
																{								
																	Self.load_view(arg,oNode,false,true);
																	if(fn_opt)
																		fn_opt();
																}			
		);
	} else { 
		if(!childrennodes) {			
				Self.load_view(parentCache,oNode,false,true);
				if (fn_opt) fn_opt();
		} else if (fn_opt) fn_opt();
	}
	
},
update_sub_tree : function(ofolder){
	var Self = this;
	var oSubRoot = this.nodes[this.getDOMID(ofolder.oxfolder.data.id)];
	if(ofolder == undefined || oSubRoot == undefined)
		return;
	this.load_view(ofolder,oSubRoot,true);
	var sLinkId = this.getDOMID(ofolder.oxfolder.data.id) + '$link';
	
	var childrennodes = false;
	for(var i in this.nodes[sLinkId])
	{
		childrennodes = true;
		break;
	}
	if(this.nodes[sLinkId] != undefined && childrennodes)	
	{		
		this.load_view(ofolder,this.nodes[sLinkId],true);
	}
	this.tree.setState(this.state);
},
delete_node : function (ofolder){
},
register_dnd : function (currentNode) {
		function fn(sFolderId) {
			return function() {
				var dwi = sFolderId;
				return dwi;
			}
		}
		function fn2(sFolderId,node){
			return function (event,dragType,data,mouseposition,targetNode,dropNode)	{
				clearOpenNOdeOver(event,dragType,data,mouseposition,null,dropNode);
				var oMovedfolder = oMainFolderTree.cache.find_folder(data);				
				if(moveAllowed(oMovedfolder.parent,
									oMovedfolder,oMainFolderTree.cache.find_folder(oMainFolderTree.getFolderID(node.id)))) {
					var dwi2 = sFolderId;
					var targetNode = oMainFolderTree.cache.find_folder(dwi2);					
					var currNode = oMainFolderTree.cache.find_folder(data);
					var sourceNode = currNode.parent;
					//targetNode.div.className = removeClass(targetNode.div.className, "dndOver");
					folderServerMove(sourceNode,currNode,targetNode,false);
				}
			}
		}
//OUT
		function clearDNDImagesFolder(e) {
			if (document.getElementById("folder_node_dd")) {
				document.getElementById("folder_node_dd").parentNode.removeChild(document.getElementById("folder_node_dd"));				
			}
		}
		var TImeOutOpen;
		function clearOpenNOdeOver(e,dragType,draggedObjects,mouseposition,OutOpen,dropNode) {
			if (TImeOutOpen) {
				clearTimeout(TImeOutOpen);
			}
			if (dropNode) {
				dropNode.className = removeClass(dropNode.className, "dndOver");	
			} else if (e) {
				var mynode = e.currentTarget || e.srcElement || false;
				if (mynode) {
					mynode.className = removeClass(mynode.className, "dndOver");  
				}
			}
		}
//IN
		function fnOver(node,bNoHighlight) {
			return function (e,dragType,draggedObjects,mouseposition,targetNode) {
				foldertreeenable(e, dragType, draggedObjects, mouseposition, targetNode, node);
				var mynode = e.currentTarget || e.srcElement || false;
				if(!bNoHighlight && !mynode.className.match(/dndOver/)) {
					mynode.className += " dndOver";
				}
				var nnNode = node;
				function setOpenFN() {
					if (!nnNode.open) {
						nnNode.setOpen(true);
					}				
				}
				if (TImeOutOpen) {
					clearTimeout(TImeOutOpen);
				}
				return (TImeOutOpen = setTimeout(setOpenFN,1000));
			}
		}	
		function fnMailAddress(node) {
			return function (event,dragType,draggedObjects,mouseposition,targetNode,dropNode) {				
				createNewContactfromMail(draggedObjects,node.data.id);
				if(dropNode) {
                    dropNode.className = removeClass(dropNode.className, "dndOver");    
                }
			}
		}			
		var oSubFolder = oMainFolderTree.cache.fast_access[oMainFolderTree.getFolderID(currentNode.id)];
		var oRights = oSubFolder.oxfolder.data.own_rights;
		var tObj = new Object();
		
		if (!(getPermAsString(true, oRights, 0).bit < 4 && oSubFolder.oxfolder.data.module != 'mail')) {
			if (oSubFolder.oxfolder.data.type == 5 && oSubFolder.oxfolder.data.id == 1) {
				tObj["folder/1"] = fn2(oSubFolder.oxfolder.data.id,currentNode);
			} else if (oSubFolder.oxfolder.data.type == 5 && oSubFolder.oxfolder.data.id == 2) {
				tObj["folder/2"] = fn2(oSubFolder.oxfolder.data.id,currentNode);
			} else if (oSubFolder.oxfolder.data.type == 5 && oSubFolder.oxfolder.data.id == 10) {
				tObj["folder/2"] = fn2(oSubFolder.oxfolder.data.id,currentNode);				
			} else {
				tObj["folder/" + oSubFolder.oxfolder.data.type] = fn2(oSubFolder.oxfolder.data.id,currentNode);
			}
		}
                
		/*
		 * check if user has writeObject permissions and if it's not a 
		 * shared folder of module calendar, tasks or contacts. move operations
		 * into a shared folder of the named modules will be declined by the server
		 */
		var tmp_module = oSubFolder.oxfolder.data.module;
		if (computePerm(oRights,0) >= 2 && 
		    !(oSubFolder.oxfolder.data.type == 3 && tmp_module == "tasks")) {			
			/*
			 * user has writeObject permissions on this folder, register event
			 * which handles move operations
			 */
			tObj[oSubFolder.oxfolder.data.module] = (function(folderID) {
				return function(e,dragType,data,mouseposition,targetNode,dropNode) { 
					data.endMoving(folderID); 
					clearOpenNOdeOver(e,dragType,data,mouseposition,null,dropNode);
				}
			})(oSubFolder.oxfolder.data.id);
		} else {
			/*
			 * user doesn't have writeObject permissions or it's a shared folder 
			 * so register dummy event
			 */
			tObj[oSubFolder.oxfolder.data.module] = function (event,dragType,draggedObjects,mouseposition,targetNode,dropNode) {
                clearOpenNOdeOver(event,dragType,draggedObjects,mouseposition,null,dropNode);
            };
		}
			
		if (oSubFolder.oxfolder.data.module != "contacts") { 
			tObj["contacts"] = function (event,dragType,draggedObjects,mouseposition,targetNode,dropNode) {
				clearOpenNOdeOver(event,dragType,draggedObjects,mouseposition,null,dropNode);
			};
		}
		if (oSubFolder.oxfolder.data.module != "tasks") { 
			tObj["tasks"] = function (event,dragType,draggedObjects,mouseposition,targetNode,dropNode) {
				clearOpenNOdeOver(event,dragType,draggedObjects,mouseposition,null,dropNode);
			};
		}
		if (oSubFolder.oxfolder.data.module != "calendar") { 
			tObj["calendar"] = function (event,dragType,draggedObjects,mouseposition,targetNode,dropNode) {
				clearOpenNOdeOver(event,dragType,draggedObjects,mouseposition,null,dropNode);
			};
		}
		if (oSubFolder.oxfolder.data.module != "infostore") { 
			tObj["infostore"] = function (event,dragType,draggedObjects,mouseposition,targetNode,dropNode) {
				clearOpenNOdeOver(event,dragType,draggedObjects,mouseposition,null,dropNode);
			};
		}
		if (oSubFolder.oxfolder.data.module != "mail") { 
			tObj["mail"] = function (event,dragType,draggedObjects,mouseposition,targetNode,dropNode) {
				clearOpenNOdeOver(event,dragType,draggedObjects,mouseposition,null,dropNode);
			};
		}
		if (oSubFolder.oxfolder.data.module == "contacts") { 
			tObj["mailaddress"] = fnMailAddress(oSubFolder.oxfolder);
		} else { 
			tObj["mailaddress"] = function (event,dragType,draggedObjects,mouseposition,targetNode,dropNode) { 
				clearOpenNOdeOver(event,dragType,draggedObjects,mouseposition,null,dropNode);
			};
		}
		
		registerTarget(currentNode.text,tObj,null,null,fnOver(currentNode),fnOver(currentNode),clearOpenNOdeOver,true);
		registerTarget(currentNode.icon,tObj,null,null,fnOver(currentNode),fnOver(currentNode),clearOpenNOdeOver,true);
		//No highlighting for plus icons (expand folders)
		registerTarget(currentNode.plus,tObj,null,null,fnOver(currentNode,true),fnOver(currentNode,true),clearOpenNOdeOver,true);			

		if (!(getPermAsString(false, oRights, 28).bit != 1 && 
		   oSubFolder.oxfolder.data.module != 'mail' || oSubFolder.oxfolder.data.standard_folder)) {
			registerSource(currentNode.text, "folder/" + oSubFolder.oxfolder.data.type, fn(oSubFolder.oxfolder.data.id),null,null, foldertreedisable, defaultdisabledremove);
			registerSource(currentNode.icon, "folder/" + oSubFolder.oxfolder.data.type, fn(oSubFolder.oxfolder.data.id), null,null,foldertreedisable, defaultdisabledremove);											
		}
	},
	action_node: emptyFunction,
	getState: function() { 
		return this.tree.getState(); 
	},
	update: function (arg_1, arg_2, arg_3) { 
		this.cache.update(arg_1,arg_2,arg_3); 
	}
}

function setActiveFolder(sFolderId,fn,bScrollTo) {
	if(sFolderId) {		
		activefolder = sFolderId;
		function cb_load_path(node) {
			if(fn) fn(node);
			oMainFolderTree.set_selected(node);		
			if(bScrollTo) oMainFolderTree.scroll_to(node);		
		}
		oMainFolderTree.display_folder(sFolderId,cb_load_path);
	}
}

/**
 * check if a folder has no object read permissions
 * common for root folders and system folders
 */
function onlyParentFunc(OXFolder) {
	return computePerm(OXFolder.data.own_rights, 7) == 0;
}

function isDefaultMailFolder(sFolderId) {
	var oDefaultMailFldrs = configGetKey("mail.folder");
	for (var nfldr in oDefaultMailFldrs) {
		if (oDefaultMailFldrs[nfldr] == sFolderId) {
			return true;
		}
	}
	return false;
}

function renamingAllowed(OXFolder) {
	oRights = OXFolder.data.own_rights;
	if ((OXFolder.data.module != 'mail' && 
        (computePerm(oRights, 28) != 1 || OXFolder.data.standard_folder || OXFolder.data.module == 'system')) || 
        (OXFolder.data.module == 'mail' && 
        (!(computePerm(oRights, 28)==1) || isDefaultMailFolder(OXFolder.data.id)))) {
		return false;	
	}
	return true;
}

function deleteAllowed(OXFolder) {
	oRights = OXFolder.data.own_rights;	
	if ((OXFolder.data.module != 'mail' && 
	    (computePerm(oRights, 28) != 1 || OXFolder.data.standard_folder) || 
	    (OXFolder.data.module == 'mail' && 
	    (!(computePerm(oRights, 28)==1) || isDefaultMailFolder(OXFolder.data.id))))) {
		return false;
	}
	return true;	
}

function moveAllowed (parentFolder1, folder1, targetFolder1, bIgnoreEqualTarget) {
	var parentFolder=parentFolder1.oxfolder;
	var folder=folder1.oxfolder;
	var targetFolder=targetFolder1.oxfolder;
	// source == parent of folder to move
	// target = targetfolder
	// folder = folder itself
	if (parentFolder.data.id == targetFolder.data.id && !bIgnoreEqualTarget) return false;
	//Prevent moving into folder itself
	if (folder.data.id == targetFolder.data.id) return false;
	//Prevent moving shared folders 
	if (folder.data.type == 3 || targetFolder.data.type == 3) return false;
	//Prevent moving system folders
	if (folder.data.type == 5) return false;
	//Prevent moving private folders to other folders than private folders
	if (folder.data.type == 1 && (targetFolder.data.type != 1 && targetFolder.data.id != 1 )) return false; 
	//Prevent moving public folders to other folders than public folders
	if (folder.data.type == 2 && (targetFolder.data.type != 2 && targetFolder.data.id != 2)) return false;
	//Prevent moving folders to other not allowed modules
	if (folder.data.module != targetFolder.data.module) {
		if (folder.data.module == "mail" && !(targetFolder.data.module=="system") || folder.data.module == "infostore") 
            return false; 
		if ((folder.data.module == "calendar" || folder.data.module=="tasks" || folder.data.module=="contacts") &&
            !(targetFolder.data.module=="calendar" || targetFolder.data.module=="tasks" || targetFolder.data.module=="contacts" || targetFolder.data.module=="system"))
            return false; 
	}
	
	//Check targetfolder is subfolder of folder
	var aparent=targetFolder1.parent;
	while (aparent) {
		if (aparent.oxfolder && aparent.oxfolder.data.id == folder.data.id) return false;
		aparent = aparent.parent 
	}
	//Check rights Admin right source folder and create subfolders targetfolder
	if (computePerm(folder.data.own_rights, 28) != 1 || computePerm(targetFolder.data.own_rights, 0) < 4) 
        return false;
	
	return true;
}

function folderServerMove(sourceFolder, Folder, targetFolder, bOnlyServerMove, fn_cb_fm) {
		if (sourceFolder.oxfolder.data.id == targetFolder.oxfolder.data.id || 
            Folder.oxfolder.data.id == targetFolder.oxfolder.data.id) return;

		var oFolder = new Object();
		oFolder['folder_id'] = targetFolder.oxfolder.data.id ;
		function cb_move_folder(arg) {
			if(arg && arg.error) return;
			triggerEvent("OX_New_Info", 4, _("Folder was moved successfully"));				
			folderTreeFolderToMove = -1;
			if (Folder.oxfolder.data.module == "mail") {
				oMainFolderTree.update(null, true, function() {
					   oMainFolderTree.display_folder(targetFolder.oxfolder.data.id);
                    });
			} else {
				oMainFolderTree.update(null, false, function() {
					   oMainFolderTree.display_folder(Folder.oxfolder.data.id);
					});
			}
			if (fn_cb_fm) fn_cb_fm();
		}
		doAjaxUpdateRequest(Folder.oxfolder.data.id, 
                oFolder,Folder.oxfolder.timestamp, cb_move_folder, function(){});
}

function setAfterLabelByFolderId(sFolderId, str, bBold) {
	var oNode = oMainFolderTree.nodes[oMainFolderTree.getDOMID(sFolderId)];
	if(oNode == undefined) return;
	oNode.afterLabel.innerHTML = str;	
	if(bBold) {	
		oNode.text.className = removeClass(oNode.text.className,'font-style-lable');
		oNode.text.className += ' font-style-lable';
	} else {		
		oNode.text.className = removeClass(oNode.text.className,'font-style-lable');
	}
}

function setUnreadNumber(sFolderId, nUnreadNumber) {
	if (nUnreadNumber > 0)
		setAfterLabelByFolderId(sFolderId, '&nbsp;(' + String(nUnreadNumber) + ')', true);
	else		
		setAfterLabelByFolderId(sFolderId, '',false);	
}

function getFolderRightFolderAdmin(ofolder) {	
	return (ofolder.oxfolder.data.own_rights & Math.pow(2,28)) ? true : false;	
}

function removeStyleClass(sClasses,sClassToRemove) {
	var sNewClasses = '';	
	var aTMP = sClasses.split(' ');
	for(var nInd=0;nInd < aTMP.length;nInd++) {
		var sCl = aTMP[nInd];
		if(sCl != sClassToRemove)
			sNewClasses += ' ' + sCl;
	}
	return sNewClasses;
}

function getFolderRightCreateSubFolders(ofolder) {
	var oLocOwnRights = ofolder.oxfolder.data.own_rights;
	var bPermFolderBit = computePerm(oLocOwnRights, 0);	
	return (bPermFolderBit>=4)?true:false;	
}

function disableIcon(sIcon) {
	var sOldIcon = sIcon;	
	var aTMP = sOldIcon.split('/');
	var newIcon = '';
	for (nInd=0; nInd<aTMP.length; nInd++)	{
		if (nInd < aTMP.length-1) {
			newIcon += aTMP[nInd] + '/';
		} else {
			var aTMP2 = aTMP[nInd].split('.');
			newIcon += aTMP2[0] + '_dis.'+ aTMP2[1];
		}
	}	
	return newIcon;					
}

function getPermAsString(bFolder, nBits, nOffset) {
	bFolder = (nOffset==0);
	var alocalPerm = new Object();
	alocalPerm['string'] = _('none');
	alocalPerm['bit'] = 0;
	if (nOffset == 28) {
		alocalPerm['bit'] = (nBits & Math.pow(2,nOffset))?1:0;		
		alocalPerm['string'] = (alocalPerm['bit'])?_('Yes'):_('No');		
	} else if (nOffset == 29) {
		alocalPerm['bit'] = (nBits & Math.pow(2,nOffset))?1:0;		
		alocalPerm['string'] = (alocalPerm['bit'])?_('Yes'):_('No');				
	} else {
		for (nBit = 0;nBit <= 6;nBit++) {	
			if (nBit == 0 && nBits & Math.pow(2,(nBit + nOffset))) {
				alocalPerm['string'] = (bFolder)?_('see'):_('own');
				alocalPerm['bit'] = 1;
			} else if (nBit == 1 && nBits & Math.pow(2,(nBit + nOffset))) {
				alocalPerm['string'] = (bFolder)?_('create objects'):_('all');
				alocalPerm['bit'] = 2;
			} else if (nBit == 2 && nBits & Math.pow(2,(nBit + nOffset))) {
				alocalPerm['string'] = _('create subfolders');
				alocalPerm['bit'] = 4;
			} else if (nBit == 6 && nBits & Math.pow(2,(nBit + nOffset))) {
				alocalPerm['string'] = _('admin');
				alocalPerm['bit'] = 64;
			}
		}
	}
	return alocalPerm;
}

/**
 * Constructor of the subscribe E-Mail folder dialog
 */
function SubscribeFolder() { };
SubscribeFolder.changedFolders = new Array(); // temp storage for changed items
SubscribeFolder.listFolder = function (myFolder, cNode) {
    for (var i in myFolder.children) {
        var OXFolder = myFolder.children[i];
        // no mail folder, so ignore it ...
        if (OXFolder.oxfolder.data && OXFolder.oxfolder.data.module == "mail") {	        	
        	SubscribeFolder.addFolderToList(OXFolder, cNode);        	
        } 
    }                                               
    $("subscribe_folder_status").style.display="none";

};
SubscribeFolder.addFolderToList = function (OXFolder, contentNode) {
	var sNode = newnode("td",{ width: "18px" },0,
                       [
                           newnode("img",{ verticalAlign: "middle" },{ className: OXFolder.oxfolder.data.subfolders ? "tree-children" : "tree-nochildren", 
                                src: OXFolder.oxfolder.data.subfolders ? getFullImgSrc("img/plus.gif") : getFullImgSrc("img/noplus.gif")})
                       ]
                   );
    sNode.onclick = (function(folder, node) {
    	   return function() {
    	   	   if (!folder.oxfolder.data.subfolders) return;
    	   	   if (node.firstChild.src.match(/plus.gif/)) {
    	   	   	   $("subscribe_folder_status").style.display="block";
    	   	   	   $("subscribe_folder_status").style.top="19px";             
                   var tNode = newnode("div",{marginLeft: "18px", whiteSpace: "nowrap"},0);
	               node.parentNode.parentNode.parentNode.parentNode.appendChild(tNode);            
	               oMainFolderTree.cache.do_load_subfolders(folder.oxfolder.data.id, oMainFolderTree.cache.find_folder(folder.oxfolder.data.id), {}, 
	                  function(arg) {
	                     node.firstChild.src=getFullImgSrc("img/minus.gif");
	                     node.firstChild.className="tree-nochildren";
	                     SubscribeFolder.listFolder(arg, tNode);
	                  });
    	   	   } else {    	   	   	   
    	   	   	   node.parentNode.parentNode.parentNode.parentNode.removeChild(node.parentNode.parentNode.parentNode.nextSibling);
                   node.firstChild.src=getFullImgSrc("img/plus.gif");
                   node.firstChild.className="tree-children";   	   	   	
    	   	   }
    	   }
    	   
        })(OXFolder, sNode);
	var iNode = newnode("input",0,{ type: "checkbox",
	           id: "sub_folder_" + OXFolder.oxfolder.data.id, 
		       oxid: OXFolder.oxfolder.data.id,		       
		       className: "noborder"
	       });
	/*
	 * setting checked attribute only if folder is subscribed
	 */ 
	if (OXFolder.oxfolder.data.subscribed == true) {
		iNode.setAttribute("checked", "checked");
	}  
	iNode.onchange = (function(node) {
                        return function() {
                        	SubscribeFolder.addChangedItem( { id: node.oxid, data: { subscribed: node.checked ? 1 : 0 } });
                        }
                    })(iNode);
	contentNode.appendChild(
	   newnode("div", 0, { className: "tree-div" }, [
	       newnode("table",{borderCollapse:"collapse",borderWidth:"0",borderSpacing:"0",width:"100%"},0,
	           [   
	               newnode("tbody",{width:"100%"},0,
		               [
					       newnode("tr",0,0,
					           [
					               sNode,
					               newnode("td",{ width: "18px" },0,[
					                   newnode("img",{ verticalAlign: "middle" },{ src: getFullImgSrc("img/folder/folder_closed.gif") })]
					               ),
					               newnode("td",0,0,[ document.createTextNode(OXFolder.oxfolder.data.title) ]),
					               newnode("td",{ width: "30px", verticalAlign: "middle" },0,[ iNode ])
					           ])
					   ])
			     ])
	       ])
        );
    /*
     * Workaround to fix checkbox selection bug in IE
     * As long as the node hasn't been appended to the DOM tree IE6 will ignore
     * setting the checked attribute. A timeout takes care that the node will be
     * appended first.
     */
    if (OXFolder.oxfolder.data.subscribed == true) {
    	setTimeout(function() { 
    		iNode.setAttribute("checked", "checked");
    	}, 0);
    }
}
SubscribeFolder.addChangedItem = function(item) {
	for (var i = 0; i < SubscribeFolder.changedFolders.length; i++) {
		if (SubscribeFolder.changedFolders[i].id == item.id) {
		  SubscribeFolder.changedFolders.splice(i, 1);
		  break;
		}
	}
	item["module"] = "folder";
	item["action"] = "update";
	SubscribeFolder.changedFolders.push(item);
}
SubscribeFolder.saveChanges = function () {
	if (SubscribeFolder.changedFolders.length == 0) {
		SubscribeFolder.closeWindow();
		return;
	}
	$("subscribe_folder_status").style.display="block";
	$("subscribe_folder_status").style.top="19px"; 
	newFolderJson.put(AjaxRoot + "/multiple?session=" + session, SubscribeFolder.changedFolders, null, 
	   function(data) {	   	   
	   	   oMainFolderTree.update(null,true, function() {
	   	   	   SubscribeFolder.closeWindow();
	   	   })
	   });
}
SubscribeFolder.openWindow = function () {
	SubscribeFolder.changedFolders = new Array(); 
    showNode("disable_background_window");
    $("disable_background_window").style.display="block";
    removeChildNodes($("subscribe_folder_content"));
    $("subscribe_folder_status").style.display="block";
    oMainFolderTree.cache.get_folder("default", 
        	function(folder) {	
        	   SubscribeFolder.listFolder(folder, $("subscribe_folder_content"));        	   
        	});
        
    showNode("subscribe_folder");
    centerPopupWindow($("subscribe_folder"));
    $("subscribe_folder").style.display="block";
}
SubscribeFolder.closeWindow = function () {
	SubscribeFolder.changedFolders = new Array();
    hideNode("subscribe_folder");
    $("subscribe_folder").style.display="none"; 
    hideNode("disable_background_window");
    $("disable_background_window").style.display="none";
}
fileloaded();
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) 2004-2007 Open-Xchange, Inc.
 * Mail: info@open-xchange.com 
 * 
 * @author Jan Finsel <jan.finsel@open-xchange.com>
 * 
 */

var globalalert = new AlertPopup();
var globalcreate= new CreatePopup();
function AlertPopup() {	
	this.header;
	this.buttons;
	this.text;
	this.div;
	this.closecb;
	this.cbok;
	this.cbcancel;
	this.cbyes;
	this.cbno;
	this.cbseries;
	this.cbappointment;
	this.cbnocomment;
	this.visibleButtons=[];
}
AlertPopup.SERIESBUTTON=50;
AlertPopup.OK=1;
AlertPopup.CANCEL=2;
AlertPopup.YES=4;
AlertPopup.NO=8;
AlertPopup.APPOINTMENT=16;
AlertPopup.SERIES=32;
AlertPopup.NOCOMMENT=64;
AlertPopup.EDIT=128;
AlertPopup.CONTINUE=256;
AlertPopup.VERSION=67;
AlertPopup.OKCANCEL=3;
AlertPopup.YESNO=12;
AlertPopup.CONTINUECANCEL=258;

AlertPopup.open = function () {
	globalalert.visibleButtons = [];
    /*
     * remove keydown event just in case!
     */
    removeDOMEvent(body, "keydown", AlertPopup.key_down);
	popupenabled = true;
	showNode("disable_background_window");
	$("disable_background_window").style.display="block";
	showNode("disable_confirm_window");
	$("disable_confirm_window").style.display="block";
	/*
	 * enable / disable required buttons
	 */
	if ((globalalert.buttons %  2) >= 1) { 
        $("alert_window_ok").style.display="block";
        setFocus($("alert_window_ok"));
        globalalert.visibleButtons.push({id: "alert_window_ok", evt: AlertPopup.commitok});
    } else { 
        $("alert_window_ok").style.display="none"; 
    }
	if ((globalalert.buttons %  4) >= 2) { 
		$("alert_window_cancel").style.display="block"; 
		globalalert.visibleButtons.push({id: "alert_window_cancel", evt: AlertPopup.commitcancel});
	} else { 
		$("alert_window_cancel").style.display="none"; 
	}
	if ((globalalert.buttons %  8) >= 4) { 
		$("alert_window_yes").style.display="block"; 
		setFocus($("alert_window_yes"));
		globalalert.visibleButtons.push({id: "alert_window_yes", evt: AlertPopup.commityes});
	} else { 
		$("alert_window_yes").style.display="none"; 
	}
	if ((globalalert.buttons % 16) >= 8) { 
		$("alert_window_no").style.display="block";
		globalalert.visibleButtons.push({id: "alert_window_no", evt: AlertPopup.commitno});
	} else { 
		$("alert_window_no").style.display="none"; 
	}	
	if ((globalalert.buttons % 64) >= 32) { 
		$("alert_window_series").style.display="block";
		globalalert.visibleButtons.push({id: "alert_window_series", evt: AlertPopup.commitseries});
	} else { 
		$("alert_window_series").style.display="none"; 
	}
	if ((globalalert.buttons % 32) >= 16) { 
		$("alert_window_appointment").style.display="block"; 
		globalalert.visibleButtons.push({id: "alert_window_appointment", evt: AlertPopup.commitappointment});
	} else { 
		$("alert_window_appointment").style.display="none"; 
	}
	if ((globalalert.buttons % 128) >= 64) { 
		$("alert_window_nocomment").style.display="block"; 
		globalalert.visibleButtons.push({id: "alert_window_nocomment", evt: AlertPopup.commitnocomment});
	} else { 
		$("alert_window_nocomment").style.display="none"; 
	}
	if ((globalalert.buttons % 256) >= 128) { 
		$("alert_window_edit").style.display="block"; 
		globalalert.visibleButtons.push({id: "alert_window_edit", evt: AlertPopup.commitedit});
	} else { 
		$("alert_window_edit").style.display="none"; 
	}
    if ((globalalert.buttons % 512) >= 256) { 
        $("alert_window_continue").style.display="block"; 
        globalalert.visibleButtons.push({id: "alert_window_continue", evt: AlertPopup.commitcontinue});
    } else { 
        $("alert_window_continue").style.display="none"; 
    }
	showNode("alert_window");
	$("alert_window-h-c").firstChild.data = globalalert.header;
	removeChildNodes($("alert_window_text"));
	if (globalalert.div) {		
		$("alert_window_text").appendChild(globalalert.div);
	} else {
		var nText = globalalert.text.split("\n");
		for (var i in nText) {
			$("alert_window_text").appendChild(document.createTextNode(nText[i]));
			$("alert_window_text").appendChild(newnode("br"));
		}
	}
	centerPopupWindow($("alert_window"));	
	$("alert_window").style.display="block";
	addDOMEvent(body, "keydown", AlertPopup.key_down);
}
AlertPopup.close = function () {
	popupenabled = false;
	if (globalalert.closecb) { 
		globalalert.closecb(); 
	}
	removeDOMEvent(body, "keydown", AlertPopup.key_down);
	hideNode("alert_window");
	$("alert_window").style.display="none";	
	hideNode("disable_confirm_window");
	$("disable_confirm_window").style.display="none";
	var childs=	$("disable_background_window").childNodes;
	var closeall=true;
	for(var i=0;i<childs.length;i++) {
		if(childs[i].tagName && childs[i].tagName == "DIV") {
			if(childs[i].style.display && childs[i].style.display != "none" ) {
			
				closeall=false;
			}
		}
	}
	globalalert.visibleButtons = {};
	if(closeall) {
		hideNode("disable_background_window");
		$("disable_background_window").style.display="none";
	}
}
AlertPopup.commityes = function () {
	if(globalalert.cbyes) { globalalert.cbyes(); }
	AlertPopup.close();
}
AlertPopup.commitno = function () {
	if(globalalert.cbno) { globalalert.cbno(); }
	AlertPopup.close();
}
AlertPopup.commitcancel = function () {
	if(globalalert.cbcancel) { globalalert.cbcancel(); }
	AlertPopup.close();
}
AlertPopup.commitok = function () {
	if(globalalert.cbok) { globalalert.cbok(); }
	AlertPopup.close();
}
AlertPopup.commitseries = function () {
	if(globalalert.cbseries) { globalalert.cbseries(); }
	AlertPopup.close();
}
AlertPopup.commitnocomment = function () {
	if(globalalert.cbnocomment) { globalalert.cbnocomment(); }
	AlertPopup.close();
}
AlertPopup.commitappointment = function () {
	if(globalalert.cbappointment) { globalalert.cbappointment(); }
	AlertPopup.close();
}
AlertPopup.commitedit = function () {
	if(globalalert.cbedit) { globalalert.cbedit(); }
	AlertPopup.close();
}
AlertPopup.key_down = function(e) {
    switch (e.keyCode || e.which) {
    	case 27: // esc
    	    AlertPopup.close();
    	    break;
        case 9: // tab
            if (!globalalert.visibleButtons.length) break;
            if (focusedElement != null) {               
                for (var i=0; i<globalalert.visibleButtons.length; i++) {
                    if (globalalert.visibleButtons[i].id == focusedElement.id) {
                        if (i+1 < globalalert.visibleButtons.length) {
                            setFocus($(globalalert.visibleButtons[i+1].id));
                        } else {
                            setFocus($(globalalert.visibleButtons[0].id));
                        }
                        break;
                    }
                }
            } else {
                setFocus($(globalalert.visibleButtons[0].id));
            }
            break;
        case 32: // space
        case 13: // enter 
            if (!globalalert.visibleButtons.length) break;
            if (focusedElement != null) {               
                for (var i=0; i<globalalert.visibleButtons.length; i++) {
                    if (globalalert.visibleButtons[i].id == focusedElement.id) {
                        globalalert.visibleButtons[i].evt();
                        break;
                    }
                }
            }
            break;         
        default:
            return;        
    }
    stopEvent(e);
    return false;
}
AlertPopup.prototype = 
{
	setOK : function (cb) {
		this.cbok=cb;
	},
	setCancel : function (cb) {
		this.cbcancel=cb;
	},
	setYes : function (cb) {
		this.cbyes=cb;
	},
	setNo : function (cb) {
		this.cbno=cb;
	},
	setSeries: function (cb) {
		this.cbseries=cb;
	},
	setAppointment: function (cb) {
		this.cbappointment=cb;
	},
	setClose: function (cb) {
		this.closecb=cb;
	},
	setNoComment : function(cb) {
		this.cbnocomment=cb;
	},
	setEdit: function(cb) {
		this.cbedit=cb;
	}
}


function CreatePopup() {	
	this.header;
	this.buttons;
	this.text;
	this.inputtext;
	this.inputvalue;
	this.closecb;
	this.cbok;
	this.cbcancel;
	this.cbadvanced;
	this.cbnocomment;
}
CreatePopup.OK=1;
CreatePopup.CANCEL=2;
CreatePopup.NOCOMMENT=4;
CreatePopup.VERSION=7;
CreatePopup.ADVANCED=64;
CreatePopup.ALL=67;

CreatePopup.open = function () {
	showNode("disable_background_window");
	$("disable_background_window").style.display="block";
	showNode("disable_confirm_window");
	$("disable_confirm_window").style.display="block"
	if ((globalcreate.buttons %  4) >= 2) { $("create_window_cancel").style.display="block"; } else { $("create_window_cancel").style.display="none"; }
	if ((globalcreate.buttons %  8) >= 4) { $("create_window_nocomment").style.display="block"; } else { $("create_window_nocomment").style.display="none"; }
	if ((globalcreate.buttons %  2) >= 1) { $("create_window_ok").style.display="block"; } else { $("create_window_ok").style.display="none"; }
	
	if ((globalcreate.buttons % 128) >= 64) { $("create_window_advanced").style.display="block"; } else { $("create_window_advanced").style.display="none"; }
	
	$("create_window-h-c").firstChild.data = globalcreate.header;
	$("create_window_text").firstChild.data = globalcreate.text;
	if(globalcreate.inputtype == "input") {
		$("create_window_table_textarea").style.display="none";
		$("create_window_table_text_input").style.display="block";
		if (globalcreate.inputtext) {
			$("create_window_text_input").style.display="";
		    $("create_window_text_input").firstChild.data = globalcreate.inputtext;
		} else {
			$("create_window_text_input").style.display="none";
		}
		$("create_window_text_field").value=globalcreate.inputvalue;
		setTimeout(function() { 
			if ($("create_window_text_field") && $("create_window_text_field").focus()) $("create_window_text_field").focus(); 
		},500);
	} else if(globalcreate.inputtype == "textarea") {
		$("create_window_table_textarea").style.display="block";
		$("create_window_table_text_input").style.display="none";
		$("create_window_textarea").firstChild.data = globalcreate.inputtext;	
		$("create_window_textarea_field").value=globalcreate.inputvalue;
		setTimeout(function() { 
			if ($("create_window_textarea_field") && $("create_window_textarea_field").focus()) $("create_window_textarea_field").focus(); 
		},500);
	}
	showNode("create_window");
	centerPopupWindow($("create_window"));
	$("create_window").style.display="block";
}
CreatePopup.close = function () {
	if(globalcreate.closecb) { globalcreate.closecb(); }
	hideNode("create_window");
	$("create_window").style.display="none";	
	hideNode("disable_confirm_window");
	$("disable_confirm_window").style.display="none";
	var childs=	$("disable_background_window").childNodes;
	var closeall=true;
	for(var i=0;i<childs.length;i++) {
		if(childs[i].tagName && childs[i].tagName == "DIV") {
			if(childs[i].style.display && childs[i].style.display != "none" ) {
			
				closeall=false;
			}
		}
	}
	if(closeall) {
		hideNode("disable_background_window");
		$("disable_background_window").style.display="none";
	}
}
CreatePopup.commitcancel = function () {
	if(globalcreate.cbcancel) { globalcreate.cbcancel(); }
	CreatePopup.close();
}
CreatePopup.commitok = function () {
	if(globalcreate.cbok) { globalcreate.cbok(); }
	CreatePopup.close();
}
CreatePopup.commitnocomment = function () {
	if(globalcreate.cbnocomment) { globalcreate.cbnocomment(); }
	CreatePopup.close();
}
CreatePopup.commitadvanced = function () {
	if(globalcreate.cbadvanced) { globalcreate.cbadvanced(); }
	CreatePopup.close();
}

CreatePopup.prototype = 
{
	setOK : function (cb) {
		this.cbok=cb;
	},
	setCancel : function (cb) {
		this.cbcancel=cb;
	},
	setAdvanced: function (cb) {
		this.cbadvanced=cb;
	},
	setClose: function (cb) {
		this.closecb=cb;
	},
	setNoComment : function (cb) {
		this.cbnocomment=cb;
	}
}
function newAlert(headertitle,alerttext,okcb,div) {
	globalalert.header=headertitle;
	globalalert.buttons=CreatePopup.OK;
	globalalert.text=alerttext;
	globalalert.div=div;
	globalalert.closecb=null;
	globalalert.cbok=okcb;
	globalalert.cbcancel=null;
	globalalert.cbnocomment=null;
	globalalert.cbyes=null;
	globalalert.cbno=null;
	AlertPopup.open();
}
function newConfirm(headertitle,contenttext,buttons,cbok,cbcancel,cbyes,cbno,cbseries,cbappointment,cbclose,cbnocomment,cbedit,div) {
	globalalert.header=headertitle;
	globalalert.buttons=buttons;
	globalalert.text=contenttext;
	globalalert.div=div;
	globalalert.closecb=cbclose;
	globalalert.cbok=cbok;
	globalalert.cbappointment=cbappointment;
	globalalert.cbseries=cbseries;
	globalalert.cbcancel=cbcancel;
	globalalert.cbedit=cbedit;
	globalalert.cbyes=cbyes;
	globalalert.cbno=cbno;
	globalalert.cbnocomment=cbnocomment;
	AlertPopup.open();	
}
function newInput(headertitle,contenttext,inputtext,inputvalue,buttons,cbok,cbcancel,cbadvanced,cbclose,
																					inputType,cbnocomment) {
	globalcreate.header=headertitle;
	globalcreate.text=contenttext;
	globalcreate.inputtext=inputtext;
	if(inputvalue)
		globalcreate.inputvalue=inputvalue;
	else
		globalcreate.inputvalue="";
	globalcreate.buttons=buttons;
	globalcreate.closecb=cbclose;
	globalcreate.cbok=cbok;
	globalcreate.cbcancel=cbcancel;
	globalcreate.cbnocomment=cbnocomment;
	globalcreate.cbadvanced=cbadvanced;
	if(inputType)
		globalcreate.inputtype=inputType;
	else
		globalcreate.inputvalue="input";
	CreatePopup.open();	
}
function newAppointmentObject(appointmentobject,myok,mycancel,myadvanced,myclose) {
	if(appointmentobject.start_date && appointmentobject.end_date) {
		var mystring=formatDate(appointmentobject.start_date,"datetime")+ " - "+formatDate(appointmentobject.end_date,"datetime"); 
		newInput(_("New Appointment"),mystring, pgettext("description", "Title"),"",CreatePopup.ALL,myok,mycancel,myadvanced,myclose);
	}
}
register("Loaded",function() {
	addOnClose($('alert_window'),function () { 
		AlertPopup.close();
	});
	addOnClose($('create_window'),function () { 
		CreatePopup.close(); 
	});
});
fileloaded();
